@@ -6,13 +6,11 @@ set -o pipefail
 
 # default to Debian testing
 DISTRO="${DISTRO:-debian}"
-RELEASE="${RELEASE:-bookworm}"
-SALSA_URL="${SALSA_URL:-https://salsa.debian.org/systemd-team/systemd.git}"
+RELEASE="${RELEASE:-bullseye}"
 BRANCH="${BRANCH:-upstream-ci}"
 ARCH="${ARCH:-amd64}"
 CONTAINER="${RELEASE}-${ARCH}"
-CACHE_DIR=/var/tmp
-TMPDIR=/var/tmp
+CACHE_DIR="${SEMAPHORE_CACHE_DIR:-/tmp}"
 AUTOPKGTEST_DIR="${CACHE_DIR}/autopkgtest"
 # semaphore cannot expose these, but useful for interactive/local runs
 ARTIFACTS_DIR=/tmp/artifacts
@@ -30,14 +28,14 @@ create_container() {
 
     # enable source repositories so that apt-get build-dep works
     sudo lxc-attach -n "$CONTAINER" -- sh -ex <<EOF
-sed 's/^deb/deb-src/' /etc/apt/sources.list >> /etc/apt/sources.list.d/sources.list
+sed 's/^deb/deb-src/' /etc/apt/sources.list >>/etc/apt/sources.list.d/sources.list
 # We might attach the console too soon
 while ! systemctl --quiet --wait is-system-running; do sleep 1; done
 # Manpages database trigger takes a lot of time and is not useful in a CI
 echo 'man-db man-db/auto-update boolean false' | debconf-set-selections
 # Speed up dpkg, image is thrown away after the test
 mkdir -p /etc/dpkg/dpkg.cfg.d/
-echo 'force-unsafe-io' > /etc/dpkg/dpkg.cfg.d/unsafe_io
+echo 'force-unsafe-io' >/etc/dpkg/dpkg.cfg.d/unsafe_io
 # For some reason, it is necessary to run this manually or the interface won't be configured
 # Note that we avoid networkd, as some of the tests will break it later on
 dhclient
@@ -71,12 +69,12 @@ for phase in "${PHASES[@]}"; do
         ;;
         RUN)
             # add current debian/ packaging
-            git fetch --depth=1 "$SALSA_URL" "$BRANCH"
+            git fetch --depth=1 https://salsa.debian.org/systemd-team/systemd.git "$BRANCH"
             git checkout FETCH_HEAD debian
 
             # craft changelog
             UPSTREAM_VER="$(git describe | sed 's/^v//;s/-/./g')"
-            cat << EOF > debian/changelog.new
+            cat <<EOF >debian/changelog.new
 systemd (${UPSTREAM_VER}.0) UNRELEASED; urgency=low
 
   * Automatic build for upstream test
@@ -92,9 +90,9 @@ EOF
             # disable autopkgtests which are not for upstream
             sed -i '/# NOUPSTREAM/ q' debian/tests/control
             # enable more unit tests
-            sed -i '/^CONFFLAGS =/ s/=/= --werror -Dsplit-usr=true /' debian/rules
+            sed -i '/^CONFFLAGS =/ s/=/= --werror -Dtests=unsafe -Dsplit-usr=true -Dslow-tests=true -Dfuzz-tests=true -Dman=true /' debian/rules
             # no orig tarball
-            echo '1.0' > debian/source/format
+            echo '1.0' >debian/source/format
 
             # build source package
             dpkg-buildpackage -S -I -I"$(basename "$CACHE_DIR")" -d -us -uc -nc
@@ -102,11 +100,8 @@ EOF
             # now build the package and run the tests
             rm -rf "$ARTIFACTS_DIR"
             # autopkgtest exits with 2 for "some tests skipped", accept that
-            sudo TMPDIR=/var/tmp "$AUTOPKGTEST_DIR/runner/autopkgtest" --env DEB_BUILD_OPTIONS="noudeb nostrip nodoc optimize=-lto" \
-                                                       --env DPKG_DEB_COMPRESSOR_TYPE="none" \
-                                                       --env DEB_BUILD_PROFILES="noudeb nodoc" \
-                                                       --env TEST_UPSTREAM=1 \
-                                                       ../systemd_*.dsc \
+            sudo "$AUTOPKGTEST_DIR/runner/autopkgtest" --env DEB_BUILD_OPTIONS=noudeb \
+                                                       --env TEST_UPSTREAM=1 ../systemd_*.dsc \
                                                        -o "$ARTIFACTS_DIR" \
                                                        -- lxc -s "$CONTAINER" \
                 || [ $? -eq 2 ]
