@@ -19,7 +19,6 @@
 #include "string-util.h"
 #include "strv.h"
 #include "terminal-util.h"
-#include "util.h"
 
 #define PCI_CLASS_GRAPHICS_CARD 0x30000
 
@@ -56,10 +55,6 @@ static int has_multiple_graphics_cards(void) {
         if (r < 0)
                 return r;
 
-        r = sd_device_enumerator_allow_uninitialized(e);
-        if (r < 0)
-                return r;
-
         r = sd_device_enumerator_add_match_subsystem(e, "pci", /* match = */ true);
         if (r < 0)
                 return r;
