@@ -10,14 +10,13 @@
 #include <sys/mman.h>
 #include <sys/stat.h>
 
-#include "def.h"
+#include "constants.h"
 #include "dirent-util.h"
 #include "env-util.h"
 #include "fd-util.h"
 #include "fileio.h"
 #include "hashmap.h"
 #include "locale-util.h"
-#include "missing_syscall.h"
 #include "path-util.h"
 #include "set.h"
 #include "string-table.h"
@@ -96,7 +95,8 @@ static int add_locales_from_archive(Set *locales) {
         const struct locarhead *h;
         const struct namehashent *e;
         const void *p = MAP_FAILED;
-        _cleanup_close_ int fd = -1;
+        _cleanup_close_ int fd = -EBADF;
+        size_t sz = 0;
         struct stat st;
         int r;
 
@@ -153,9 +153,9 @@ static int add_locales_from_archive(Set *locales) {
 
         r = 0;
 
-finish:
+ finish:
         if (p != MAP_FAILED)
-                munmap((void*) p, st.st_size);
+                munmap((void*) p, sz);
 
         return r;
 }
@@ -280,9 +280,15 @@ int locale_is_installed(const char *name) {
         return true;
 }
 
+void init_gettext(void) {
+        setlocale(LC_ALL, "");
+        textdomain(GETTEXT_PACKAGE);
+}
+
 bool is_locale_utf8(void) {
-        const char *set;
         static int cached_answer = -1;
+        const char *set;
+        int r;
 
         /* Note that we default to 'true' here, since today UTF8 is
          * pretty much supported everywhere. */
@@ -290,11 +296,12 @@ bool is_locale_utf8(void) {
         if (cached_answer >= 0)
                 goto out;
 
-        /* This function may be called from libsystemd, and setlocale() is not thread safe. Assuming yes. */
-        if (gettid() != raw_getpid()) {
-                cached_answer = true;
+        r = getenv_bool_secure("SYSTEMD_UTF8");
+        if (r >= 0) {
+                cached_answer = r;
                 goto out;
-        }
+        } else if (r != -ENXIO)
+                log_debug_errno(r, "Failed to parse $SYSTEMD_UTF8, ignoring: %m");
 
         if (!setlocale(LC_ALL, "")) {
                 cached_answer = true;
