@@ -44,9 +44,8 @@ int path_is_extension_tree(const char *path, const char *extension, bool relax_e
         /* Does the path exist at all? If not, generate an error immediately. This is useful so that a missing root dir
          * always results in -ENOENT, and we can properly distinguish the case where the whole root doesn't exist from
          * the case where just the os-release file is missing. */
-        r = laccess(path, F_OK);
-        if (r < 0)
-                return r;
+        if (laccess(path, F_OK) < 0)
+                return -errno;
 
         /* We use /usr/lib/extension-release.d/extension-release[.NAME] as flag for something being a system extension,
          * and {/etc|/usr/lib}/os-release as a flag for something being an OS (when not an extension). */
@@ -172,7 +171,7 @@ int open_extension_release(const char *root, const char *extension, bool relax_e
 
                                 /* We already found what we were looking for, but there's another candidate?
                                  * We treat this as an error, as we want to enforce that there are no ambiguities
-                                 * in case we are in the fallback path.*/
+                                 * in case we are in the fallback path. */
                                 if (r == 0) {
                                         r = -ENOTUNIQ;
                                         break;
@@ -228,7 +227,7 @@ int open_extension_release(const char *root, const char *extension, bool relax_e
 
 int fopen_extension_release(const char *root, const char *extension, bool relax_extension_release_check, char **ret_path, FILE **ret_file) {
         _cleanup_free_ char *p = NULL;
-        _cleanup_close_ int fd = -1;
+        _cleanup_close_ int fd = -EBADF;
         FILE *f;
         int r;
 
@@ -337,7 +336,7 @@ int load_extension_release_pairs(const char *root, const char *extension, bool r
         return load_env_file_pairs(f, p, ret);
 }
 
-int os_release_support_ended(const char *support_end, bool quiet) {
+int os_release_support_ended(const char *support_end, bool quiet, usec_t *ret_eol) {
         _cleanup_free_ char *_support_end_alloc = NULL;
         int r;
 
@@ -347,27 +346,38 @@ int os_release_support_ended(const char *support_end, bool quiet) {
 
                 r = parse_os_release(NULL,
                                      "SUPPORT_END", &_support_end_alloc);
-                if (r < 0)
-                        return log_full_errno((r == -ENOENT || quiet) ? LOG_DEBUG : LOG_WARNING, r,
+                if (r < 0 && r != -ENOENT)
+                        return log_full_errno(quiet ? LOG_DEBUG : LOG_WARNING, r,
                                               "Failed to read os-release file, ignoring: %m");
-                if (!_support_end_alloc)
-                        return false;  /* no end date defined */
 
                 support_end = _support_end_alloc;
         }
 
-        struct tm tm = {};
+        if (isempty(support_end)) /* An empty string is a explicit way to say "no EOL exists" */
+                return false;  /* no end date defined */
 
+        struct tm tm = {};
         const char *k = strptime(support_end, "%Y-%m-%d", &tm);
         if (!k || *k)
                 return log_full_errno(quiet ? LOG_DEBUG : LOG_WARNING, SYNTHETIC_ERRNO(EINVAL),
                                       "Failed to parse SUPPORT_END= in os-release file, ignoring: %m");
 
-        time_t eol = mktime(&tm);
+        time_t eol = timegm(&tm);
         if (eol == (time_t) -1)
                 return log_full_errno(quiet ? LOG_DEBUG : LOG_WARNING, SYNTHETIC_ERRNO(EINVAL),
                                       "Failed to convert SUPPORT_END= in os-release file, ignoring: %m");
 
-        usec_t ts = now(CLOCK_REALTIME);
-        return DIV_ROUND_UP(ts, USEC_PER_SEC) > (usec_t) eol;
+        if (ret_eol)
+                *ret_eol = eol * USEC_PER_SEC;
+
+        return DIV_ROUND_UP(now(CLOCK_REALTIME), USEC_PER_SEC) > (usec_t) eol;
+}
+
+const char *os_release_pretty_name(const char *pretty_name, const char *name) {
+        /* Distills a "pretty" name to show from os-release data. First argument is supposed to be the
+         * PRETTY_NAME= field, the second one the NAME= field. This function is trivial, of course, and
+         * exists mostly to ensure we use the same logic wherever possible. */
+
+        return empty_to_null(pretty_name) ?:
+                empty_to_null(name) ?: "Linux";
 }
