@@ -880,8 +880,7 @@ char **env_generator_binary_paths(bool is_system) {
 }
 
 int find_portable_profile(const char *name, const char *unit, char **ret_path) {
-        const char *p, *dot;
-        int r;
+        const char *dot;
 
         assert(name);
         assert(ret_path);
@@ -895,13 +894,13 @@ int find_portable_profile(const char *name, const char *unit, char **ret_path) {
                 if (!joined)
                         return -ENOMEM;
 
-                r = laccess(joined, F_OK);
-                if (r >= 0) {
+                if (laccess(joined, F_OK) >= 0) {
                         *ret_path = TAKE_PTR(joined);
                         return 0;
                 }
-                if (r != -ENOENT)
-                        return r;
+
+                if (errno != ENOENT)
+                        return -errno;
         }
 
         return -ENOENT;
