@@ -96,7 +96,6 @@ static const char* const unit_load_state_table[_UNIT_LOAD_STATE_MAX] = {
 
 DEFINE_STRING_TABLE_LOOKUP(unit_load_state, UnitLoadState);
 
-/* Keep in sync with man/unit-states.xml */
 static const char* const unit_active_state_table[_UNIT_ACTIVE_STATE_MAX] = {
         [UNIT_ACTIVE]       = "active",
         [UNIT_RELOADING]    = "reloading",
@@ -189,6 +188,8 @@ static const char* const service_state_table[_SERVICE_STATE_MAX] = {
         [SERVICE_RUNNING]        = "running",
         [SERVICE_EXITED]         = "exited",
         [SERVICE_RELOAD]         = "reload",
+        [SERVICE_RELOAD_SIGNAL]  = "reload-signal",
+        [SERVICE_RELOAD_NOTIFY]  = "reload-notify",
         [SERVICE_STOP]           = "stop",
         [SERVICE_STOP_WATCHDOG]  = "stop-watchdog",
         [SERVICE_STOP_SIGTERM]   = "stop-sigterm",
