@@ -5,6 +5,7 @@
 
 #include "alloc-util.h"
 #include "bootspec.h"
+#include "build.h"
 #include "devnum-util.h"
 #include "efi-api.h"
 #include "efi-loader.h"
@@ -19,7 +20,6 @@
 #include "pretty-print.h"
 #include "sync-util.h"
 #include "terminal-util.h"
-#include "util.h"
 #include "verbs.h"
 #include "virt.h"
 
@@ -360,7 +360,7 @@ static int verb_status(int argc, char *argv[], void *userdata) {
                   left, done);
 
         STRV_FOREACH(p, arg_path) {
-                _cleanup_close_ int fd = -1;
+                _cleanup_close_ int fd = -EBADF;
 
                 fd = open(*p, O_DIRECTORY|O_CLOEXEC|O_RDONLY);
                 if (fd < 0) {
@@ -439,7 +439,7 @@ static int verb_set(int argc, char *argv[], void *userdata) {
         }
 
         STRV_FOREACH(p, arg_path) {
-                _cleanup_close_ int fd = -1;
+                _cleanup_close_ int fd = -EBADF;
 
                 fd = open(*p, O_DIRECTORY|O_CLOEXEC|O_RDONLY);
                 if (fd < 0)
@@ -476,14 +476,13 @@ static int verb_set(int argc, char *argv[], void *userdata) {
                 /* First, fsync() the directory these files are located in */
                 r = fsync_parent_at(fd, skip_slash(target));
                 if (r < 0)
-                        log_debug_errno(r, "Failed to synchronize image directory, ignoring: %m");
+                        log_debug_errno(errno, "Failed to synchronize image directory, ignoring: %m");
 
                 /* Secondly, syncfs() the whole file system these files are located in */
                 if (syncfs(fd) < 0)
                         log_debug_errno(errno, "Failed to synchronize $BOOT partition, ignoring: %m");
 
                 log_info("Marked boot as '%s'. (Boot attempt counter is at %" PRIu64".)", argv[0], done);
-                return 0;
         }
 
         log_error_errno(SYNTHETIC_ERRNO(EBUSY), "Can't find boot counter source file for '%s': %m", target);
