@@ -1,13 +1,16 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 #pragma once
 
+#include <efi.h>
+#include <uchar.h>
+
 struct devicetree_state {
         EFI_PHYSICAL_ADDRESS addr;
-        UINTN pages;
+        size_t pages;
         void *orig;
 };
 
-EFI_STATUS devicetree_install(struct devicetree_state *state, EFI_FILE *root_dir, CHAR16 *name);
+EFI_STATUS devicetree_install(struct devicetree_state *state, EFI_FILE *root_dir, char16_t *name);
 EFI_STATUS devicetree_install_from_memory(
-                struct devicetree_state *state, const VOID *dtb_buffer, UINTN dtb_length);
+                struct devicetree_state *state, const void *dtb_buffer, size_t dtb_length);
 void devicetree_cleanup(struct devicetree_state *state);
