@@ -10,6 +10,7 @@
 #include "sd-bus.h"
 
 #include "alloc-util.h"
+#include "build.h"
 #include "bus-error.h"
 #include "bus-util.h"
 #include "cgroup-show.h"
@@ -55,10 +56,11 @@ typedef struct Group {
         uint64_t io_input_bps, io_output_bps;
 } Group;
 
+/* Counted objects, enum order matters */
 typedef enum PidsCount {
-        COUNT_USERSPACE_PROCESSES,
+        COUNT_USERSPACE_PROCESSES,      /* least */
         COUNT_ALL_PROCESSES,
-        COUNT_PIDS,
+        COUNT_PIDS,                     /* most, requires pids controller */
 } PidsCount;
 
 static unsigned arg_depth = 3;
@@ -308,9 +310,9 @@ static int process(
 
                         if (all_unified) {
                                 while (!isempty(l)) {
-                                        if (sscanf(l, "rbytes=%" SCNu64, &k) == 1)
+                                        if (sscanf(l, "rbytes=%" SCNu64, &k))
                                                 rd += k;
-                                        else if (sscanf(l, "wbytes=%" SCNu64, &k) == 1)
+                                        else if (sscanf(l, "wbytes=%" SCNu64, &k))
                                                 wr += k;
 
                                         l += strcspn(l, WHITESPACE);
@@ -931,6 +933,7 @@ static int run(int argc, char *argv[]) {
         if (r < 0)
                 return log_error_errno(r, "Failed to determine supported controllers: %m");
 
+        /* honor user selection unless pids controller is unavailable */
         possible_count = (mask & CGROUP_MASK_PIDS) ? COUNT_PIDS : COUNT_ALL_PROCESSES;
         arg_count = MIN(possible_count, arg_count);
 
