@@ -55,7 +55,7 @@ static bool bpf_can_link_lsm_program(struct bpf_program *prog) {
 
 static int prepare_restrict_fs_bpf(struct restrict_fs_bpf **ret_obj) {
         _cleanup_(restrict_fs_bpf_freep) struct restrict_fs_bpf *obj = NULL;
-        _cleanup_close_ int inner_map_fd = -1;
+        _cleanup_close_ int inner_map_fd = -EBADF;
         int r;
 
         assert(ret_obj);
@@ -263,14 +263,11 @@ int lsm_bpf_cleanup(const Unit *u) {
         if (!u->manager->restrict_fs)
                 return 0;
 
-        if (u->cgroup_id == 0)
-                return 0;
-
         int fd = sym_bpf_map__fd(u->manager->restrict_fs->maps.cgroup_hash);
         if (fd < 0)
                 return log_unit_error_errno(u, errno, "bpf-lsm: Failed to get BPF map fd: %m");
 
-        if (sym_bpf_map_delete_elem(fd, &u->cgroup_id) != 0 && errno != ENOENT)
+        if (sym_bpf_map_delete_elem(fd, &u->cgroup_id) != 0)
                 return log_unit_debug_errno(u, errno, "bpf-lsm: Failed to delete cgroup entry from LSM BPF map: %m");
 
         return 0;
@@ -329,7 +326,6 @@ int lsm_bpf_parse_filesystem(
 
         if (name[0] == '@') {
                 const FilesystemSet *set;
-                const char *i;
 
                 set = filesystem_set_find(name);
                 if (!set) {
