@@ -9,6 +9,7 @@
 #include "format-util.h"
 #include "fs-util.h"
 #include "hexdecoct.h"
+#include "initrd-util.h"
 #include "import-creds.h"
 #include "io-util.h"
 #include "mkdir-label.h"
@@ -146,7 +147,7 @@ static int finalize_credentials_dir(const char *dir, const char *envvar) {
 
 static int import_credentials_boot(void) {
         _cleanup_(import_credentials_context_free) ImportCredentialContext context = {
-                .target_dir_fd = -1,
+                .target_dir_fd = -EBADF,
         };
         int r;
 
@@ -165,7 +166,7 @@ static int import_credentials_boot(void) {
                        "/.extra/global_credentials/") { /* boot partition wide */
 
                 _cleanup_free_ DirectoryEntries *de = NULL;
-                _cleanup_close_ int source_dir_fd = -1;
+                _cleanup_close_ int source_dir_fd = -EBADF;
 
                 source_dir_fd = open(p, O_RDONLY|O_DIRECTORY|O_CLOEXEC|O_NOFOLLOW);
                 if (source_dir_fd < 0) {
@@ -186,7 +187,7 @@ static int import_credentials_boot(void) {
 
                 for (size_t i = 0; i < de->n_entries; i++) {
                         const struct dirent *d = de->entries[i];
-                        _cleanup_close_ int cfd = -1, nfd = -1;
+                        _cleanup_close_ int cfd = -EBADF, nfd = -EBADF;
                         _cleanup_free_ char *n = NULL;
                         const char *e;
                         struct stat st;
@@ -294,7 +295,7 @@ static int acquire_credential_directory(ImportCredentialContext *c) {
 static int proc_cmdline_callback(const char *key, const char *value, void *data) {
         ImportCredentialContext *c = ASSERT_PTR(data);
         _cleanup_free_ char *n = NULL;
-        _cleanup_close_ int nfd = -1;
+        _cleanup_close_ int nfd = -EBADF;
         const char *colon;
         size_t l;
         int r;
@@ -365,7 +366,7 @@ static int import_credentials_proc_cmdline(ImportCredentialContext *c) {
 
 static int import_credentials_qemu(ImportCredentialContext *c) {
         _cleanup_free_ DirectoryEntries *de = NULL;
-        _cleanup_close_ int source_dir_fd = -1;
+        _cleanup_close_ int source_dir_fd = -EBADF;
         int r;
 
         assert(c);
@@ -392,7 +393,7 @@ static int import_credentials_qemu(ImportCredentialContext *c) {
 
         for (size_t i = 0; i < de->n_entries; i++) {
                 const struct dirent *d = de->entries[i];
-                _cleanup_close_ int vfd = -1, rfd = -1, nfd = -1;
+                _cleanup_close_ int vfd = -EBADF, rfd = -EBADF, nfd = -EBADF;
                 _cleanup_free_ char *szs = NULL;
                 uint64_t sz;
 
@@ -471,7 +472,7 @@ static int parse_smbios_strings(ImportCredentialContext *c, const char *data, si
         for (p = data, left = size; left > 0; p += skip, left -= skip) {
                 _cleanup_free_ void *buf = NULL;
                 _cleanup_free_ char *cn = NULL;
-                _cleanup_close_ int nfd = -1;
+                _cleanup_close_ int nfd = -EBADF;
                 const char *nul, *n, *eq;
                 const void *cdata;
                 size_t buflen, cdata_len;
@@ -512,7 +513,7 @@ static int parse_smbios_strings(ImportCredentialContext *c, const char *data, si
                         return log_oom();
 
                 if (!credential_name_valid(cn)) {
-                        log_warning("SMBIOS credential name '%s' is not valid, ignoring.", cn);
+                        log_warning("SMBIOS credential name '%s' is not valid, ignoring: %m", cn);
                         continue;
                 }
 
@@ -585,11 +586,9 @@ static int import_credentials_smbios(ImportCredentialContext *c) {
                         return log_oom();
 
                 r = read_virtual_file(p, sizeof(dmi_field_header) + CREDENTIALS_TOTAL_SIZE_MAX, (char**) &data, &size);
-                if (r == -ENOENT) /* Once we reach ENOENT there are no more DMI Type 11 fields around. */
-                        break;
                 if (r < 0) {
                         /* Once we reach ENOENT there are no more DMI Type 11 fields around. */
-                        log_warning_errno(r, "Failed to open '%s', ignoring: %m", p);
+                        log_full_errno(r == -ENOENT ? LOG_DEBUG : LOG_WARNING, r, "Failed to open '%s', ignoring: %m", p);
                         break;
                 }
 
@@ -614,7 +613,7 @@ static int import_credentials_smbios(ImportCredentialContext *c) {
 
 static int import_credentials_trusted(void) {
         _cleanup_(import_credentials_context_free) ImportCredentialContext c = {
-                .target_dir_fd = -1,
+                .target_dir_fd = -EBADF,
         };
         int q, w, r;
 
@@ -714,5 +713,18 @@ int import_credentials(void) {
                         r = q;
         }
 
+        if (r >= 0) {
+                _cleanup_free_ char *address = NULL;
+
+                r = read_credential("vmm.notify_socket", (void **)&address, /* ret_size= */ NULL);
+                if (r < 0 && !IN_SET(r, -ENOENT, -ENXIO))
+                        log_warning_errno(r, "Failed to read 'vmm.notify_socket' credential, ignoring: %m");
+                else if (r >= 0 && !isempty(address)) {
+                        r = setenv("NOTIFY_SOCKET", address, /* replace= */ 1);
+                        if (r < 0)
+                                log_warning_errno(errno, "Failed to set $NOTIFY_SOCKET environment variable, ignoring: %m");
+                }
+        }
+
         return r;
 }
