@@ -5,6 +5,7 @@
 #include "alloc-util.h"
 #include "bus-util.h"
 #include "capability-util.h"
+#include "efi-api.h"
 #include "fileio.h"
 #include "kmod-setup.h"
 #include "macro.h"
@@ -30,7 +31,7 @@ static void systemd_kmod_log(
         REENABLE_WARNING;
 }
 
-static int match_modalias_recurse_dir_cb(
+static int has_virtio_rng_recurse_dir_cb(
                 RecurseDirEvent event,
                 const char *path,
                 int dir_fd,
@@ -40,7 +41,6 @@ static int match_modalias_recurse_dir_cb(
                 void *userdata) {
 
         _cleanup_free_ char *alias = NULL;
-        char **modaliases = ASSERT_PTR(userdata);
         int r;
 
         if (event != RECURSE_DIR_ENTRY)
@@ -58,13 +58,13 @@ static int match_modalias_recurse_dir_cb(
                 return RECURSE_DIR_LEAVE_DIRECTORY;
         }
 
-        if (startswith_strv(alias, modaliases))
+        if (STARTSWITH_SET(alias, "pci:v00001AF4d00001005", "pci:v00001AF4d00001044"))
                 return 1;
 
         return RECURSE_DIR_LEAVE_DIRECTORY;
 }
 
-static bool has_virtio_feature(const char *name, char **modaliases) {
+static bool has_virtio_rng(void) {
         int r;
 
         /* Directory traversal might be slow, hence let's do a cheap check first if it's even worth it */
@@ -75,28 +75,16 @@ static bool has_virtio_feature(const char *name, char **modaliases) {
                         AT_FDCWD,
                         "/sys/devices/pci0000:00",
                         /* statx_mask= */ 0,
-                        /* n_depth_max= */ 3,
+                        /* n_depth_max= */ 2,
                         RECURSE_DIR_ENSURE_TYPE,
-                        match_modalias_recurse_dir_cb,
-                        modaliases);
+                        has_virtio_rng_recurse_dir_cb,
+                        NULL);
         if (r < 0)
-                log_debug_errno(r, "Failed to determine whether host has %s device, ignoring: %m", name);
+                log_debug_errno(r, "Failed to determine whether host has virtio-rng device, ignoring: %m");
 
         return r > 0;
 }
 
-static bool has_virtio_rng(void) {
-        return has_virtio_feature("virtio-rng", STRV_MAKE("pci:v00001AF4d00001005", "pci:v00001AF4d00001044"));
-}
-
-static bool has_virtiofs(void) {
-        return has_virtio_feature("virtiofs", STRV_MAKE("virtio:d0000001Av"));
-}
-
-static bool has_virtio_pci(void) {
-        return has_virtio_feature("virtio-pci", STRV_MAKE("pci:v00001AF4d"));
-}
-
 static bool in_qemu(void) {
         return IN_SET(detect_vm(), VIRTUALIZATION_KVM, VIRTUALIZATION_QEMU);
 }
@@ -114,41 +102,37 @@ int kmod_setup(void) {
         } kmod_table[] = {
                 /* This one we need to load explicitly, since auto-loading on use doesn't work
                  * before udev created the ghost device nodes, and we need it earlier than that. */
-                { "autofs4",   "/sys/class/misc/autofs",    true,   false,   NULL      },
+                { "autofs4",     "/sys/class/misc/autofs",    true,  false, NULL           },
 
                 /* This one we need to load explicitly, since auto-loading of IPv6 is not done when
                  * we try to configure ::1 on the loopback device. */
-                { "ipv6",      "/sys/module/ipv6",          false,  true,    NULL      },
+                { "ipv6",        "/sys/module/ipv6",          false, true,  NULL           },
 
                 /* This should never be a module */
-                { "unix",      "/proc/net/unix",            true,   true,    NULL      },
+                { "unix",        "/proc/net/unix",            true,  true,  NULL           },
 
 #if HAVE_LIBIPTC
                 /* netfilter is needed by networkd, nspawn among others, and cannot be autoloaded */
-                { "ip_tables", "/proc/net/ip_tables_names", false,  false,   NULL      },
+                { "ip_tables",   "/proc/net/ip_tables_names", false, false, NULL           },
 #endif
                 /* virtio_rng would be loaded by udev later, but real entropy might be needed very early */
-                { "virtio_rng", NULL,                       false,  false,   has_virtio_rng },
-
-                /* We can't wait for specific virtiofs tags to show up as device nodes so we have to load the
-                 * virtiofs and virtio_pci modules early to make sure the virtiofs tags are found when
-                 * sysroot.mount is started.
-                 *
-                 * TODO: Remove these again once https://gitlab.com/virtio-fs/virtiofsd/-/issues/128 is
-                 * resolved and the kernel fix is widely available. */
-                { "virtiofs",                   "/sys/module/virtiofs",      false, false, has_virtiofs       },
-                { "virtio_pci",                 "/sys/module/virtio_pci",    false, false, has_virtio_pci     },
+                { "virtio_rng",  NULL,                        false, false, has_virtio_rng },
 
                 /* qemu_fw_cfg would be loaded by udev later, but we want to import credentials from it super early */
-                { "qemu_fw_cfg", "/sys/firmware/qemu_fw_cfg", false, false,  in_qemu   },
+                { "qemu_fw_cfg", "/sys/firmware/qemu_fw_cfg", false, false, in_qemu        },
 
                 /* dmi-sysfs is needed to import credentials from it super early */
-                { "dmi-sysfs", "/sys/firmware/dmi/entries", false, false,  NULL   },
+                { "dmi-sysfs",   "/sys/firmware/dmi/entries", false, false, NULL           },
+
+#if HAVE_TPM2
+                /* Make sure the tpm subsystem is available which ConditionSecurity=tpm2 depends on. */
+                { "tpm",         "/sys/class/tpmrm",          false, false, efi_has_tpm2   },
+#endif
         };
         _cleanup_(kmod_unrefp) struct kmod_ctx *ctx = NULL;
         unsigned i;
 
-        if (have_effective_cap(CAP_SYS_MODULE) == 0)
+        if (have_effective_cap(CAP_SYS_MODULE) <= 0)
                 return 0;
 
         for (i = 0; i < ELEMENTSOF(kmod_table); i++) {
