@@ -81,7 +81,7 @@ int path_spec_watch(PathSpec *s, sd_event_io_handler_t handler) {
                         tmp = *cut;
                         *cut = '\0';
 
-                        flags = IN_MOVE_SELF | IN_DELETE_SELF | IN_CREATE | IN_MOVED_TO;
+                        flags = IN_MOVE_SELF | IN_DELETE_SELF | IN_ATTRIB | IN_CREATE | IN_MOVED_TO;
                 } else {
                         cut = NULL;
                         flags = flags_table[s->type];
@@ -266,7 +266,7 @@ static void path_spec_dump(PathSpec *s, FILE *f, const char *prefix) {
 
 void path_spec_done(PathSpec *s) {
         assert(s);
-        assert(s->inotify_fd == -1);
+        assert(s->inotify_fd == -EBADF);
 
         free(s->path);
 }
@@ -589,7 +589,7 @@ static void path_enter_waiting(Path *p, bool initial, bool from_trigger_notify)
         }
 
         if (path_check_good(p, initial, from_trigger_notify, &trigger_path)) {
-                log_unit_debug(UNIT(p), "Got triggered by '%s'.", trigger_path);
+                log_unit_debug(UNIT(p), "Got triggered.");
                 path_enter_running(p, trigger_path);
                 return;
         }
@@ -603,7 +603,7 @@ static void path_enter_waiting(Path *p, bool initial, bool from_trigger_notify)
          * recheck */
 
         if (path_check_good(p, false, from_trigger_notify, &trigger_path)) {
-                log_unit_debug(UNIT(p), "Got triggered by '%s'.", trigger_path);
+                log_unit_debug(UNIT(p), "Got triggered.");
                 path_enter_running(p, trigger_path);
                 return;
         }
