@@ -399,7 +399,7 @@ static void timer_enter_waiting(Timer *t, bool time_change) {
                          * to that. If we don't, just start from
                          * the activation time. */
 
-                        if (dual_timestamp_is_set(&t->last_trigger))
+                        if (t->last_trigger.realtime > 0)
                                 b = t->last_trigger.realtime;
                         else if (dual_timestamp_is_set(&UNIT(t)->inactive_exit_timestamp))
                                 b = UNIT(t)->inactive_exit_timestamp.realtime;
@@ -862,7 +862,7 @@ static int timer_clean(Unit *u, ExecCleanMask mask) {
         if (t->state != TIMER_DEAD)
                 return -EBUSY;
 
-        if (!IN_SET(mask, EXEC_CLEAN_STATE))
+        if (mask != EXEC_CLEAN_STATE)
                 return -EUNATCH;
 
         r = timer_setup_persistent(t);
