@@ -21,6 +21,7 @@
 #include "compress.h"
 #include "conf-parser.h"
 #include "copy.h"
+#include "coredump-util.h"
 #include "coredump-vacuum.h"
 #include "dirent-util.h"
 #include "elf-util.h"
@@ -79,7 +80,7 @@
  * size. See DATA_SIZE_MAX in journal-importer.h. */
 assert_cc(JOURNAL_SIZE_MAX <= DATA_SIZE_MAX);
 
-typedef enum {
+enum {
         /* We use these as array indexes for our process metadata cache.
          *
          * The first indices of the cache stores the same metadata as the ones passed by
@@ -92,12 +93,7 @@ typedef enum {
         META_ARGV_SIGNAL,       /* %s: number of signal causing dump */
         META_ARGV_TIMESTAMP,    /* %t: time of dump, expressed as seconds since the Epoch (we expand this to µs granularity) */
         META_ARGV_RLIMIT,       /* %c: core file size soft resource limit */
-        _META_ARGV_REQUIRED,
-        /* The fields below were added to kernel/core_pattern at later points, so they might be missing. */
-        META_ARGV_HOSTNAME = _META_ARGV_REQUIRED,  /* %h: hostname */
-        META_ARGV_DUMPABLE,     /* %d: as set by the kernel */
-        /* If new fields are added, they should be added here, to maintain compatibility
-         * with callers which don't know about the new fields. */
+        META_ARGV_HOSTNAME,     /* %h: hostname */
         _META_ARGV_MAX,
 
         /* The following indexes are cached for a couple of special fields we use (and
@@ -106,15 +102,16 @@ typedef enum {
          * environment. */
 
         META_COMM = _META_ARGV_MAX,
+        _META_MANDATORY_MAX,
 
         /* The rest are similar to the previous ones except that we won't fail if one of
-         * them is missing in a message sent over the socket. */
+         * them is missing. */
 
-        META_EXE,
+        META_EXE = _META_MANDATORY_MAX,
         META_UNIT,
         META_PROC_AUXV,
         _META_MAX
-} meta_argv_t;
+};
 
 static const char * const meta_field_names[_META_MAX] = {
         [META_ARGV_PID]       = "COREDUMP_PID=",
@@ -124,7 +121,6 @@ static const char * const meta_field_names[_META_MAX] = {
         [META_ARGV_TIMESTAMP] = "COREDUMP_TIMESTAMP=",
         [META_ARGV_RLIMIT]    = "COREDUMP_RLIMIT=",
         [META_ARGV_HOSTNAME]  = "COREDUMP_HOSTNAME=",
-        [META_ARGV_DUMPABLE]  = "COREDUMP_DUMPABLE=",
         [META_COMM]           = "COREDUMP_COMM=",
         [META_EXE]            = "COREDUMP_EXE=",
         [META_UNIT]           = "COREDUMP_UNIT=",
@@ -135,7 +131,6 @@ typedef struct Context {
         const char *meta[_META_MAX];
         size_t meta_size[_META_MAX];
         pid_t pid;
-        unsigned dumpable;
         bool is_pid1;
         bool is_journald;
 } Context;
@@ -451,16 +446,14 @@ static int grant_user_access(int core_fd, const Context *context) {
         if (r < 0)
                 return r;
 
-        /* We allow access if dumpable on the command line was exactly 1, we got all the data,
-         * at_secure is not set, and the uid/gid match euid/egid. */
+        /* We allow access if we got all the data and at_secure is not set and
+         * the uid/gid matches euid/egid. */
         bool ret =
-                context->dumpable == 1 &&
                 at_secure == 0 &&
                 uid != UID_INVALID && euid != UID_INVALID && uid == euid &&
                 gid != GID_INVALID && egid != GID_INVALID && gid == egid;
-        log_debug("Will %s access (dumpable=%u uid="UID_FMT " euid="UID_FMT " gid="GID_FMT " egid="GID_FMT " at_secure=%s)",
+        log_debug("Will %s access (uid="UID_FMT " euid="UID_FMT " gid="GID_FMT " egid="GID_FMT " at_secure=%s)",
                   ret ? "permit" : "restrict",
-                  context->dumpable,
                   uid, euid, gid, egid, yes_no(at_secure));
         return ret;
 }
@@ -477,7 +470,7 @@ static int save_external_coredump(
 
         _cleanup_(unlink_and_freep) char *tmp = NULL;
         _cleanup_free_ char *fn = NULL;
-        _cleanup_close_ int fd = -1;
+        _cleanup_close_ int fd = -EBADF;
         uint64_t rlimit, process_limit, max_size;
         bool truncated, storage_on_tmpfs;
         struct stat st;
@@ -568,21 +561,17 @@ static int save_external_coredump(
                                                   bus_error_message(&error, r));
                 }
 
-                /* First, ensure we are not going to go over the cgroup limit */
                 max_size = MIN(cgroup_limit, max_size);
-                /* tmpfs might get full quickly, so check the available space too. But don't worry about
-                 * errors here, failing to access the storage location will be better logged when writing to
-                 * it. */
-                if (statvfs("/var/lib/systemd/coredump/", &sv) >= 0)
-                        max_size = MIN((uint64_t)sv.f_frsize * (uint64_t)sv.f_bfree, max_size);
-                /* Impose a lower minimum, otherwise we will miss the basic headers. */
-                max_size = MAX(PROCESS_SIZE_MIN, max_size);
-                /* Ensure we can always switch to compressing on the fly in case we are running out of space
-                 * by keeping half of the space/memory available, plus 1KB metadata overhead from the
-                 * compression algorithm. */
-                max_size = LESS_BY(max_size, 1024U) / 2;
+                max_size = LESS_BY(max_size, 1024U) / 2; /* Account for 1KB metadata overhead for compressing */
+                max_size = MAX(PROCESS_SIZE_MIN, max_size); /* Impose a lower minimum */
 
-                log_debug("Limiting core file size to %" PRIu64 " bytes due to cgroup and/or filesystem limits.", max_size);
+                /* tmpfs might get full quickly, so check the available space too.
+                 * But don't worry about errors here, failing to access the storage
+                 * location will be better logged when writing to it. */
+                if (fstatvfs(fd, &sv) >= 0)
+                        max_size = MIN((uint64_t)sv.f_frsize * (uint64_t)sv.f_bfree, max_size);
+
+                log_debug("Limiting core file size to %" PRIu64 " bytes due to cgroup memory limits.", max_size);
         }
 
         r = copy_bytes(input_fd, fd, max_size, 0);
@@ -597,7 +586,7 @@ static int save_external_coredump(
         if (arg_compress) {
                 _cleanup_(unlink_and_freep) char *tmp_compressed = NULL;
                 _cleanup_free_ char *fn_compressed = NULL;
-                _cleanup_close_ int fd_compressed = -1;
+                _cleanup_close_ int fd_compressed = -EBADF;
                 uint64_t uncompressed_size = 0;
 
                 if (lseek(fd, 0, SEEK_SET) == (off_t) -1)
@@ -687,15 +676,14 @@ static int allocate_journal_field(int fd, size_t size, char **ret, size_t *ret_s
                 return log_warning_errno(errno, "Failed to seek: %m");
 
         field = malloc(9 + size);
-        if (!field)
-                return log_warning_errno(SYNTHETIC_ERRNO(ENOMEM),
-                                         "Failed to allocate memory for coredump, coredump will not be stored.");
+        if (!field) {
+                log_warning("Failed to allocate memory for coredump, coredump will not be stored.");
+                return -ENOMEM;
+        }
 
         memcpy(field, "COREDUMP=", 9);
 
-        /* NB: simple read() would fail for overly large coredumps, since read() on Linux can only deal with
-         * 0x7ffff000 bytes max. Hence call things in a loop. */
-        n = loop_read(fd, field + 9, size, /* do_poll= */ false);
+        n = read(fd, field + 9, size);
         if (n < 0)
                 return log_error_errno((int) n, "Failed to read core data: %m");
         if ((size_t) n < size)
@@ -724,7 +712,7 @@ static int allocate_journal_field(int fd, size_t size, char **ret, size_t *ret_s
  */
 static int compose_open_fds(pid_t pid, char **open_fds) {
         _cleanup_closedir_ DIR *proc_fd_dir = NULL;
-        _cleanup_close_ int proc_fdinfo_fd = -1;
+        _cleanup_close_ int proc_fdinfo_fd = -EBADF;
         _cleanup_free_ char *buffer = NULL;
         _cleanup_fclose_ FILE *stream = NULL;
         const char *fddelim = "", *path;
@@ -750,7 +738,7 @@ static int compose_open_fds(pid_t pid, char **open_fds) {
         FOREACH_DIRENT(de, proc_fd_dir, return -errno) {
                 _cleanup_fclose_ FILE *fdinfo = NULL;
                 _cleanup_free_ char *fdname = NULL;
-                _cleanup_close_ int fd = -1;
+                _cleanup_close_ int fd = -EBADF;
 
                 r = readlinkat_malloc(dirfd(proc_fd_dir), de->d_name, &fdname);
                 if (r < 0)
@@ -796,7 +784,7 @@ static int compose_open_fds(pid_t pid, char **open_fds) {
 static int get_process_ns(pid_t pid, const char *namespace, ino_t *ns) {
         const char *p;
         struct stat stbuf;
-        _cleanup_close_ int proc_ns_dir_fd = -1;
+        _cleanup_close_ int proc_ns_dir_fd = -EBADF;
 
         p = procfs_file_alloca(pid, "ns");
 
@@ -911,12 +899,13 @@ static int submit_coredump(
                 int input_fd) {
 
         _cleanup_(json_variant_unrefp) JsonVariant *json_metadata = NULL;
-        _cleanup_close_ int coredump_fd = -1, coredump_node_fd = -1;
+        _cleanup_close_ int coredump_fd = -EBADF, coredump_node_fd = -EBADF;
         _cleanup_free_ char *filename = NULL, *coredump_data = NULL;
         _cleanup_free_ char *stacktrace = NULL;
+        char *core_message;
         const char *module_name;
         uint64_t coredump_size = UINT64_MAX, coredump_compressed_size = UINT64_MAX;
-        bool truncated = false, written = false;
+        bool truncated = false;
         JsonVariant *module_json;
         int r;
 
@@ -928,69 +917,60 @@ static int submit_coredump(
         (void) coredump_vacuum(-1, arg_keep_free, arg_max_use);
 
         /* Always stream the coredump to disk, if that's possible */
-        written = save_external_coredump(
-                        context, input_fd,
-                        &filename, &coredump_node_fd, &coredump_fd,
-                        &coredump_size, &coredump_compressed_size, &truncated) >= 0;
-        if (written) {
-                /* If we could write it to disk we can now process it. */
-                /* If we don't want to keep the coredump on disk, remove it now, as later on we
-                 * will lack the privileges for it. However, we keep the fd to it, so that we can
-                 * still process it and log it. */
-                r = maybe_remove_external_coredump(filename, coredump_node_fd >= 0 ? coredump_compressed_size : coredump_size);
-                if (r < 0)
-                        return r;
-                if (r == 0)
-                        (void) iovw_put_string_field(iovw, "COREDUMP_FILENAME=", filename);
-                else if (arg_storage == COREDUMP_STORAGE_EXTERNAL)
-                        log_info("The core will not be stored: size %"PRIu64" is greater than %"PRIu64" (the configured maximum)",
-                                 coredump_node_fd >= 0 ? coredump_compressed_size : coredump_size, arg_external_size_max);
+        r = save_external_coredump(context, input_fd,
+                                   &filename, &coredump_node_fd, &coredump_fd,
+                                   &coredump_size, &coredump_compressed_size, &truncated);
+        if (r < 0)
+                /* Skip whole core dumping part */
+                goto log;
 
-                /* Vacuum again, but exclude the coredump we just created */
-                (void) coredump_vacuum(coredump_node_fd >= 0 ? coredump_node_fd : coredump_fd, arg_keep_free, arg_max_use);
-        }
+        /* If we don't want to keep the coredump on disk, remove it now, as later on we
+         * will lack the privileges for it. However, we keep the fd to it, so that we can
+         * still process it and log it. */
+        r = maybe_remove_external_coredump(filename, coredump_node_fd >= 0 ? coredump_compressed_size : coredump_size);
+        if (r < 0)
+                return r;
+        if (r == 0)
+                (void) iovw_put_string_field(iovw, "COREDUMP_FILENAME=", filename);
+        else if (arg_storage == COREDUMP_STORAGE_EXTERNAL)
+                log_info("The core will not be stored: size %"PRIu64" is greater than %"PRIu64" (the configured maximum)",
+                         coredump_node_fd >= 0 ? coredump_compressed_size : coredump_size, arg_external_size_max);
 
-        /* Now, let's drop privileges to become the user who owns the segfaulted process and allocate the
-         * coredump memory under the user's uid. This also ensures that the credentials journald will see are
-         * the ones of the coredumping user, thus making sure the user gets access to the core dump. Let's
-         * also get rid of all capabilities, if we run as root, we won't need them anymore. */
+        /* Vacuum again, but exclude the coredump we just created */
+        (void) coredump_vacuum(coredump_node_fd >= 0 ? coredump_node_fd : coredump_fd, arg_keep_free, arg_max_use);
+
+        /* Now, let's drop privileges to become the user who owns the segfaulted process
+         * and allocate the coredump memory under the user's uid. This also ensures that
+         * the credentials journald will see are the ones of the coredumping user, thus
+         * making sure the user gets access to the core dump. Let's also get rid of all
+         * capabilities, if we run as root, we won't need them anymore. */
         r = change_uid_gid(context);
         if (r < 0)
                 return log_error_errno(r, "Failed to drop privileges: %m");
 
-        if (written) {
-                /* Try to get a stack trace if we can */
-                if (coredump_size > arg_process_size_max)
-                        log_debug("Not generating stack trace: core size %"PRIu64" is greater "
-                                  "than %"PRIu64" (the configured maximum)",
-                                  coredump_size, arg_process_size_max);
-                else if (coredump_fd >= 0) {
-                        bool skip = startswith(context->meta[META_COMM], "systemd-coredum"); /* COMM is 16 bytes usually */
+        /* Try to get a stack trace if we can */
+        if (coredump_size > arg_process_size_max)
+                log_debug("Not generating stack trace: core size %"PRIu64" is greater "
+                          "than %"PRIu64" (the configured maximum)",
+                          coredump_size, arg_process_size_max);
+        else if (coredump_fd >= 0) {
+                bool skip = startswith(context->meta[META_COMM], "systemd-coredum"); /* COMM is 16 bytes usually */
 
-                        (void) parse_elf_object(coredump_fd,
-                                                context->meta[META_EXE],
-                                                /* fork_disable_dump= */ skip, /* avoid loops */
-                                                &stacktrace,
-                                                &json_metadata);
-                }
+                (void) parse_elf_object(coredump_fd,
+                                        context->meta[META_EXE],
+                                        /* fork_disable_dump= */ skip, /* avoid loops */
+                                        &stacktrace,
+                                        &json_metadata);
         }
 
-        _cleanup_free_ char *core_message = NULL;
-        core_message = strjoin(
-                        "Process ", context->meta[META_ARGV_PID],
-                        " (", context->meta[META_COMM],
-                        ") of user ", context->meta[META_ARGV_UID],
-                        written ? " dumped core." : " terminated abnormally without generating a coredump.");
-        if (!core_message)
-                return log_oom();
+log:
+        core_message = strjoina("Process ", context->meta[META_ARGV_PID],
+                                " (", context->meta[META_COMM], ") of user ",
+                                context->meta[META_ARGV_UID], " dumped core.",
+                                context->is_journald && filename ? "\nCoredump diverted to " : NULL,
+                                context->is_journald && filename ? filename : NULL);
 
-        if (context->is_journald && filename)
-                if (!strextend(&core_message, "\nCoredump diverted to ", filename))
-                        return log_oom();
-
-        if (stacktrace)
-                if (!strextend(&core_message, "\n\n", stacktrace))
-                        return log_oom();
+        core_message = strjoina(core_message, stacktrace ? "\n\n" : NULL, stacktrace);
 
         if (context->is_journald)
                 /* We might not be able to log to the journal, so let's always print the message to another
@@ -1085,6 +1065,7 @@ static int save_context(Context *context, const struct iovec_wrapper *iovw) {
 
         assert(context);
         assert(iovw);
+        assert(iovw->count >= _META_ARGV_MAX);
 
         /* The context does not allocate any memory on its own */
 
@@ -1114,16 +1095,6 @@ static int save_context(Context *context, const struct iovec_wrapper *iovw) {
         if (r < 0)
                 return log_error_errno(r, "Failed to parse PID \"%s\": %m", context->meta[META_ARGV_PID]);
 
-        /* The value is set to contents of /proc/sys/fs/suid_dumpable, which we set to 2,
-         * if the process is marked as not dumpable, see PR_SET_DUMPABLE(2const). */
-        if (context->meta[META_ARGV_DUMPABLE]) {
-                r = safe_atou(context->meta[META_ARGV_DUMPABLE], &context->dumpable);
-                if (r < 0)
-                        return log_error_errno(r, "Failed to parse dumpable field \"%s\": %m", context->meta[META_ARGV_DUMPABLE]);
-                if (context->dumpable > 2)
-                        log_notice("Got unexpected %%d/dumpable value %u.", context->dumpable);
-        }
-
         unit = context->meta[META_UNIT];
         context->is_pid1 = streq(context->meta[META_ARGV_PID], "1") || streq_ptr(unit, SPECIAL_INIT_SCOPE);
         context->is_journald = streq_ptr(unit, SPECIAL_JOURNALD_SERVICE);
@@ -1132,7 +1103,7 @@ static int save_context(Context *context, const struct iovec_wrapper *iovw) {
 }
 
 static int process_socket(int fd) {
-        _cleanup_close_ int input_fd = -1;
+        _cleanup_close_ int input_fd = -EBADF;
         Context context = {};
         struct iovec_wrapper iovw = {};
         struct iovec iovec;
@@ -1213,24 +1184,12 @@ static int process_socket(int fd) {
         if (r < 0)
                 goto finish;
 
-        /* Make sure we received all the expected fields. We support being called by an *older*
-         * systemd-coredump from the outside, so we require only the basic set of fields that
-         * was being sent when the support for sending to containers over a socket was added
-         * in a108c43e36d3ceb6e34efe37c014fc2cda856000. */
-        meta_argv_t i;
-        VA_ARGS_FOREACH(i,
-                        META_ARGV_PID,
-                        META_ARGV_UID,
-                        META_ARGV_GID,
-                        META_ARGV_SIGNAL,
-                        META_ARGV_TIMESTAMP,
-                        META_ARGV_RLIMIT,
-                        META_ARGV_HOSTNAME,
-                        META_COMM)
+        /* Make sure we received at least all fields we need. */
+        for (int i = 0; i < _META_MANDATORY_MAX; i++)
                 if (!context.meta[i]) {
                         r = log_error_errno(SYNTHETIC_ERRNO(EINVAL),
-                                            "Mandatory argument %s not received on socket, aborting.",
-                                            meta_field_names[i]);
+                                            "A mandatory argument (%i) has not been sent, aborting.",
+                                            i);
                         goto finish;
                 }
 
@@ -1242,7 +1201,7 @@ finish:
 }
 
 static int send_iovec(const struct iovec_wrapper *iovw, int input_fd) {
-        _cleanup_close_ int fd = -1;
+        _cleanup_close_ int fd = -EBADF;
         int r;
 
         assert(iovw);
@@ -1311,17 +1270,14 @@ static int gather_pid_metadata_from_argv(
         char *t;
 
         /* We gather all metadata that were passed via argv[] into an array of iovecs that
-         * we'll forward to the socket unit.
-         *
-         * We require at least _META_ARGV_REQUIRED args, but will accept more.
-         * We know how to parse _META_ARGV_MAX args. The rest will be ignored. */
+         * we'll forward to the socket unit */
 
-        if (argc < _META_ARGV_REQUIRED)
+        if (argc < _META_ARGV_MAX)
                 return log_error_errno(SYNTHETIC_ERRNO(EINVAL),
-                                       "Not enough arguments passed by the kernel (%i, expected between %i and %i).",
-                                       argc, _META_ARGV_REQUIRED, _META_ARGV_MAX);
+                                       "Not enough arguments passed by the kernel (%i, expected %i).",
+                                       argc, _META_ARGV_MAX);
 
-        for (int i = 0; i < MIN(argc, _META_ARGV_MAX); i++) {
+        for (int i = 0; i < _META_ARGV_MAX; i++) {
 
                 t = argv[i];
 
@@ -1478,7 +1434,7 @@ static int process_kernel(int argc, char* argv[]) {
         /* When we're invoked by the kernel, stdout/stderr are closed which is dangerous because the fds
          * could get reallocated. To avoid hard to debug issues, let's instead bind stdout/stderr to
          * /dev/null. */
-        r = rearrange_stdio(STDIN_FILENO, -1, -1);
+        r = rearrange_stdio(STDIN_FILENO, -EBADF, -EBADF);
         if (r < 0)
                 return log_error_errno(r, "Failed to connect stdout/stderr to /dev/null: %m");
 
@@ -1576,7 +1532,7 @@ static int process_backtrace(int argc, char *argv[]) {
 
                 r = iovw_put_string_field(iovw, "MESSAGE=", message);
                 if (r < 0)
-                        goto finish;
+                        return r;
         } else {
                 /* The imported iovecs are not supposed to be freed by us so let's store
                  * them at the end of the array so we can skip them while freeing the
