@@ -6,10 +6,10 @@
 #include <stdlib.h>
 
 #include "alloc-util.h"
+#include "build.h"
 #include "main-func.h"
 #include "pretty-print.h"
 #include "string-table.h"
-#include "util.h"
 #include "virt.h"
 
 static bool arg_quiet = false;
