@@ -3,6 +3,7 @@
 #include <getopt.h>
 #include <locale.h>
 #include <stdbool.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -11,6 +12,7 @@
 
 #include "alloc-util.h"
 #include "architecture.h"
+#include "build.h"
 #include "bus-common-errors.h"
 #include "bus-error.h"
 #include "bus-map-properties.h"
@@ -23,7 +25,6 @@
 #include "pretty-print.h"
 #include "spawn-polkit-agent.h"
 #include "terminal-util.h"
-#include "util.h"
 #include "verbs.h"
 
 static bool arg_ask_password = true;
@@ -46,14 +47,14 @@ typedef struct StatusInfo {
         const char *kernel_release;
         const char *os_pretty_name;
         const char *os_cpe_name;
+        usec_t os_support_end;
         const char *virtualization;
         const char *architecture;
         const char *home_url;
         const char *hardware_vendor;
         const char *hardware_model;
         const char *firmware_version;
-        sd_id128_t machine_id;
-        sd_id128_t boot_id;
+        usec_t firmware_date;
 } StatusInfo;
 
 static const char* chassis_string_to_glyph(const char *chassis) {
@@ -76,27 +77,42 @@ static const char* chassis_string_to_glyph(const char *chassis) {
         return NULL;
 }
 
+static const char *os_support_end_color(usec_t n, usec_t eol) {
+        usec_t left;
+
+        /* If the end of support is over, color output in red. If only a month is left, color output in
+         * yellow. If more than a year is left, color green. In between just show in regular color. */
+
+        if (n >= eol)
+                return ANSI_HIGHLIGHT_RED;
+        left = eol - n;
+        if (left < USEC_PER_MONTH)
+                return ANSI_HIGHLIGHT_YELLOW;
+        if (left > USEC_PER_YEAR)
+                return ANSI_HIGHLIGHT_GREEN;
+
+        return NULL;
+}
+
 static int print_status_info(StatusInfo *i) {
         _cleanup_(table_unrefp) Table *table = NULL;
+        sd_id128_t mid = {}, bid = {};
         TableCell *cell;
         int r;
 
         assert(i);
 
-        table = table_new("key", "value");
+        table = table_new_vertical();
         if (!table)
                 return log_oom();
 
         assert_se(cell = table_get_cell(table, 0, 0));
         (void) table_set_ellipsize_percent(table, cell, 100);
-        (void) table_set_align_percent(table, cell, 100);
-
-        table_set_header(table, false);
 
         table_set_ersatz_string(table, TABLE_ERSATZ_UNSET);
 
         r = table_add_many(table,
-                           TABLE_STRING, "Static hostname:",
+                           TABLE_FIELD, "Static hostname",
                            TABLE_STRING, i->static_hostname);
         if (r < 0)
                 return table_log_add_error(r);
@@ -104,7 +120,7 @@ static int print_status_info(StatusInfo *i) {
         if (!isempty(i->pretty_hostname) &&
             !streq_ptr(i->pretty_hostname, i->static_hostname)) {
                 r = table_add_many(table,
-                                   TABLE_STRING, "Pretty hostname:",
+                                   TABLE_FIELD, "Pretty hostname",
                                    TABLE_STRING, i->pretty_hostname);
                 if (r < 0)
                         return table_log_add_error(r);
@@ -113,7 +129,7 @@ static int print_status_info(StatusInfo *i) {
         if (!isempty(i->hostname) &&
             !streq_ptr(i->hostname, i->static_hostname)) {
                 r = table_add_many(table,
-                                   TABLE_STRING, "Transient hostname:",
+                                   TABLE_FIELD, "Transient hostname",
                                    TABLE_STRING, i->hostname);
                 if (r < 0)
                         return table_log_add_error(r);
@@ -121,7 +137,7 @@ static int print_status_info(StatusInfo *i) {
 
         if (!isempty(i->icon_name)) {
                 r = table_add_many(table,
-                                   TABLE_STRING, "Icon name:",
+                                   TABLE_FIELD, "Icon name",
                                    TABLE_STRING, i->icon_name);
                 if (r < 0)
                         return table_log_add_error(r);
@@ -135,7 +151,7 @@ static int print_status_info(StatusInfo *i) {
                         v = strjoina(i->chassis, " ", v);
 
                 r = table_add_many(table,
-                                   TABLE_STRING, "Chassis:",
+                                   TABLE_FIELD, "Chassis",
                                    TABLE_STRING, v ?: i->chassis);
                 if (r < 0)
                         return table_log_add_error(r);
@@ -143,7 +159,7 @@ static int print_status_info(StatusInfo *i) {
 
         if (!isempty(i->deployment)) {
                 r = table_add_many(table,
-                                   TABLE_STRING, "Deployment:",
+                                   TABLE_FIELD, "Deployment",
                                    TABLE_STRING, i->deployment);
                 if (r < 0)
                         return table_log_add_error(r);
@@ -151,31 +167,33 @@ static int print_status_info(StatusInfo *i) {
 
         if (!isempty(i->location)) {
                 r = table_add_many(table,
-                                   TABLE_STRING, "Location:",
+                                   TABLE_FIELD, "Location",
                                    TABLE_STRING, i->location);
                 if (r < 0)
                         return table_log_add_error(r);
         }
 
-        if (!sd_id128_is_null(i->machine_id)) {
+        r = sd_id128_get_machine(&mid);
+        if (r >= 0) {
                 r = table_add_many(table,
-                                   TABLE_STRING, "Machine ID:",
-                                   TABLE_ID128, i->machine_id);
+                                   TABLE_FIELD, "Machine ID",
+                                   TABLE_ID128, mid);
                 if (r < 0)
                         return table_log_add_error(r);
         }
 
-        if (!sd_id128_is_null(i->boot_id)) {
+        r = sd_id128_get_boot(&bid);
+        if (r >= 0) {
                 r = table_add_many(table,
-                                   TABLE_STRING, "Boot ID:",
-                                   TABLE_ID128, i->boot_id);
+                                   TABLE_FIELD, "Boot ID",
+                                   TABLE_ID128, bid);
                 if (r < 0)
                         return table_log_add_error(r);
         }
 
         if (!isempty(i->virtualization)) {
                 r = table_add_many(table,
-                                   TABLE_STRING, "Virtualization:",
+                                   TABLE_FIELD, "Virtualization",
                                    TABLE_STRING, i->virtualization);
                 if (r < 0)
                         return table_log_add_error(r);
@@ -183,7 +201,7 @@ static int print_status_info(StatusInfo *i) {
 
         if (!isempty(i->os_pretty_name)) {
                 r = table_add_many(table,
-                                   TABLE_STRING, "Operating System:",
+                                   TABLE_FIELD, "Operating System",
                                    TABLE_STRING, i->os_pretty_name,
                                    TABLE_SET_URL, i->home_url);
                 if (r < 0)
@@ -192,18 +210,31 @@ static int print_status_info(StatusInfo *i) {
 
         if (!isempty(i->os_cpe_name)) {
                 r = table_add_many(table,
-                                   TABLE_STRING, "CPE OS Name:",
+                                   TABLE_FIELD, "CPE OS Name",
                                    TABLE_STRING, i->os_cpe_name);
                 if (r < 0)
                         return table_log_add_error(r);
         }
 
+        if (i->os_support_end != USEC_INFINITY) {
+                usec_t n = now(CLOCK_REALTIME);
+
+                r = table_add_many(table,
+                                   TABLE_FIELD, "OS Support End",
+                                   TABLE_TIMESTAMP_DATE, i->os_support_end,
+                                   TABLE_FIELD, n < i->os_support_end ? "OS Support Remaining" : "OS Support Expired",
+                                   TABLE_TIMESPAN_DAY, n < i->os_support_end ? i->os_support_end - n : n - i->os_support_end,
+                                   TABLE_SET_COLOR, os_support_end_color(n, i->os_support_end));
+                if (r < 0)
+                        return table_log_add_error(r);
+        }
+
         if (!isempty(i->kernel_name) && !isempty(i->kernel_release)) {
                 const char *v;
 
                 v = strjoina(i->kernel_name, " ", i->kernel_release);
                 r = table_add_many(table,
-                                   TABLE_STRING, "Kernel:",
+                                   TABLE_FIELD, "Kernel",
                                    TABLE_STRING, v);
                 if (r < 0)
                         return table_log_add_error(r);
@@ -211,7 +242,7 @@ static int print_status_info(StatusInfo *i) {
 
         if (!isempty(i->architecture)) {
                 r = table_add_many(table,
-                                   TABLE_STRING, "Architecture:",
+                                   TABLE_FIELD, "Architecture",
                                    TABLE_STRING, i->architecture);
                 if (r < 0)
                         return table_log_add_error(r);
@@ -219,7 +250,7 @@ static int print_status_info(StatusInfo *i) {
 
         if (!isempty(i->hardware_vendor)) {
                 r = table_add_many(table,
-                                   TABLE_STRING, "Hardware Vendor:",
+                                   TABLE_FIELD, "Hardware Vendor",
                                    TABLE_STRING, i->hardware_vendor);
                 if (r < 0)
                         return table_log_add_error(r);
@@ -227,7 +258,7 @@ static int print_status_info(StatusInfo *i) {
 
         if (!isempty(i->hardware_model)) {
                 r = table_add_many(table,
-                                   TABLE_STRING, "Hardware Model:",
+                                   TABLE_FIELD, "Hardware Model",
                                    TABLE_STRING, i->hardware_model);
                 if (r < 0)
                         return table_log_add_error(r);
@@ -235,12 +266,20 @@ static int print_status_info(StatusInfo *i) {
 
         if (!isempty(i->firmware_version)) {
                 r = table_add_many(table,
-                                   TABLE_STRING, "Firmware Version:",
+                                   TABLE_FIELD, "Firmware Version",
                                    TABLE_STRING, i->firmware_version);
                 if (r < 0)
                         return table_log_add_error(r);
         }
 
+        if (timestamp_is_set(i->firmware_date)) {
+                r = table_add_many(table,
+                                   TABLE_FIELD, "Firmware Date",
+                                   TABLE_TIMESTAMP_DATE, i->firmware_date);
+                if (r < 0)
+                        return table_log_add_error(r);
+        }
+
         r = table_print(table, NULL);
         if (r < 0)
                 return table_log_print_error(r);
@@ -302,10 +341,12 @@ static int show_all_names(sd_bus *bus) {
                 { "KernelRelease",             "s", NULL, offsetof(StatusInfo, kernel_release)   },
                 { "OperatingSystemPrettyName", "s", NULL, offsetof(StatusInfo, os_pretty_name)   },
                 { "OperatingSystemCPEName",    "s", NULL, offsetof(StatusInfo, os_cpe_name)      },
+                { "OperatingSystemSupportEnd", "t", NULL, offsetof(StatusInfo, os_support_end)   },
                 { "HomeURL",                   "s", NULL, offsetof(StatusInfo, home_url)         },
                 { "HardwareVendor",            "s", NULL, offsetof(StatusInfo, hardware_vendor)  },
                 { "HardwareModel",             "s", NULL, offsetof(StatusInfo, hardware_model)   },
                 { "FirmwareVersion",           "s", NULL, offsetof(StatusInfo, firmware_version) },
+                { "FirmwareDate",              "t", NULL, offsetof(StatusInfo, firmware_date)    },
                 {}
         };
 
@@ -341,13 +382,6 @@ static int show_all_names(sd_bus *bus) {
         if (r < 0)
                 return log_error_errno(r, "Failed to query system properties: %s", bus_error_message(&error, r));
 
-        if (!arg_host) {
-                if (sd_id128_is_null(info.machine_id))
-                        (void) sd_id128_get_machine(&info.machine_id);
-                if (sd_id128_is_null(info.boot_id))
-                        (void) sd_id128_get_boot(&info.boot_id);
-        }
-
         return print_status_info(&info);
 }
 
