@@ -28,7 +28,6 @@
 #include "tmpfile-util.h"
 #include "user-util.h"
 #include "utf8.h"
-#include "util.h"
 #include "web-util.h"
 
 typedef enum TarProgress {
@@ -246,7 +245,7 @@ static int tar_pull_make_local_copy(TarPull *i) {
                                 BTRFS_SNAPSHOT_FALLBACK_DIRECTORY|
                                 BTRFS_SNAPSHOT_RECURSIVE);
         else
-                r = copy_tree(i->final_path, t, UID_INVALID, GID_INVALID, COPY_REFLINK|COPY_HARDLINKS);
+                r = copy_tree(i->final_path, t, UID_INVALID, GID_INVALID, COPY_REFLINK|COPY_HARDLINKS, NULL);
         if (r < 0)
                 return log_error_errno(r, "Failed to create local image: %m");
 
@@ -408,7 +407,7 @@ static void tar_pull_job_on_finished(PullJob *j) {
                 r = install_file(
                                 AT_FDCWD, i->local,
                                 AT_FDCWD, NULL,
-                                (i->flags & PULL_READ_ONLY ? INSTALL_READ_ONLY : 0) |
+                                (i->flags & PULL_READ_ONLY) ? INSTALL_READ_ONLY : 0 |
                                 (i->flags & PULL_SYNC ? INSTALL_SYNCFS : 0));
                 if (r < 0) {
                         log_error_errno(r, "Failed to finalize '%s': %m", i->local);
