@@ -14,8 +14,8 @@
 #include "alloc-util.h"
 #include "bus-error.h"
 #include "bus-util.h"
+#include "constants.h"
 #include "daemon-util.h"
-#include "def.h"
 #include "fd-util.h"
 #include "format-util.h"
 #include "initreq.h"
@@ -24,6 +24,7 @@
 #include "main-func.h"
 #include "memory-util.h"
 #include "process-util.h"
+#include "reboot-util.h"
 #include "special.h"
 
 #define SERVER_FD_MAX 16
@@ -310,7 +311,7 @@ static int process_event(Server *s, struct epoll_event *ev) {
 }
 
 static int run(int argc, char *argv[]) {
-        _cleanup_(server_done) Server server = { .epoll_fd = -1 };
+        _cleanup_(server_done) Server server = { .epoll_fd = -EBADF };
         _unused_ _cleanup_(notify_on_cleanup) const char *notify_stop = NULL;
         int r, n;
 
@@ -324,7 +325,8 @@ static int run(int argc, char *argv[]) {
 
         n = sd_listen_fds(true);
         if (n < 0)
-                return log_error_errno(n, "Failed to read listening file descriptors from environment: %m");
+                return log_error_errno(errno,
+                                       "Failed to read listening file descriptors from environment: %m");
 
         if (n <= 0 || n > SERVER_FD_MAX)
                 return log_error_errno(SYNTHETIC_ERRNO(EINVAL),
