@@ -10,7 +10,7 @@
 
 #include "af-list.h"
 #include "alloc-util.h"
-#include "def.h"
+#include "constants.h"
 #include "errno-util.h"
 #include "escape.h"
 #include "fd-util.h"
@@ -294,7 +294,7 @@ int journal_remote_add_raw_socket(RemoteServer *s, int fd) {
         if (r < 0)
                 return r;
 
-        fd_ = -1;
+        fd_ = -EBADF;
         s->active++;
         return 0;
 }
@@ -362,6 +362,8 @@ void journal_remote_server_destroy(RemoteServer *s) {
         writer_unref(s->_single_writer);
         hashmap_free(s->writers);
 
+        sd_event_source_unref(s->sigterm_event);
+        sd_event_source_unref(s->sigint_event);
         sd_event_source_unref(s->listen_event);
         sd_event_unref(s->events);
 
@@ -481,7 +483,7 @@ static int accept_connection(
                 SocketAddress *addr,
                 char **hostname) {
 
-        _cleanup_close_ int fd2 = -1;
+        _cleanup_close_ int fd2 = -EBADF;
         int r;
 
         log_debug("Accepting new %s connection on fd:%d", type, fd);
@@ -495,9 +497,7 @@ static int accept_connection(
 
         switch (socket_address_family(addr)) {
         case AF_INET:
-        case AF_INET6:
-        case AF_VSOCK:
-        case AF_UNIX: {
+        case AF_INET6: {
                 _cleanup_free_ char *a = NULL;
                 char *b;
 
