@@ -10,9 +10,10 @@
 #include "sd-daemon.h"
 
 #include "alloc-util.h"
+#include "build.h"
 #include "conf-parser.h"
+#include "constants.h"
 #include "daemon-util.h"
-#include "def.h"
 #include "env-file.h"
 #include "fd-util.h"
 #include "fileio.h"
@@ -32,7 +33,6 @@
 #include "string-util.h"
 #include "strv.h"
 #include "tmpfile-util.h"
-#include "util.h"
 #include "version.h"
 
 #define PRIV_KEY_FILE CERTIFICATE_ROOT "/private/journal-upload.pem"
@@ -55,8 +55,6 @@ static int arg_follow = -1;
 static const char *arg_save_state = NULL;
 static usec_t arg_network_timeout_usec = USEC_INFINITY;
 
-STATIC_DESTRUCTOR_REGISTER(arg_file, strv_freep);
-
 static void close_fd_input(Uploader *u);
 
 #define SERVER_ANSWER_KEEP 2048
@@ -379,6 +377,38 @@ static int open_file_for_upload(Uploader *u, const char *filename) {
         return r;
 }
 
+static int dispatch_sigterm(sd_event_source *event,
+                            const struct signalfd_siginfo *si,
+                            void *userdata) {
+        Uploader *u = ASSERT_PTR(userdata);
+
+        log_received_signal(LOG_INFO, si);
+
+        close_fd_input(u);
+        close_journal_input(u);
+
+        sd_event_exit(u->events, 0);
+        return 0;
+}
+
+static int setup_signals(Uploader *u) {
+        int r;
+
+        assert(u);
+
+        assert_se(sigprocmask_many(SIG_SETMASK, NULL, SIGINT, SIGTERM, -1) >= 0);
+
+        r = sd_event_add_signal(u->events, &u->sigterm_event, SIGTERM, dispatch_sigterm, u);
+        if (r < 0)
+                return r;
+
+        r = sd_event_add_signal(u->events, &u->sigint_event, SIGINT, dispatch_sigterm, u);
+        if (r < 0)
+                return r;
+
+        return 0;
+}
+
 static int setup_uploader(Uploader *u, const char *url, const char *state_file) {
         int r;
         const char *host, *proto = "";
@@ -418,9 +448,9 @@ static int setup_uploader(Uploader *u, const char *url, const char *state_file)
         if (r < 0)
                 return log_error_errno(r, "sd_event_default failed: %m");
 
-        r = sd_event_set_signal_exit(u->events, true);
+        r = setup_signals(u);
         if (r < 0)
-                return log_error_errno(r, "Failed to install SIGINT/SIGTERM handlers: %m");
+                return log_error_errno(r, "Failed to set up signals: %m");
 
         (void) sd_watchdog_enabled(false, &u->watchdog_usec);
 
@@ -444,6 +474,8 @@ static void destroy_uploader(Uploader *u) {
         close_fd_input(u);
         close_journal_input(u);
 
+        sd_event_source_unref(u->sigterm_event);
+        sd_event_source_unref(u->sigint_event);
         sd_event_unref(u->events);
 }
 
