@@ -31,7 +31,7 @@
 static int icmp6_bind_router_message(const struct icmp6_filter *filter,
                                      const struct ipv6_mreq *mreq) {
         int ifindex = mreq->ipv6mr_interface;
-        _cleanup_close_ int s = -1;
+        _cleanup_close_ int s = -EBADF;
         int r;
 
         assert(filter);
@@ -144,17 +144,13 @@ int icmp6_send_router_solicitation(int s, const struct ether_addr *ether_addr) {
         return 0;
 }
 
-int icmp6_receive(
-                int fd,
-                void *buffer,
-                size_t size,
-                struct in6_addr *ret_sender,
-                triple_timestamp *ret_timestamp) {
+int icmp6_receive(int fd, void *buffer, size_t size, struct in6_addr *ret_dst,
+                  triple_timestamp *ret_timestamp) {
 
         /* This needs to be initialized with zero. See #20741. */
         CMSG_BUFFER_TYPE(CMSG_SPACE(sizeof(int)) + /* ttl */
                          CMSG_SPACE_TIMEVAL) control = {};
-        struct iovec iov = { buffer, size };
+        struct iovec iov = {};
         union sockaddr_union sa = {};
         struct msghdr msg = {
                 .msg_name = &sa.sa,
@@ -169,6 +165,8 @@ int icmp6_receive(
         triple_timestamp t = {};
         ssize_t len;
 
+        iov = IOVEC_MAKE(buffer, size);
+
         len = recvmsg_safe(fd, &msg, MSG_DONTWAIT);
         if (len < 0)
                 return (int) len;
@@ -180,7 +178,7 @@ int icmp6_receive(
             sa.in6.sin6_family == AF_INET6)  {
 
                 addr = sa.in6.sin6_addr;
-                if (!in6_addr_is_link_local(&addr) && !in6_addr_is_null(&addr))
+                if (!in6_addr_is_link_local(&addr))
                         return -EADDRNOTAVAIL;
 
         } else if (msg.msg_namelen > 0)
@@ -206,14 +204,10 @@ int icmp6_receive(
                         triple_timestamp_from_realtime(&t, timeval_load((struct timeval*) CMSG_DATA(cmsg)));
         }
 
-        if (ret_timestamp) {
-                if (triple_timestamp_is_set(&t))
-                        *ret_timestamp = t;
-                else
-                        triple_timestamp_get(ret_timestamp);
-        }
+        if (!triple_timestamp_is_set(&t))
+                triple_timestamp_get(&t);
 
-        if (ret_sender)
-                *ret_sender = addr;
+        *ret_dst = addr;
+        *ret_timestamp = t;
         return 0;
 }
