@@ -14,7 +14,6 @@
 #include "errno-util.h"
 #include "string-util.h"
 #include "strv.h"
-#include "util.h"
 
 BUS_ERROR_MAP_ELF_REGISTER const sd_bus_error_map bus_standard_errors[] = {
         SD_BUS_ERROR_MAP("org.freedesktop.DBus.Error.Failed",                           EACCES),
@@ -278,16 +277,14 @@ _public_ int sd_bus_error_setf(sd_bus_error *e, const char *name, const char *fo
 
                 va_start(ap, format);
                 r = sd_bus_error_setfv(e, name, format, ap);
-                if (name)
-                        assert(r < 0);
+                assert(!name || r < 0);
                 va_end(ap);
 
                 return r;
         }
 
         r = sd_bus_error_set(e, name, NULL);
-        if (name)
-                assert(r < 0);
+        assert(!name || r < 0);
         return r;
 }
 
