@@ -18,10 +18,12 @@
 #include "macro.h"
 #include "string-util.h"
 #include "tests.h"
-#include "util.h"
 
 static int match_callback(sd_bus_message *m, void *userdata, sd_bus_error *ret_error) {
-        log_info("Match triggered! interface=%s member=%s", strna(sd_bus_message_get_interface(m)), strna(sd_bus_message_get_member(m)));
+        log_info("Match triggered! destination=%s interface=%s member=%s",
+                 strna(sd_bus_message_get_destination(m)),
+                 strna(sd_bus_message_get_interface(m)),
+                 strna(sd_bus_message_get_member(m)));
         return 0;
 }
 
@@ -95,6 +97,12 @@ static int server_init(sd_bus **_bus) {
                 goto fail;
         }
 
+        r = sd_bus_match_signal(bus, NULL, NULL, NULL, "foo.bar", "NotifyTo", match_callback, NULL);
+        if (r < 0) {
+                log_error_errno(r, "Failed to request match: %m");
+                goto fail;
+        }
+
         r = sd_bus_add_match(bus, NULL, "type='signal',interface='org.freedesktop.DBus',member='NameOwnerChanged'", match_callback, NULL);
         if (r < 0) {
                 log_error_errno(r, "Failed to add match: %m");
@@ -117,7 +125,6 @@ static int server(sd_bus *bus) {
 
         while (!client1_gone || !client2_gone) {
                 _cleanup_(sd_bus_message_unrefp) sd_bus_message *m = NULL;
-                _cleanup_(sd_bus_creds_unrefp) sd_bus_creds *creds = NULL;
                 pid_t pid = 0;
                 const char *label = NULL;
 
@@ -140,17 +147,8 @@ static int server(sd_bus *bus) {
                 if (!m)
                         continue;
 
-                r = sd_bus_query_sender_creds(m, SD_BUS_CREDS_AUGMENT | SD_BUS_CREDS_PID | SD_BUS_CREDS_SELINUX_CONTEXT, &creds);
-                if (r < 0)
-                        log_debug_errno(r, "Failed to query sender credentials, ignoring: %m");
-                else {
-                        r = sd_bus_creds_get_pid(creds, &pid);
-                        if (r < 0)
-                                return log_error_errno(r, "Failed to get sender pid: %m");
-
-                        (void) sd_bus_creds_get_selinux_context(creds, &label);
-                }
-
+                sd_bus_creds_get_pid(sd_bus_message_get_creds(m), &pid);
+                sd_bus_creds_get_selinux_context(sd_bus_message_get_creds(m), &label);
                 log_info("Got message! member=%s pid="PID_FMT" label=%s",
                          strna(sd_bus_message_get_member(m)),
                          pid,
@@ -262,7 +260,7 @@ static void* client1(void *p) {
         _cleanup_(sd_bus_error_free) sd_bus_error error = SD_BUS_ERROR_NULL;
         const char *hello;
         int r;
-        _cleanup_close_pair_ int pp[2] = { -1, -1 };
+        _cleanup_close_pair_ int pp[2] = PIPE_EBADF;
         char x;
 
         r = sd_bus_open_user(&bus);
@@ -407,6 +405,26 @@ static void* client2(void *p) {
 
         m = sd_bus_message_unref(m);
 
+        r = sd_bus_message_new_signal_to(
+                        bus,
+                        &m,
+                        "org.freedesktop.systemd.test",
+                        "/foobar",
+                        "foo.bar",
+                        "NotifyTo");
+        if (r < 0) {
+                log_error_errno(r, "Failed to allocate signal to: %m");
+                goto finish;
+        }
+
+        r = sd_bus_send(bus, m, NULL);
+        if (r < 0) {
+                log_error("Failed to issue signal to: %s", bus_error_message(&error, r));
+                goto finish;
+        }
+
+        m = sd_bus_message_unref(m);
+
         r = sd_bus_message_new_method_call(
                         bus,
                         &m,
@@ -451,12 +469,9 @@ static void* client2(void *p) {
 
         r = sd_bus_call(bus, m, 200 * USEC_PER_MSEC, &error, &reply);
         if (r < 0)
-                log_debug("Failed to issue method call: %s", bus_error_message(&error, r));
-        else {
-                log_error("Slow call unexpectedly succeed.");
-                r = -ENOANO;
-                goto finish;
-        }
+                log_info("Failed to issue method call: %s", bus_error_message(&error, r));
+        else
+                log_info("Slow call succeed.");
 
         m = sd_bus_message_unref(m);
 
