@@ -131,7 +131,7 @@ static void test_sd_device_one(sd_device *d) {
                         assert_se(streq(syspath, val));
                         dev = sd_device_unref(dev);
 
-                        _cleanup_close_ int fd = -1;
+                        _cleanup_close_ int fd = -EBADF;
                         fd = sd_device_open(d, O_CLOEXEC| O_NONBLOCK | (is_block ? O_RDONLY : O_NOCTTY | O_PATH));
                         assert_se(fd >= 0 || ERRNO_IS_PRIVILEGE(fd));
                 } else
@@ -430,8 +430,6 @@ static void check_parent_match(sd_device_enumerator *e, sd_device *dev) {
 TEST(sd_device_enumerator_add_match_parent) {
         _cleanup_(sd_device_enumerator_unrefp) sd_device_enumerator *e = NULL;
         sd_device *dev;
-        /* Some devices have thousands of children. Avoid spending too much time in the double loop below. */
-        unsigned iterations = 200;
         int r;
 
         assert_se(sd_device_enumerator_new(&e) >= 0);
@@ -449,9 +447,6 @@ TEST(sd_device_enumerator_add_match_parent) {
                 const char *syspath;
                 sd_device *parent;
 
-                if (iterations-- == 0)
-                        break;
-
                 assert_se(sd_device_get_syspath(dev, &syspath) >= 0);
 
                 r = sd_device_get_parent(dev, &parent);
@@ -481,8 +476,6 @@ TEST(sd_device_enumerator_add_match_parent) {
 TEST(sd_device_get_child) {
         _cleanup_(sd_device_enumerator_unrefp) sd_device_enumerator *e = NULL;
         sd_device *dev;
-        /* Some devices have thousands of children. Avoid spending too much time in the double loop below. */
-        unsigned iterations = 3000;
         int r;
 
         assert_se(sd_device_enumerator_new(&e) >= 0);
@@ -516,9 +509,6 @@ TEST(sd_device_get_child) {
                 FOREACH_DEVICE_CHILD_WITH_SUFFIX(parent, child, suffix) {
                         const char *s;
 
-                        if (iterations-- == 0)
-                                return;
-
                         assert_se(child);
                         assert_se(suffix);
 
@@ -545,7 +535,7 @@ TEST(sd_device_new_from_nulstr) {
 
         _cleanup_(sd_device_unrefp) sd_device *device = NULL, *from_nulstr = NULL;
         _cleanup_free_ char *nulstr_copy = NULL;
-        const char *devlink, *nulstr;
+        const char *nulstr;
         size_t len;
 
         assert_se(sd_device_new_from_syspath(&device, "/sys/class/net/lo") >= 0);
