@@ -495,7 +495,7 @@ _public_ int sd_resolve_new(sd_resolve **ret) {
         resolve->original_pid = getpid_cached();
 
         for (i = 0; i < _FD_MAX; i++)
-                resolve->fds[i] = -1;
+                resolve->fds[i] = -EBADF;
 
         if (socketpair(AF_UNIX, SOCK_DGRAM|SOCK_CLOEXEC, 0, resolve->fds + REQUEST_RECV_FD) < 0)
                 return -errno;
