@@ -20,11 +20,15 @@
 #include "mkdir-label.h"
 #include "nulstr-util.h"
 #include "process-util.h"
+#include "stat-util.h"
 #include "string-util.h"
 #include "strv.h"
 #include "tmpfile-util.h"
 
 static bool startswith_comma(const char *s, const char *prefix) {
+        assert(s);
+        assert(prefix);
+
         s = startswith(s, prefix);
         if (!s)
                 return false;
@@ -52,30 +56,201 @@ static const char* systemd_language_fallback_map(void) {
         return SYSTEMD_LANGUAGE_FALLBACK_MAP;
 }
 
-static void context_free_x11(Context *c) {
-        c->x11_layout = mfree(c->x11_layout);
-        c->x11_options = mfree(c->x11_options);
-        c->x11_model = mfree(c->x11_model);
-        c->x11_variant = mfree(c->x11_variant);
+void x11_context_clear(X11Context *xc) {
+        assert(xc);
+
+        xc->layout  = mfree(xc->layout);
+        xc->options = mfree(xc->options);
+        xc->model   = mfree(xc->model);
+        xc->variant = mfree(xc->variant);
 }
 
-static void context_free_vconsole(Context *c) {
-        c->vc_keymap = mfree(c->vc_keymap);
-        c->vc_keymap_toggle = mfree(c->vc_keymap_toggle);
+void x11_context_replace(X11Context *dest, X11Context *src) {
+        assert(dest);
+        assert(src);
+
+        x11_context_clear(dest);
+        *dest = *src;
+        *src = (X11Context) {};
+}
+
+bool x11_context_isempty(const X11Context *xc) {
+        assert(xc);
+
+        return
+                isempty(xc->layout)  &&
+                isempty(xc->model)   &&
+                isempty(xc->variant) &&
+                isempty(xc->options);
+}
+
+void x11_context_empty_to_null(X11Context *xc) {
+        assert(xc);
+
+        /* Do not call x11_context_clear() for the passed object. */
+
+        xc->layout  = empty_to_null(xc->layout);
+        xc->model   = empty_to_null(xc->model);
+        xc->variant = empty_to_null(xc->variant);
+        xc->options = empty_to_null(xc->options);
+}
+
+bool x11_context_is_safe(const X11Context *xc) {
+        assert(xc);
+
+        return
+                (!xc->layout  || string_is_safe(xc->layout))  &&
+                (!xc->model   || string_is_safe(xc->model))   &&
+                (!xc->variant || string_is_safe(xc->variant)) &&
+                (!xc->options || string_is_safe(xc->options));
+}
+
+bool x11_context_equal(const X11Context *a, const X11Context *b) {
+        assert(a);
+        assert(b);
+
+        return
+                streq_ptr(a->layout,  b->layout)  &&
+                streq_ptr(a->model,   b->model)   &&
+                streq_ptr(a->variant, b->variant) &&
+                streq_ptr(a->options, b->options);
+}
+
+int x11_context_copy(X11Context *dest, const X11Context *src) {
+        bool modified;
+        int r;
+
+        assert(dest);
+
+        if (dest == src)
+                return 0;
+
+        if (!src) {
+                modified = !x11_context_isempty(dest);
+                x11_context_clear(dest);
+                return modified;
+        }
+
+        r = free_and_strdup(&dest->layout, src->layout);
+        if (r < 0)
+                return r;
+        modified = r > 0;
+
+        r = free_and_strdup(&dest->model, src->model);
+        if (r < 0)
+                return r;
+        modified = modified || r > 0;
+
+        r = free_and_strdup(&dest->variant, src->variant);
+        if (r < 0)
+                return r;
+        modified = modified || r > 0;
+
+        r = free_and_strdup(&dest->options, src->options);
+        if (r < 0)
+                return r;
+        modified = modified || r > 0;
+
+        return modified;
+}
+
+void vc_context_clear(VCContext *vc) {
+        assert(vc);
+
+        vc->keymap = mfree(vc->keymap);
+        vc->toggle = mfree(vc->toggle);
+}
+
+void vc_context_replace(VCContext *dest, VCContext *src) {
+        assert(dest);
+        assert(src);
+
+        vc_context_clear(dest);
+        *dest = *src;
+        *src = (VCContext) {};
+}
+
+bool vc_context_isempty(const VCContext *vc) {
+        assert(vc);
+
+        return
+                isempty(vc->keymap) &&
+                isempty(vc->toggle);
+}
+
+void vc_context_empty_to_null(VCContext *vc) {
+        assert(vc);
+
+        /* Do not call vc_context_clear() for the passed object. */
+
+        vc->keymap = empty_to_null(vc->keymap);
+        vc->toggle = empty_to_null(vc->toggle);
+}
+
+bool vc_context_equal(const VCContext *a, const VCContext *b) {
+        assert(a);
+        assert(b);
+
+        return
+                streq_ptr(a->keymap, b->keymap) &&
+                streq_ptr(a->toggle, b->toggle);
+}
+
+int vc_context_copy(VCContext *dest, const VCContext *src) {
+        bool modified;
+        int r;
+
+        assert(dest);
+
+        if (dest == src)
+                return 0;
+
+        if (!src) {
+                modified = !vc_context_isempty(dest);
+                vc_context_clear(dest);
+                return modified;
+        }
+
+        r = free_and_strdup(&dest->keymap, src->keymap);
+        if (r < 0)
+                return r;
+        modified = r > 0;
+
+        r = free_and_strdup(&dest->toggle, src->toggle);
+        if (r < 0)
+                return r;
+        modified = modified || r > 0;
+
+        return modified;
 }
 
 void context_clear(Context *c) {
+        assert(c);
+
         locale_context_clear(&c->locale_context);
-        context_free_x11(c);
-        context_free_vconsole(c);
+        x11_context_clear(&c->x11_from_xorg);
+        x11_context_clear(&c->x11_from_vc);
+        vc_context_clear(&c->vc);
 
-        sd_bus_message_unref(c->locale_cache);
-        sd_bus_message_unref(c->x11_cache);
-        sd_bus_message_unref(c->vc_cache);
+        c->locale_cache = sd_bus_message_unref(c->locale_cache);
+        c->x11_cache = sd_bus_message_unref(c->x11_cache);
+        c->vc_cache = sd_bus_message_unref(c->vc_cache);
 
-        bus_verify_polkit_async_registry_free(c->polkit_registry);
+        c->polkit_registry = bus_verify_polkit_async_registry_free(c->polkit_registry);
 };
 
+X11Context *context_get_x11_context(Context *c) {
+        assert(c);
+
+        if (!x11_context_isempty(&c->x11_from_vc))
+                return &c->x11_from_vc;
+
+        if (!x11_context_isempty(&c->x11_from_xorg))
+                return &c->x11_from_xorg;
+
+        return &c->x11_from_vc;
+}
+
 int locale_read_data(Context *c, sd_bus_message *m) {
         assert(c);
 
@@ -92,8 +267,10 @@ int locale_read_data(Context *c, sd_bus_message *m) {
 }
 
 int vconsole_read_data(Context *c, sd_bus_message *m) {
+        _cleanup_close_ int fd = -EBADF;
         struct stat st;
-        usec_t t;
+
+        assert(c);
 
         /* Do not try to re-read the file within single bus operation. */
         if (m) {
@@ -104,35 +281,45 @@ int vconsole_read_data(Context *c, sd_bus_message *m) {
                 c->vc_cache = sd_bus_message_ref(m);
         }
 
-        if (stat("/etc/vconsole.conf", &st) < 0) {
-                if (errno != ENOENT)
-                        return -errno;
-
-                c->vc_mtime = USEC_INFINITY;
-                context_free_vconsole(c);
+        fd = RET_NERRNO(open("/etc/vconsole.conf", O_CLOEXEC | O_PATH));
+        if (fd == -ENOENT) {
+                c->vc_stat = (struct stat) {};
+                vc_context_clear(&c->vc);
+                x11_context_clear(&c->x11_from_vc);
                 return 0;
         }
+        if (fd < 0)
+                return fd;
 
-        /* If mtime is not changed, then we do not need to re-read */
-        t = timespec_load(&st.st_mtim);
-        if (c->vc_mtime != USEC_INFINITY && t == c->vc_mtime)
+        if (fstat(fd, &st) < 0)
+                return -errno;
+
+        /* If the file is not changed, then we do not need to re-read */
+        if (stat_inode_unmodified(&c->vc_stat, &st))
                 return 0;
 
-        c->vc_mtime = t;
-        context_free_vconsole(c);
+        c->vc_stat = st;
+        vc_context_clear(&c->vc);
+        x11_context_clear(&c->x11_from_vc);
 
-        return parse_env_file(NULL, "/etc/vconsole.conf",
-                              "KEYMAP",        &c->vc_keymap,
-                              "KEYMAP_TOGGLE", &c->vc_keymap_toggle);
+        return parse_env_file_fd(fd, "/etc/vconsole.conf",
+                                 "KEYMAP",        &c->vc.keymap,
+                                 "KEYMAP_TOGGLE", &c->vc.toggle,
+                                 "XKBLAYOUT",     &c->x11_from_vc.layout,
+                                 "XKBMODEL",      &c->x11_from_vc.model,
+                                 "XKBVARIANT",    &c->x11_from_vc.variant,
+                                 "XKBOPTIONS",    &c->x11_from_vc.options);
 }
 
 int x11_read_data(Context *c, sd_bus_message *m) {
+        _cleanup_close_ int fd = -EBADF, fd_ro = -EBADF;
         _cleanup_fclose_ FILE *f = NULL;
         bool in_section = false;
         struct stat st;
-        usec_t t;
         int r;
 
+        assert(c);
+
         /* Do not try to re-read the file within single bus operation. */
         if (m) {
                 if (m == c->x11_cache)
@@ -142,27 +329,35 @@ int x11_read_data(Context *c, sd_bus_message *m) {
                 c->x11_cache = sd_bus_message_ref(m);
         }
 
-        if (stat("/etc/X11/xorg.conf.d/00-keyboard.conf", &st) < 0) {
-                if (errno != ENOENT)
-                        return -errno;
-
-                c->x11_mtime = USEC_INFINITY;
-                context_free_x11(c);
+        fd = RET_NERRNO(open("/etc/X11/xorg.conf.d/00-keyboard.conf", O_CLOEXEC | O_PATH));
+        if (fd == -ENOENT) {
+                c->x11_stat = (struct stat) {};
+                x11_context_clear(&c->x11_from_xorg);
                 return 0;
         }
+        if (fd < 0)
+                return fd;
 
-        /* If mtime is not changed, then we do not need to re-read */
-        t = timespec_load(&st.st_mtim);
-        if (c->x11_mtime != USEC_INFINITY && t == c->x11_mtime)
+        if (fstat(fd, &st) < 0)
+                return -errno;
+
+        /* If the file is not changed, then we do not need to re-read */
+        if (stat_inode_unmodified(&c->x11_stat, &st))
                 return 0;
 
-        c->x11_mtime = t;
-        context_free_x11(c);
+        c->x11_stat = st;
+        x11_context_clear(&c->x11_from_xorg);
 
-        f = fopen("/etc/X11/xorg.conf.d/00-keyboard.conf", "re");
+        fd_ro = fd_reopen(fd, O_CLOEXEC | O_RDONLY);
+        if (fd_ro < 0)
+                return fd_ro;
+
+        f = fdopen(fd_ro, "re");
         if (!f)
                 return -errno;
 
+        TAKE_FD(fd_ro);
+
         for (;;) {
                 _cleanup_free_ char *line = NULL;
                 char *l;
@@ -188,13 +383,13 @@ int x11_read_data(Context *c, sd_bus_message *m) {
                                 char **p = NULL;
 
                                 if (streq(a[1], "XkbLayout"))
-                                        p = &c->x11_layout;
+                                        p = &c->x11_from_xorg.layout;
                                 else if (streq(a[1], "XkbModel"))
-                                        p = &c->x11_model;
+                                        p = &c->x11_from_xorg.model;
                                 else if (streq(a[1], "XkbVariant"))
-                                        p = &c->x11_variant;
+                                        p = &c->x11_from_xorg.variant;
                                 else if (streq(a[1], "XkbOptions"))
-                                        p = &c->x11_options;
+                                        p = &c->x11_from_xorg.options;
 
                                 if (p)
                                         free_and_replace(*p, a[2]);
@@ -219,18 +414,38 @@ int x11_read_data(Context *c, sd_bus_message *m) {
 
 int vconsole_write_data(Context *c) {
         _cleanup_strv_free_ char **l = NULL;
-        struct stat st;
+        const X11Context *xc;
         int r;
 
+        assert(c);
+
+        xc = context_get_x11_context(c);
+
         r = load_env_file(NULL, "/etc/vconsole.conf", &l);
         if (r < 0 && r != -ENOENT)
                 return r;
 
-        r = strv_env_assign(&l, "KEYMAP", empty_to_null(c->vc_keymap));
+        r = strv_env_assign(&l, "KEYMAP", empty_to_null(c->vc.keymap));
         if (r < 0)
                 return r;
 
-        r = strv_env_assign(&l, "KEYMAP_TOGGLE", empty_to_null(c->vc_keymap_toggle));
+        r = strv_env_assign(&l, "KEYMAP_TOGGLE", empty_to_null(c->vc.toggle));
+        if (r < 0)
+                return r;
+
+        r = strv_env_assign(&l, "XKBLAYOUT", empty_to_null(xc->layout));
+        if (r < 0)
+                return r;
+
+        r = strv_env_assign(&l, "XKBMODEL", empty_to_null(xc->model));
+        if (r < 0)
+                return r;
+
+        r = strv_env_assign(&l, "XKBVARIANT", empty_to_null(xc->variant));
+        if (r < 0)
+                return r;
+
+        r = strv_env_assign(&l, "XKBOPTIONS", empty_to_null(xc->options));
         if (r < 0)
                 return r;
 
@@ -238,7 +453,7 @@ int vconsole_write_data(Context *c) {
                 if (unlink("/etc/vconsole.conf") < 0)
                         return errno == ENOENT ? 0 : -errno;
 
-                c->vc_mtime = USEC_INFINITY;
+                c->vc_stat = (struct stat) {};
                 return 0;
         }
 
@@ -246,27 +461,26 @@ int vconsole_write_data(Context *c) {
         if (r < 0)
                 return r;
 
-        if (stat("/etc/vconsole.conf", &st) >= 0)
-                c->vc_mtime = timespec_load(&st.st_mtim);
+        if (stat("/etc/vconsole.conf", &c->vc_stat) < 0)
+                return -errno;
 
         return 0;
 }
 
 int x11_write_data(Context *c) {
         _cleanup_fclose_ FILE *f = NULL;
-        _cleanup_free_ char *temp_path = NULL;
-        struct stat st;
+        _cleanup_(unlink_and_freep) char *temp_path = NULL;
+        const X11Context *xc;
         int r;
 
-        if (isempty(c->x11_layout) &&
-            isempty(c->x11_model) &&
-            isempty(c->x11_variant) &&
-            isempty(c->x11_options)) {
+        assert(c);
 
+        xc = context_get_x11_context(c);
+        if (x11_context_isempty(xc)) {
                 if (unlink("/etc/X11/xorg.conf.d/00-keyboard.conf") < 0)
                         return errno == ENOENT ? 0 : -errno;
 
-                c->vc_mtime = USEC_INFINITY;
+                c->x11_stat = (struct stat) {};
                 return 0;
         }
 
@@ -284,52 +498,50 @@ int x11_write_data(Context *c) {
               "        Identifier \"system-keyboard\"\n"
               "        MatchIsKeyboard \"on\"\n", f);
 
-        if (!isempty(c->x11_layout))
-                fprintf(f, "        Option \"XkbLayout\" \"%s\"\n", c->x11_layout);
+        if (!isempty(xc->layout))
+                fprintf(f, "        Option \"XkbLayout\" \"%s\"\n", xc->layout);
 
-        if (!isempty(c->x11_model))
-                fprintf(f, "        Option \"XkbModel\" \"%s\"\n", c->x11_model);
+        if (!isempty(xc->model))
+                fprintf(f, "        Option \"XkbModel\" \"%s\"\n", xc->model);
 
-        if (!isempty(c->x11_variant))
-                fprintf(f, "        Option \"XkbVariant\" \"%s\"\n", c->x11_variant);
+        if (!isempty(xc->variant))
+                fprintf(f, "        Option \"XkbVariant\" \"%s\"\n", xc->variant);
 
-        if (!isempty(c->x11_options))
-                fprintf(f, "        Option \"XkbOptions\" \"%s\"\n", c->x11_options);
+        if (!isempty(xc->options))
+                fprintf(f, "        Option \"XkbOptions\" \"%s\"\n", xc->options);
 
         fputs("EndSection\n", f);
 
         r = fflush_sync_and_check(f);
         if (r < 0)
-                goto fail;
+                return r;
 
-        if (rename(temp_path, "/etc/X11/xorg.conf.d/00-keyboard.conf") < 0) {
-                r = -errno;
-                goto fail;
-        }
+        if (rename(temp_path, "/etc/X11/xorg.conf.d/00-keyboard.conf") < 0)
+                return -errno;
 
-        if (stat("/etc/X11/xorg.conf.d/00-keyboard.conf", &st) >= 0)
-                c->x11_mtime = timespec_load(&st.st_mtim);
+        if (stat("/etc/X11/xorg.conf.d/00-keyboard.conf", &c->x11_stat) < 0)
+                return -errno;
 
         return 0;
-
-fail:
-        if (temp_path)
-                (void) unlink(temp_path);
-
-        return r;
 }
 
-static int read_next_mapping(const char* filename,
-                             unsigned min_fields, unsigned max_fields,
-                             FILE *f, unsigned *n, char ***a) {
+static int read_next_mapping(
+                const char *filename,
+                unsigned min_fields,
+                unsigned max_fields,
+                FILE *f,
+                unsigned *n,
+                char ***ret) {
+
         assert(f);
         assert(n);
-        assert(a);
+        assert(ret);
 
         for (;;) {
+                _cleanup_strv_free_ char **b = NULL;
                 _cleanup_free_ char *line = NULL;
                 size_t length;
-                char *l, **b;
+                const char *l;
                 int r;
 
                 r = read_line(f, LONG_LINE_MAX, &line);
@@ -350,95 +562,72 @@ static int read_next_mapping(const char* filename,
 
                 length = strv_length(b);
                 if (length < min_fields || length > max_fields) {
-                        log_error("Invalid line %s:%u, ignoring.", filename, *n);
-                        strv_free(b);
+                        log_debug("Invalid line %s:%u, ignoring.", strna(filename), *n);
                         continue;
 
                 }
 
-                *a = b;
+                *ret = TAKE_PTR(b);
                 return 1;
         }
 
+        *ret = NULL;
         return 0;
 }
 
-int vconsole_convert_to_x11(Context *c) {
+int vconsole_convert_to_x11(const VCContext *vc, X11Context *ret) {
+        _cleanup_fclose_ FILE *f = NULL;
         const char *map;
-        int modified = -1;
+        int r;
 
-        map = systemd_kbd_model_map();
+        assert(vc);
+        assert(ret);
 
-        if (isempty(c->vc_keymap)) {
-                modified =
-                        !isempty(c->x11_layout) ||
-                        !isempty(c->x11_model) ||
-                        !isempty(c->x11_variant) ||
-                        !isempty(c->x11_options);
-
-                context_free_x11(c);
-        } else {
-                _cleanup_fclose_ FILE *f = NULL;
-                unsigned n = 0;
-
-                f = fopen(map, "re");
-                if (!f)
-                        return -errno;
-
-                for (;;) {
-                        _cleanup_strv_free_ char **a = NULL;
-                        int r;
-
-                        r = read_next_mapping(map, 5, UINT_MAX, f, &n, &a);
-                        if (r < 0)
-                                return r;
-                        if (r == 0)
-                                break;
-
-                        if (!streq(c->vc_keymap, a[0]))
-                                continue;
-
-                        if (!streq_ptr(c->x11_layout, empty_or_dash_to_null(a[1])) ||
-                            !streq_ptr(c->x11_model, empty_or_dash_to_null(a[2])) ||
-                            !streq_ptr(c->x11_variant, empty_or_dash_to_null(a[3])) ||
-                            !streq_ptr(c->x11_options, empty_or_dash_to_null(a[4]))) {
-
-                                if (free_and_strdup(&c->x11_layout, empty_or_dash_to_null(a[1])) < 0 ||
-                                    free_and_strdup(&c->x11_model, empty_or_dash_to_null(a[2])) < 0 ||
-                                    free_and_strdup(&c->x11_variant, empty_or_dash_to_null(a[3])) < 0 ||
-                                    free_and_strdup(&c->x11_options, empty_or_dash_to_null(a[4])) < 0)
-                                        return -ENOMEM;
-
-                                modified = true;
-                        }
-
-                        break;
-                }
+        if (isempty(vc->keymap)) {
+                *ret = (X11Context) {};
+                return 0;
         }
 
-        if (modified > 0)
-                log_info("Changing X11 keyboard layout to '%s' model '%s' variant '%s' options '%s'",
-                         strempty(c->x11_layout),
-                         strempty(c->x11_model),
-                         strempty(c->x11_variant),
-                         strempty(c->x11_options));
-        else if (modified < 0)
-                log_notice("X11 keyboard layout was not modified: no conversion found for \"%s\".",
-                           c->vc_keymap);
-        else
-                log_debug("X11 keyboard layout did not need to be modified.");
+        map = systemd_kbd_model_map();
+        f = fopen(map, "re");
+        if (!f)
+                return -errno;
 
-        return modified > 0;
+        for (unsigned n = 0;;) {
+                _cleanup_strv_free_ char **a = NULL;
+
+                r = read_next_mapping(map, 5, UINT_MAX, f, &n, &a);
+                if (r < 0)
+                        return r;
+                if (r == 0) {
+                        *ret = (X11Context) {};
+                        return 0;
+                }
+
+                if (!streq(vc->keymap, a[0]))
+                        continue;
+
+                return x11_context_copy(ret,
+                                     &(X11Context) {
+                                             .layout  = empty_or_dash_to_null(a[1]),
+                                             .model   = empty_or_dash_to_null(a[2]),
+                                             .variant = empty_or_dash_to_null(a[3]),
+                                             .options = empty_or_dash_to_null(a[4]),
+                                     });
+        }
 }
 
-int find_converted_keymap(const char *x11_layout, const char *x11_variant, char **new_keymap) {
-        const char *dir;
+int find_converted_keymap(const X11Context *xc, char **ret) {
         _cleanup_free_ char *n = NULL;
 
-        if (x11_variant)
-                n = strjoin(x11_layout, "-", x11_variant);
+        assert(xc);
+        assert(!isempty(xc->layout));
+        assert(ret);
+
+        if (xc->variant)
+                n = strjoin(xc->layout, "-", xc->variant);
         else
-                n = strdup(x11_layout);
+                n = strdup(xc->layout);
         if (!n)
                 return -ENOMEM;
 
@@ -453,34 +642,32 @@ int find_converted_keymap(const char *x11_layout, const char *x11_variant, char
 
                 uncompressed = access(p, F_OK) == 0;
                 if (uncompressed || access(pz, F_OK) == 0) {
-                        log_debug("Found converted keymap %s at %s",
-                                  n, uncompressed ? p : pz);
-
-                        *new_keymap = TAKE_PTR(n);
+                        log_debug("Found converted keymap %s at %s", n, uncompressed ? p : pz);
+                        *ret = TAKE_PTR(n);
                         return 1;
                 }
         }
 
+        *ret = NULL;
         return 0;
 }
 
-int find_legacy_keymap(Context *c, char **ret) {
+int find_legacy_keymap(const X11Context *xc, char **ret) {
         const char *map;
         _cleanup_fclose_ FILE *f = NULL;
         _cleanup_free_ char *new_keymap = NULL;
-        unsigned n = 0;
         unsigned best_matching = 0;
         int r;
 
-        assert(!isempty(c->x11_layout));
+        assert(xc);
+        assert(!isempty(xc->layout));
 
         map = systemd_kbd_model_map();
-
         f = fopen(map, "re");
         if (!f)
                 return -errno;
 
-        for (;;) {
+        for (unsigned n = 0;;) {
                 _cleanup_strv_free_ char **a = NULL;
                 unsigned matching = 0;
 
@@ -491,34 +678,36 @@ int find_legacy_keymap(Context *c, char **ret) {
                         break;
 
                 /* Determine how well matching this entry is */
-                if (streq(c->x11_layout, a[1]))
-                        /* If we got an exact match, this is best */
+                if (streq(xc->layout, a[1]))
+                        /* If we got an exact match, this is the best */
                         matching = 10;
                 else {
                         /* We have multiple X layouts, look for an
                          * entry that matches our key with everything
                          * but the first layout stripped off. */
-                        if (startswith_comma(c->x11_layout, a[1]))
+                        if (startswith_comma(xc->layout, a[1]))
                                 matching = 5;
-                        else  {
+                        else {
                                 _cleanup_free_ char *x = NULL;
 
                                 /* If that didn't work, strip off the
                                  * other layouts from the entry, too */
-                                x = strndup(a[1], strcspn(a[1], ","));
-                                if (startswith_comma(c->x11_layout, x))
+                                x = strdupcspn(a[1], ",");
+                                if (!x)
+                                        return -ENOMEM;
+                                if (startswith_comma(xc->layout, x))
                                         matching = 1;
                         }
                 }
 
                 if (matching > 0) {
-                        if (isempty(c->x11_model) || streq_ptr(c->x11_model, a[2])) {
+                        if (isempty(xc->model) || streq_ptr(xc->model, a[2])) {
                                 matching++;
 
-                                if (streq_ptr(c->x11_variant, a[3])) {
+                                if (streq_ptr(xc->variant, a[3])) {
                                         matching++;
 
-                                        if (streq_ptr(c->x11_options, a[4]))
+                                        if (streq_ptr(xc->options, a[4]))
                                                 matching++;
                                 }
                         }
@@ -526,8 +715,7 @@ int find_legacy_keymap(Context *c, char **ret) {
 
                 /* The best matching entry so far, then let's save that */
                 if (matching >= MAX(best_matching, 1u)) {
-                        log_debug("Found legacy keymap %s with score %u",
-                                  a[0], matching);
+                        log_debug("Found legacy keymap %s with score %u", a[0], matching);
 
                         if (matching > best_matching) {
                                 best_matching = matching;
@@ -539,18 +727,30 @@ int find_legacy_keymap(Context *c, char **ret) {
                 }
         }
 
-        if (best_matching < 10 && c->x11_layout) {
+        if (best_matching < 10 && !isempty(xc->layout)) {
+                _cleanup_free_ char *l = NULL, *v = NULL, *converted = NULL;
+
                 /* The best match is only the first part of the X11
                  * keymap. Check if we have a converted map which
                  * matches just the first layout.
                  */
-                char *l, *v = NULL, *converted;
 
-                l = strndupa_safe(c->x11_layout, strcspn(c->x11_layout, ","));
-                if (c->x11_variant)
-                        v = strndupa_safe(c->x11_variant,
-                                          strcspn(c->x11_variant, ","));
-                r = find_converted_keymap(l, v, &converted);
+                l = strndup(xc->layout, strcspn(xc->layout, ","));
+                if (!l)
+                        return -ENOMEM;
+
+                if (!isempty(xc->variant)) {
+                        v = strndup(xc->variant, strcspn(xc->variant, ","));
+                        if (!v)
+                                return -ENOMEM;
+                }
+
+                r = find_converted_keymap(
+                                &(X11Context) {
+                                        .layout = l,
+                                        .variant = v,
+                                },
+                                &converted);
                 if (r < 0)
                         return r;
                 if (r > 0)
@@ -558,26 +758,25 @@ int find_legacy_keymap(Context *c, char **ret) {
         }
 
         *ret = TAKE_PTR(new_keymap);
-        return (bool) *ret;
+        return !!*ret;
 }
 
-int find_language_fallback(const char *lang, char **language) {
+int find_language_fallback(const char *lang, char **ret) {
         const char *map;
         _cleanup_fclose_ FILE *f = NULL;
         unsigned n = 0;
+        int r;
 
         assert(lang);
-        assert(language);
+        assert(ret);
 
         map = systemd_language_fallback_map();
-
         f = fopen(map, "re");
         if (!f)
                 return -errno;
 
         for (;;) {
                 _cleanup_strv_free_ char **a = NULL;
-                int r;
 
                 r = read_next_mapping(map, 2, 2, f, &n, &a);
                 if (r <= 0)
@@ -585,56 +784,34 @@ int find_language_fallback(const char *lang, char **language) {
 
                 if (streq(lang, a[0])) {
                         assert(strv_length(a) == 2);
-                        *language = TAKE_PTR(a[1]);
+                        *ret = TAKE_PTR(a[1]);
                         return 1;
                 }
         }
-
-        assert_not_reached();
 }
 
-int x11_convert_to_vconsole(Context *c) {
-        bool modified = false;
+int x11_convert_to_vconsole(const X11Context *xc, VCContext *ret) {
+        _cleanup_free_ char *keymap = NULL;
+        int r;
 
-        if (isempty(c->x11_layout)) {
-                modified =
-                        !isempty(c->vc_keymap) ||
-                        !isempty(c->vc_keymap_toggle);
+        assert(xc);
+        assert(ret);
 
-                context_free_vconsole(c);
-        } else {
-                _cleanup_free_ char *new_keymap = NULL;
-                int r;
-
-                r = find_converted_keymap(c->x11_layout, c->x11_variant, &new_keymap);
-                if (r < 0)
-                        return r;
-                else if (r == 0) {
-                        r = find_legacy_keymap(c, &new_keymap);
-                        if (r < 0)
-                                return r;
-                }
-                if (r == 0)
-                        /* We search for layout-variant match first, but then we also look
-                         * for anything which matches just the layout. So it's accurate to say
-                         * that we couldn't find anything which matches the layout. */
-                        log_notice("No conversion to virtual console map found for \"%s\".",
-                                   c->x11_layout);
-
-                if (!streq_ptr(c->vc_keymap, new_keymap)) {
-                        free_and_replace(c->vc_keymap, new_keymap);
-                        c->vc_keymap_toggle = mfree(c->vc_keymap_toggle);
-                        modified = true;
-                }
+        if (isempty(xc->layout)) {
+                *ret = (VCContext) {};
+                return 0;
         }
 
-        if (modified)
-                log_info("Changing virtual console keymap to '%s' toggle '%s'",
-                         strempty(c->vc_keymap), strempty(c->vc_keymap_toggle));
-        else
-                log_debug("Virtual console keymap was not modified.");
+        r = find_converted_keymap(xc, &keymap);
+        if (r == 0)
+                r = find_legacy_keymap(xc, &keymap);
+        if (r < 0)
+                return r;
 
-        return modified;
+        *ret = (VCContext) {
+                .keymap = TAKE_PTR(keymap),
+        };
+        return 0;
 }
 
 bool locale_gen_check_available(void) {
