@@ -401,14 +401,10 @@ int manager_get_user_by_pid(Manager *m, pid_t pid, User **ret) {
 int manager_get_idle_hint(Manager *m, dual_timestamp *t) {
         Session *s;
         bool idle_hint;
-        dual_timestamp ts;
+        dual_timestamp ts = DUAL_TIMESTAMP_NULL;
 
         assert(m);
 
-        /* Initialize the baseline timestamp with the time the manager got initialized to avoid reporting
-         * unreasonable large idle periods starting with the Unix epoch. */
-        ts = m->init_ts;
-
         idle_hint = !manager_is_inhibited(m, INHIBIT_IDLE, INHIBIT_BLOCK, t, false, false, 0, NULL);
 
         HASHMAP_FOREACH(s, m->sessions) {
@@ -496,7 +492,7 @@ int config_parse_n_autovts(
 static int vt_is_busy(unsigned vtnr) {
         struct vt_stat vt_stat;
         int r;
-        _cleanup_close_ int fd = -1;
+        _cleanup_close_ int fd = -EBADF;
 
         assert(vtnr >= 1);
 
