@@ -38,7 +38,6 @@
 #include "tmpfile-util.h"
 #include "uid-alloc-range.h"
 #include "user-util.h"
-#include "util.h"
 
 #define RELEASE_USEC (20*USEC_PER_SEC)
 
@@ -62,8 +61,8 @@ int session_new(Session **ret, Manager *m, const char *id) {
 
         *s = (Session) {
                 .manager = m,
-                .fifo_fd = -1,
-                .vtfd = -1,
+                .fifo_fd = -EBADF,
+                .vtfd = -EBADF,
                 .audit_id = AUDIT_SESSION_INVALID,
                 .tty_validity = _TTY_VALIDITY_INVALID,
         };
@@ -1269,20 +1268,10 @@ SessionState session_get_state(Session *s) {
 int session_kill(Session *s, KillWho who, int signo) {
         assert(s);
 
-        switch (who) {
+        if (!s->scope)
+                return -ESRCH;
 
-        case KILL_ALL:
-                if (!s->scope)
-                        return -ESRCH;
-
-                return manager_kill_unit(s->manager, s->scope, KILL_ALL, signo, NULL);
-
-        case KILL_LEADER:
-                return RET_NERRNO(kill(s->leader, signo));
-
-        default:
-                assert_not_reached();
-        }
+        return manager_kill_unit(s->manager, s->scope, who, signo, NULL);
 }
 
 static int session_open_vt(Session *s) {
@@ -1407,23 +1396,6 @@ void session_leave_vt(Session *s) {
 
         session_device_pause_all(s);
         r = vt_release(s->vtfd, false);
-        if (r == -EIO) {
-                int vt, old_fd;
-
-                /* It might happen if the controlling process exited before or while we were
-                 * restoring the VT as it would leave the old file-descriptor in a hung-up
-                 * state. In this case let's retry with a fresh handle to the virtual terminal. */
-
-                /* We do a little dance to avoid having the terminal be available
-                 * for reuse before we've cleaned it up. */
-                old_fd = TAKE_FD(s->vtfd);
-
-                vt = session_open_vt(s);
-                safe_close(old_fd);
-
-                if (vt >= 0)
-                        r = vt_release(vt, false);
-        }
         if (r < 0)
                 log_debug_errno(r, "Cannot release VT of session %s: %m", s->id);
 }
