@@ -275,35 +275,25 @@ static int dhcp6_address_acquired(Link *link) {
                         return r;
         }
 
-        return 0;
-}
+        if (link->network->dhcp6_use_hostname) {
+                const char *dhcpname = NULL;
+                _cleanup_free_ char *hostname = NULL;
 
-static int dhcp6_request_hostname(Link *link) {
-        _cleanup_free_ char *hostname = NULL;
-        const char *dhcpname = NULL;
-        int r;
+                (void) sd_dhcp6_lease_get_fqdn(link->dhcp6_lease, &dhcpname);
 
-        assert(link);
-        assert(link->network);
-
-        if (!link->network->dhcp6_use_hostname)
-                return 0;
-
-        r = sd_dhcp6_lease_get_fqdn(link->dhcp6_lease, &dhcpname);
-        if (r == -ENODATA)
-                return 0;
-        if (r < 0)
-                return r;
-
-        r = shorten_overlong(dhcpname, &hostname);
-        if (r < 0)
-                return log_link_warning_errno(link, r, "Unable to shorten overlong DHCP hostname '%s': %m", dhcpname);
-        if (r == 1)
-                log_link_notice(link, "Overlong DHCP hostname received, shortened from '%s' to '%s'", dhcpname, hostname);
-
-        r = manager_set_hostname(link->manager, hostname);
-        if (r < 0)
-                log_link_warning_errno(link, r, "Failed to set transient hostname to '%s', ignoring: %m", hostname);
+                if (dhcpname) {
+                        r = shorten_overlong(dhcpname, &hostname);
+                        if (r < 0)
+                                log_link_warning_errno(link, r, "Unable to shorten overlong DHCP hostname '%s', ignoring: %m", dhcpname);
+                        if (r == 1)
+                                log_link_notice(link, "Overlong DHCP hostname received, shortened from '%s' to '%s'", dhcpname, hostname);
+                }
+                if (hostname) {
+                        r = manager_set_hostname(link->manager, hostname);
+                        if (r < 0)
+                                log_link_error_errno(link, r, "Failed to set transient hostname to '%s': %m", hostname);
+                }
+        }
 
         return 0;
 }
@@ -323,10 +313,6 @@ static int dhcp6_lease_ip_acquired(sd_dhcp6_client *client, Link *link) {
         lease_old = TAKE_PTR(link->dhcp6_lease);
         link->dhcp6_lease = sd_dhcp6_lease_ref(lease);
 
-        r = dhcp6_request_hostname(link);
-        if (r < 0)
-                return r;
-
         r = dhcp6_address_acquired(link);
         if (r < 0)
                 return r;
@@ -607,6 +593,10 @@ static int dhcp6_configure(Link *link) {
         if (r < 0)
                 return log_link_debug_errno(link, r, "DHCPv6 CLIENT: Failed to attach event: %m");
 
+        r = sd_dhcp6_client_attach_device(client, link->dev);
+        if (r < 0)
+                return log_link_debug_errno(link, r, "DHCPv6 CLIENT: Failed to attach device: %m");
+
         r = dhcp6_set_identifier(link, client);
         if (r < 0)
                 return log_link_debug_errno(link, r, "DHCPv6 CLIENT: Failed to set identifier: %m");
@@ -717,6 +707,12 @@ static int dhcp6_configure(Link *link) {
                                             "DHCPv6 CLIENT: Failed to %s rapid commit: %m",
                                             enable_disable(link->network->dhcp6_use_rapid_commit));
 
+        r = sd_dhcp6_client_set_send_release(client, link->network->dhcp6_send_release);
+        if (r < 0)
+                return log_link_debug_errno(link, r,
+                                            "DHCPv6 CLIENT: Failed to %s sending release message on stop: %m",
+                                            enable_disable(link->network->dhcp6_send_release));
+
         link->dhcp6_client = TAKE_PTR(client);
 
         return 0;
