@@ -72,6 +72,10 @@ typedef struct Link {
         sd_device *dev;
         char *driver;
 
+        /* to prevent multiple ethtool calls */
+        bool ethtool_driver_read;
+        bool ethtool_permanent_hw_addr_read;
+
         /* link-local addressing */
         IPv6LinkLocalAddressGenMode ipv6ll_address_gen_mode;
 
@@ -123,7 +127,7 @@ typedef struct Link {
         sd_dhcp_lease *dhcp_lease;
         char *lease_file;
         unsigned dhcp4_messages;
-        bool dhcp4_configured;
+        bool dhcp4_configured:1;
         char *dhcp4_6rd_tunnel_name;
 
         sd_ipv4ll *ipv4ll;
@@ -236,7 +240,6 @@ LinkState link_state_from_string(const char *s) _pure_;
 
 int link_reconfigure_impl(Link *link, bool force);
 int link_reconfigure(Link *link, bool force);
-int link_reconfigure_after_sleep(Link *link);
 
 int manager_udev_process_link(Manager *m, sd_device *device, sd_device_action_t action);
 int manager_rtnl_process_link(sd_netlink *rtnl, sd_netlink_message *message, Manager *m);
