@@ -42,7 +42,6 @@
 #include "string-util.h"
 #include "strv.h"
 #include "tclass.h"
-#include "util.h"
 
 /* Let's assume that anything above this number is a user misconfiguration. */
 #define MAX_NTP_SERVERS 128U
@@ -283,8 +282,10 @@ int network_verify(Network *network) {
                 network->ignore_carrier_loss_usec = USEC_INFINITY;
         }
 
-        if (!network->ignore_carrier_loss_set) /* Set implied default. */
+        if (!network->ignore_carrier_loss_set) {
+                network->ignore_carrier_loss_set = true;
                 network->ignore_carrier_loss_usec = network->configure_without_carrier ? USEC_INFINITY : 0;
+        }
 
         if (IN_SET(network->activation_policy, ACTIVATION_POLICY_DOWN, ACTIVATION_POLICY_ALWAYS_DOWN, ACTIVATION_POLICY_MANUAL)) {
                 if (network->required_for_online < 0 ||
@@ -415,6 +416,7 @@ int network_load_one(Manager *manager, OrderedHashmap **networks, const char *fi
                 .dhcp6_use_rapid_commit = true,
                 .dhcp6_duid.type = _DUID_TYPE_INVALID,
                 .dhcp6_client_start_mode = _DHCP6_CLIENT_START_MODE_INVALID,
+                .dhcp6_send_release = true,
 
                 .dhcp_pd = -1,
                 .dhcp_pd_announce = true,
@@ -479,7 +481,9 @@ int network_load_one(Manager *manager, OrderedHashmap **networks, const char *fi
                 .ipv6_accept_ra_use_onlink_prefix = true,
                 .ipv6_accept_ra_use_mtu = true,
                 .ipv6_accept_ra_route_table = RT_TABLE_MAIN,
-                .ipv6_accept_ra_route_metric = DHCP_ROUTE_METRIC,
+                .ipv6_accept_ra_route_metric_high = IPV6RA_ROUTE_METRIC_HIGH,
+                .ipv6_accept_ra_route_metric_medium = IPV6RA_ROUTE_METRIC_MEDIUM,
+                .ipv6_accept_ra_route_metric_low = IPV6RA_ROUTE_METRIC_LOW,
                 .ipv6_accept_ra_start_dhcp6_client = IPV6_ACCEPT_RA_START_DHCP6_CLIENT_YES,
 
                 .can_termination = -1,
@@ -549,7 +553,7 @@ int network_load_one(Manager *manager, OrderedHashmap **networks, const char *fi
                         CONFIG_PARSE_WARN,
                         network,
                         &network->stats_by_path,
-                        NULL);
+                        &network->dropins);
         if (r < 0)
                 return r; /* config_parse_many() logs internally. */
 
@@ -667,6 +671,7 @@ static Network *network_free(Network *network) {
         free(network->name);
         free(network->filename);
         free(network->description);
+        strv_free(network->dropins);
         hashmap_free(network->stats_by_path);
 
         /* conditions */
