@@ -3,6 +3,7 @@
 #include <linux/rtnetlink.h>
 
 #include "alloc-util.h"
+#include "logarithm.h"
 #include "missing_threads.h"
 #include "networkd-address.h"
 #include "networkd-link.h"
@@ -38,14 +39,6 @@ unsigned routes_max(void) {
         return cached;
 }
 
-static bool route_lifetime_is_valid(const Route *route) {
-        assert(route);
-
-        return
-                route->lifetime_usec == USEC_INFINITY ||
-                route->lifetime_usec > now(CLOCK_BOOTTIME);
-}
-
 static Route *link_find_default_gateway(Link *link, int family, Route *gw) {
         Route *route;
 
@@ -130,8 +123,6 @@ bool gateway_is_ready(Link *link, bool onlink, int family, const union in_addr_u
         SET_FOREACH(route, link->routes) {
                 if (!route_exists(route))
                         continue;
-                if (!route_lifetime_is_valid(route))
-                        continue;
                 if (route->family != family)
                         continue;
                 if (!in_addr_is_set(route->family, &route->dst) && route->dst_prefixlen == 0)
@@ -152,9 +143,9 @@ bool gateway_is_ready(Link *link, bool onlink, int family, const union in_addr_u
                         continue;
                 if (FLAGS_SET(a->flags, IFA_F_NOPREFIXROUTE))
                         continue;
-                if (in_addr_prefix_covers(a->family,
-                                          in_addr_is_set(a->family, &a->in_addr_peer) ? &a->in_addr_peer : &a->in_addr,
-                                          a->prefixlen, gw) > 0)
+                if (in_addr_is_set(a->family, &a->in_addr_peer))
+                        continue;
+                if (in_addr_prefix_covers(family, &a->in_addr, a->prefixlen, gw) > 0)
                         return true;
         }
 
@@ -178,9 +169,6 @@ static int link_address_is_reachable_internal(
                 if (!route_exists(route))
                         continue;
 
-                if (!route_lifetime_is_valid(route))
-                        continue;
-
                 if (route->type != RTN_UNICAST)
                         continue;
 
