@@ -551,24 +551,22 @@ static int link_is_ready_to_set_link(Link *link, Request *req) {
                         m = link->network->vrf->ifindex;
                 }
 
+                if (m == (uint32_t) link->master_ifindex) {
+                        /* The requested master is already set. */
+                        link->master_set = true;
+                        return -EALREADY; /* indicate to cancel the request. */
+                }
+
                 req->userdata = UINT32_TO_PTR(m);
                 break;
         }
         case REQUEST_TYPE_SET_LINK_MTU: {
-                if (ordered_set_contains(link->manager->request_queue,
-                                         &(const Request) {
-                                                 .link = link,
-                                                 .type = REQUEST_TYPE_SET_LINK_IPOIB,
-                                         }))
-                        return false;
+                Request req_ipoib = {
+                        .link = link,
+                        .type = REQUEST_TYPE_SET_LINK_IPOIB,
+                };
 
-                /* Changing FD mode may affect MTU. */
-                if (ordered_set_contains(link->manager->request_queue,
-                                         &(const Request) {
-                                                 .link = link,
-                                                 .type = REQUEST_TYPE_SET_LINK_CAN,
-                                         }))
-                        return false;
+                return !ordered_set_contains(link->manager->request_queue, &req_ipoib);
         }
         default:
                 break;
@@ -584,6 +582,8 @@ static int link_process_set_link(Request *req, Link *link, void *userdata) {
         assert(link);
 
         r = link_is_ready_to_set_link(link, req);
+        if (r == -EALREADY)
+                return 1; /* Cancel the request. */
         if (r <= 0)
                 return r;
 
@@ -840,7 +840,7 @@ int link_request_to_set_master(Link *link) {
 
 int link_request_to_set_mtu(Link *link, uint32_t mtu) {
         const char *origin;
-        uint32_t min_mtu, max_mtu;
+        uint32_t min_mtu;
         Request *req;
         int r;
 
@@ -868,19 +868,10 @@ int link_request_to_set_mtu(Link *link, uint32_t mtu) {
                 mtu = min_mtu;
         }
 
-        max_mtu = link->max_mtu;
-        if (link->iftype == ARPHRD_CAN)
-                /* The maximum MTU may be changed when FD mode is changed.
-                 * See https://docs.kernel.org/networking/can.html#can-fd-flexible-data-rate-driver-support
-                 *   MTU = 16 (CAN_MTU)   => Classical CAN device
-                 *   MTU = 72 (CANFD_MTU) => CAN FD capable device
-                 * So, even if the current maximum is 16, we should not reduce the requested value now. */
-                max_mtu = MAX(max_mtu, 72u);
-
-        if (mtu > max_mtu) {
+        if (mtu > link->max_mtu) {
                 log_link_warning(link, "Reducing the requested MTU %"PRIu32" to the interface's maximum MTU %"PRIu32".",
-                                 mtu, max_mtu);
-                mtu = max_mtu;
+                                 mtu, link->max_mtu);
+                mtu = link->max_mtu;
         }
 
         if (link->mtu == mtu)
