@@ -70,7 +70,7 @@ int network_config_state_to_string_alloc(NetworkConfigState s, char **ret);
                                                                         \
                 t->state = (t->state & ~mask) | (value & mask);         \
         }                                                               \
-        static inline bool name##_exists(const type *t) {               \
+        static inline bool name##_exists(type *t) {                     \
                 assert(t);                                              \
                                                                         \
                 if ((t->state & (NETWORK_CONFIG_STATE_CONFIGURING |     \
@@ -90,7 +90,8 @@ int network_config_state_to_string_alloc(NetworkConfigState s, char **ret);
                                     NETWORK_CONFIG_STATE_REQUESTING,    \
                                     0);                                 \
         }                                                               \
-        static inline bool name##_is_requesting(const type *t) {        \
+        static inline bool name##_is_requesting(type *t) {              \
+                assert(t);                                              \
                 return FLAGS_SET(t->state, NETWORK_CONFIG_STATE_REQUESTING); \
         }                                                               \
         static inline void name##_enter_configuring(type *t) {          \
@@ -114,7 +115,7 @@ int network_config_state_to_string_alloc(NetworkConfigState s, char **ret);
         static inline void name##_unmark(type *t) {                     \
                 name##_update_state(t, NETWORK_CONFIG_STATE_MARKED, 0); \
         }                                                               \
-        static inline bool name##_is_marked(const type *t) {            \
+        static inline bool name##_is_marked(type *t) {                  \
                 assert(t);                                              \
                 return FLAGS_SET(t->state, NETWORK_CONFIG_STATE_MARKED); \
         }                                                               \
@@ -124,10 +125,6 @@ int network_config_state_to_string_alloc(NetworkConfigState s, char **ret);
                                     NETWORK_CONFIG_STATE_REMOVING,      \
                                     NETWORK_CONFIG_STATE_REMOVING);     \
         }                                                               \
-        static inline bool name##_is_removing(const type *t) {          \
-                assert(t);                                              \
-                return FLAGS_SET(t->state, NETWORK_CONFIG_STATE_REMOVING); \
-        }                                                               \
         static inline void name##_enter_removed(type *t) {              \
                 name##_update_state(t,                                  \
                                     NETWORK_CONFIG_STATE_CONFIGURED |   \
