@@ -13,7 +13,6 @@
 #include "process-util.h"
 #include "signal-util.h"
 #include "string-util.h"
-#include "util.h"
 
 static bool arg_skip = false;
 static bool arg_force = false;
@@ -33,9 +32,14 @@ static int parse_proc_cmdline_item(const char *key, const char *value, void *dat
                         arg_skip = true;
                 else
                         log_warning("Invalid quotacheck.mode= parameter '%s'. Ignoring.", value);
+        }
 
-        } else if (streq(key, "forcequotacheck") && !value)
+#if HAVE_SYSV_COMPAT
+        else if (streq(key, "forcequotacheck") && !value) {
+                log_warning("Please use 'quotacheck.mode=force' rather than 'forcequotacheck' on the kernel command line.");
                 arg_force = true;
+        }
+#endif
 
         return 0;
 }
