@@ -12,7 +12,6 @@
 #include "missing_capability.h"
 #include "resolved-bus.h"
 #include "resolved-def.h"
-#include "resolved-dns-stream.h"
 #include "resolved-dns-synthesize.h"
 #include "resolved-dnssd-bus.h"
 #include "resolved-dnssd.h"
@@ -1133,11 +1132,6 @@ static void resolve_service_all_complete(DnsQuery *query) {
         if (r < 0)
                 goto finish;
 
-        if (isempty(type)) {
-                r = reply_method_errorf(q, BUS_ERROR_NO_SUCH_SERVICE, "'%s' does not provide valid service", dns_query_string(q));
-                goto finish;
-        }
-
         r = sd_bus_message_append(
                         reply,
                         "ssst",
@@ -1377,7 +1371,11 @@ static int bus_method_resolve_service(sd_bus_message *message, void *userdata, s
         return 1;
 }
 
-int bus_dns_server_append(sd_bus_message *reply, DnsServer *s, bool with_ifindex, bool extended) {
+int bus_dns_server_append(
+                sd_bus_message *reply,
+                DnsServer *s,
+                bool with_ifindex, /* include "ifindex" field */
+                bool extended) {   /* also include port number and server name */
         int r;
 
         assert(reply);
@@ -1396,7 +1394,11 @@ int bus_dns_server_append(sd_bus_message *reply, DnsServer *s, bool with_ifindex
                 }
         }
 
-        r = sd_bus_message_open_container(reply, 'r', with_ifindex ? (extended ? "iiayqs" : "iiay") : (extended ? "iayqs" : "iay"));
+        r = sd_bus_message_open_container(
+                        reply,
+                        'r',
+                        with_ifindex ? (extended ? "iiayqs" : "iiay") :
+                                       (extended ? "iayqs" : "iay"));
         if (r < 0)
                 return r;
 
@@ -1761,7 +1763,7 @@ static int get_any_link(Manager *m, int ifindex, Link **ret, sd_bus_error *error
 
 static int call_link_method(Manager *m, sd_bus_message *message, sd_bus_message_handler_t handler, sd_bus_error *error) {
         int ifindex, r;
-        Link *l = NULL;  /* avoid false maybe-uninitialized warning */
+        Link *l;
 
         assert(m);
         assert(message);
@@ -1822,7 +1824,7 @@ static int bus_method_get_link(sd_bus_message *message, void *userdata, sd_bus_e
         _cleanup_free_ char *p = NULL;
         Manager *m = ASSERT_PTR(userdata);
         int r, ifindex;
-        Link *l = NULL;  /* avoid false maybe-uninitialized warning */
+        Link *l;
 
         assert(message);
 
@@ -1860,7 +1862,6 @@ static int bus_method_reset_server_features(sd_bus_message *message, void *userd
 
         bus_client_log(message, "server feature reset");
 
-        (void) dns_stream_disconnect_all(m);
         manager_reset_server_features(m);
 
         return sd_bus_reply_method_return(message, NULL);
@@ -2247,15 +2248,9 @@ static int match_prepare_for_sleep(sd_bus_message *message, void *userdata, sd_b
         if (b)
                 return 0;
 
-        log_debug("Coming back from suspend, closing all TCP connections...");
-        (void) dns_stream_disconnect_all(m);
-
-        log_debug("Coming back from suspend, resetting all probed server features...");
-        manager_reset_server_features(m);
-
         log_debug("Coming back from suspend, verifying all RRs...");
-        manager_verify_all(m);
 
+        manager_verify_all(m);
         return 0;
 }
 
