@@ -14,7 +14,6 @@
 #include "strv.h"
 #include "unaligned.h"
 #include "utf8.h"
-#include "util.h"
 
 #define EDNS0_OPT_DO (1<<15)
 
@@ -308,27 +307,13 @@ int dns_packet_validate_query(DnsPacket *p) {
         if (DNS_PACKET_OPCODE(p) != 0)
                 return -EBADMSG;
 
+        if (DNS_PACKET_TC(p))
+                return -EBADMSG;
+
         switch (p->protocol) {
 
-        case DNS_PROTOCOL_DNS:
-                if (DNS_PACKET_TC(p))
-                        return -EBADMSG;
-
-                if (DNS_PACKET_QDCOUNT(p) != 1)
-                        return -EBADMSG;
-
-                if (DNS_PACKET_ANCOUNT(p) > 0)
-                        return -EBADMSG;
-
-                /* Note, in most cases, DNS query packet does not have authority section. But some query
-                 * types, e.g. IXFR, have Authority sections. Hence, unlike the check for LLMNR, we do not
-                 * check DNS_PACKET_NSCOUNT(p) here. */
-                break;
-
         case DNS_PROTOCOL_LLMNR:
-                if (DNS_PACKET_TC(p))
-                        return -EBADMSG;
-
+        case DNS_PROTOCOL_DNS:
                 /* RFC 4795, Section 2.1.1. says to discard all queries with QDCOUNT != 1 */
                 if (DNS_PACKET_QDCOUNT(p) != 1)
                         return -EBADMSG;
@@ -344,9 +329,6 @@ int dns_packet_validate_query(DnsPacket *p) {
                 break;
 
         case DNS_PROTOCOL_MDNS:
-                /* Note, mDNS query may have truncation flag. So, unlike the check for DNS and LLMNR,
-                 * we do not check DNS_PACKET_TC(p) here. */
-
                 /* RFC 6762, Section 18 specifies that messages with non-zero RCODE
                  * must be silently ignored, and that we must ignore the values of
                  * AA, RD, RA, AD, and CD bits. */
@@ -1389,14 +1371,14 @@ int dns_packet_read_uint32(DnsPacket *p, uint32_t *ret, size_t *start) {
 }
 
 int dns_packet_read_string(DnsPacket *p, char **ret, size_t *start) {
-        assert(p);
-
         _cleanup_(rewind_dns_packet) DnsPacketRewinder rewinder = REWINDER_INIT(p);
+        _cleanup_free_ char *t = NULL;
         const void *d;
-        char *t;
         uint8_t c;
         int r;
 
+        assert(p);
+
         r = dns_packet_read_uint8(p, &c, NULL);
         if (r < 0)
                 return r;
@@ -1405,19 +1387,14 @@ int dns_packet_read_string(DnsPacket *p, char **ret, size_t *start) {
         if (r < 0)
                 return r;
 
-        if (memchr(d, 0, c))
+        r = make_cstring(d, c, MAKE_CSTRING_REFUSE_TRAILING_NUL, &t);
+        if (r < 0)
+                return r;
+
+        if (!utf8_is_valid(t))
                 return -EBADMSG;
 
-        t = memdup_suffix0(d, c);
-        if (!t)
-                return -ENOMEM;
-
-        if (!utf8_is_valid(t)) {
-                free(t);
-                return -EBADMSG;
-        }
-
-        *ret = t;
+        *ret = TAKE_PTR(t);
 
         if (start)
                 *start = rewinder.saved_rindex;
@@ -1725,9 +1702,9 @@ int dns_packet_read_rr(
         if (r < 0)
                 return r;
 
-        /* RFC 2181, Section 8, suggests to treat a TTL with the MSB set as a zero TTL. We avoid doing this
-         * for OPT records so that all 8 bits of the extended RCODE may be used .*/
-        if (key->type != DNS_TYPE_OPT && rr->ttl & UINT32_C(0x80000000))
+        /* RFC 2181, Section 8, suggests to
+         * treat a TTL with the MSB set as a zero TTL. */
+        if (rr->ttl & UINT32_C(0x80000000))
                 rr->ttl = 0;
 
         r = dns_packet_read_uint16(p, &rdlength, NULL);
@@ -2365,11 +2342,8 @@ static int dns_packet_extract_answer(DnsPacket *p, DnsAnswer **ret_answer) {
                 } else {
                         DnsAnswerFlags flags = 0;
 
-                        if (p->protocol == DNS_PROTOCOL_MDNS) {
-                                flags |= DNS_ANSWER_REFUSE_TTL_NO_MATCH;
-                                if (!cache_flush)
-                                        flags |= DNS_ANSWER_SHARED_OWNER;
-                        }
+                        if (p->protocol == DNS_PROTOCOL_MDNS && !cache_flush)
+                                flags |= DNS_ANSWER_SHARED_OWNER;
 
                         /* According to RFC 4795, section 2.9. only the RRs from the Answer section shall be
                          * cached. Hence mark only those RRs as cacheable by default, but not the ones from
