@@ -10,6 +10,7 @@
 #include "sd-event.h"
 
 #include "alloc-util.h"
+#include "build.h"
 #include "bus-error.h"
 #include "bus-locator.h"
 #include "bus-map-properties.h"
@@ -636,8 +637,6 @@ static int parse_argv(int argc, char *argv[]) {
 static int transient_unit_set_properties(sd_bus_message *m, UnitType t, char **properties) {
         int r;
 
-        assert(m);
-
         r = sd_bus_message_append(m, "(sv)", "Description", "s", arg_description);
         if (r < 0)
                 return bus_log_create_error(r);
@@ -648,17 +647,6 @@ static int transient_unit_set_properties(sd_bus_message *m, UnitType t, char **p
                         return bus_log_create_error(r);
         }
 
-        r = sd_bus_is_bus_client(sd_bus_message_get_bus(m));
-        if (r < 0)
-                return log_error_errno(r, "Can't determine if bus connection is direct or to broker: %m");
-        if (r > 0) {
-                /* Pin the object as least as long as we are around. Note that AddRef (currently) only works
-                 * if we talk via the bus though. */
-                r = sd_bus_message_append(m, "(sv)", "AddRef", "b", 1);
-                if (r < 0)
-                        return bus_log_create_error(r);
-        }
-
         return bus_append_unit_property_assignment_many(m, t, properties);
 }
 
@@ -735,6 +723,12 @@ static int transient_service_set_properties(sd_bus_message *m, const char *pty_p
         if (r < 0)
                 return r;
 
+        if (arg_wait || arg_stdio != ARG_STDIO_NONE) {
+                r = sd_bus_message_append(m, "(sv)", "AddRef", "b", 1);
+                if (r < 0)
+                        return bus_log_create_error(r);
+        }
+
         if (arg_remain_after_exit) {
                 r = sd_bus_message_append(m, "(sv)", "RemainAfterExit", "b", arg_remain_after_exit);
                 if (r < 0)
@@ -772,17 +766,11 @@ static int transient_service_set_properties(sd_bus_message *m, const char *pty_p
         }
 
         if (pty_path) {
-                _cleanup_close_ int pty_slave = -EBADF;
-
-                pty_slave = open_terminal(pty_path, O_RDWR|O_NOCTTY|O_CLOEXEC);
-                if (pty_slave < 0)
-                        return pty_slave;
-
                 r = sd_bus_message_append(m,
                                           "(sv)(sv)(sv)(sv)",
-                                          "StandardInputFileDescriptor", "h", pty_slave,
-                                          "StandardOutputFileDescriptor", "h", pty_slave,
-                                          "StandardErrorFileDescriptor", "h", pty_slave,
+                                          "StandardInput", "s", "tty",
+                                          "StandardOutput", "s", "tty",
+                                          "StandardError", "s", "tty",
                                           "TTYPath", "s", pty_path);
                 if (r < 0)
                         return bus_log_create_error(r);
@@ -1033,7 +1021,7 @@ static void run_context_check_done(RunContext *c) {
         else
                 done = true;
 
-        if (c->forward && !pty_forward_is_done(c->forward) && done) /* If the service is gone, it's time to drain the output */
+        if (c->forward && done) /* If the service is gone, it's time to drain the output */
                 done = pty_forward_drain(c->forward);
 
         if (done)
@@ -1101,18 +1089,11 @@ static int on_properties_changed(sd_bus_message *m, void *userdata, sd_bus_error
 }
 
 static int pty_forward_handler(PTYForward *f, int rcode, void *userdata) {
-        RunContext *c = ASSERT_PTR(userdata);
+        RunContext *c = userdata;
 
         assert(f);
 
-        if (rcode == -ECANCELED) {
-                log_debug_errno(rcode, "PTY forwarder disconnected.");
-                if (!arg_wait)
-                        return sd_event_exit(c->event, EXIT_SUCCESS);
-
-                /* If --wait is specified, we'll only exit the pty forwarding, but will continue to wait
-                 * for the service to end. If the user hits ^C we'll exit too. */
-        } else if (rcode < 0) {
+        if (rcode < 0) {
                 sd_event_exit(c->event, EXIT_FAILURE);
                 return log_error_errno(rcode, "Error on PTY forwarding logic: %m");
         }
@@ -1129,7 +1110,7 @@ static int start_transient_service(
         _cleanup_(sd_bus_error_free) sd_bus_error error = SD_BUS_ERROR_NULL;
         _cleanup_(bus_wait_for_jobs_freep) BusWaitForJobs *w = NULL;
         _cleanup_free_ char *service = NULL, *pty_path = NULL;
-        _cleanup_close_ int master = -1;
+        _cleanup_close_ int master = -EBADF;
         int r;
 
         assert(bus);
@@ -1755,12 +1736,10 @@ static int run(int argc, char* argv[]) {
 
                 _cleanup_free_ char *command = NULL;
                 r = find_executable(arg_cmdline[0], &command);
-                if (ERRNO_IS_PRIVILEGE(r))
-                        log_debug_errno(r, "Failed to find executable '%s' due to permission problems, leaving path as is: %m", arg_cmdline[0]);
-                else if (r < 0)
+                if (r < 0)
                         return log_error_errno(r, "Failed to find executable %s: %m", arg_cmdline[0]);
-                else
-                        free_and_replace(arg_cmdline[0], command);
+
+                free_and_replace(arg_cmdline[0], command);
         }
 
         if (!arg_description) {
