@@ -10,7 +10,7 @@
 #include "alloc-util.h"
 #include "conf-files.h"
 #include "conf-parser.h"
-#include "def.h"
+#include "constants.h"
 #include "dns-domain.h"
 #include "escape.h"
 #include "ether-addr-util.h"
@@ -155,11 +155,7 @@ static int next_assignment(
         /* Warn about unknown non-extension fields. */
         if (!(flags & CONFIG_PARSE_RELAXED) && !startswith(lvalue, "X-"))
                 log_syntax(unit, LOG_WARNING, filename, line, 0,
-                           "Unknown key '%s'%s%s%s, ignoring.",
-                           lvalue,
-                           section ? " in section [" : "",
-                           strempty(section),
-                           section ? "]" : "");
+                           "Unknown key name '%s' in section '%s', ignoring.", lvalue, section);
 
         return 0;
 }
@@ -215,7 +211,6 @@ static int parse_line(
 
                 if (sections && !nulstr_contains(sections, n)) {
                         bool ignore;
-                        const char *t;
 
                         ignore = (flags & CONFIG_PARSE_RELAXED) || startswith(n, "X-");
 
@@ -467,7 +462,7 @@ int hashmap_put_stats_by_path(Hashmap **stats_by_path, const char *path, const s
                 return -ENOMEM;
 
         path_copy = strdup(path);
-        if (!path_copy)
+        if (!path)
                 return -ENOMEM;
 
         r = hashmap_put(*stats_by_path, path_copy, st_copy);
@@ -694,10 +689,6 @@ int config_get_stats_by_path(
                         return -errno;
                 }
 
-                /* Skipping an empty file. */
-                if (null_or_empty(&st))
-                        continue;
-
                 r = hashmap_put_stats_by_path(&stats_by_path, *f, &st);
                 if (r < 0)
                         return r;
