@@ -51,8 +51,6 @@ void fdset_close(FDSet *s) {
         void *p;
 
         while ((p = set_steal_first(MAKE_SET(s)))) {
-                int fd = PTR_TO_FD(p);
-
                 /* Valgrind's fd might have ended up in this set here, due to fdset_new_fill(). We'll ignore
                  * all failures here, so that the EBADFD that valgrind will return us on close() doesn't
                  * influence us */
@@ -61,14 +59,8 @@ void fdset_close(FDSet *s) {
                  * which has no effect at all, since they are only duplicates. So don't be surprised about
                  * these log messages. */
 
-                if (DEBUG_LOGGING) {
-                        _cleanup_free_ char *path = NULL;
-
-                        (void) fd_get_path(fd, &path);
-                        log_debug("Closing set fd %i (%s)", fd, strna(path));
-                }
-
-                (void) close_nointr(fd);
+                log_debug("Closing set fd %i", PTR_TO_FD(p));
+                (void) close_nointr(PTR_TO_FD(p));
         }
 }
 
@@ -141,8 +133,8 @@ int fdset_new_fill(
 
         assert(_s);
 
-        /* Creates an fdset and fills in all currently open file descriptors. Also set all collected fds
-         * to CLOEXEC. */
+        /* Creates an fdset and fills in all currently open file
+         * descriptors. */
 
         d = opendir("/proc/self/fd");
         if (!d)
@@ -155,7 +147,7 @@ int fdset_new_fill(
         }
 
         FOREACH_DIRENT(de, d, return -errno) {
-                int fd = -1;
+                int fd = -EBADF;
 
                 r = safe_atoi(de->d_name, &fd);
                 if (r < 0)
@@ -173,7 +165,6 @@ int fdset_new_fill(
                         /* If user asked for that filter by O_CLOEXEC. This is useful so that fds that have
                          * been passed in can be collected and fds which have been created locally can be
                          * ignored, under the assumption that only the latter have O_CLOEXEC set. */
-
                         fl = fcntl(fd, F_GETFD);
                         if (fl < 0)
                                 return -errno;
@@ -182,13 +173,6 @@ int fdset_new_fill(
                                 continue;
                 }
 
-                /* We need to set CLOEXEC manually only if we're collecting non-CLOEXEC fds. */
-                if (filter_cloexec <= 0) {
-                        r = fd_cloexec(fd, true);
-                        if (r < 0)
-                                return r;
-                }
-
                 r = fdset_put(s, fd);
                 if (r < 0)
                         goto finish;
