@@ -17,7 +17,6 @@
 #include "proc-cmdline.h"
 #include "string-table.h"
 #include "string-util.h"
-#include "util.h"
 
 static int sethostname_idempotent_full(const char *s, bool really) {
         struct utsname u;
@@ -151,7 +150,7 @@ int hostname_setup(bool really) {
         if (r < 0)
                 log_warning_errno(r, "Failed to retrieve system hostname from kernel command line, ignoring: %m");
         else if (r > 0) {
-                if (hostname_is_valid(b, VALID_HOSTNAME_TRAILING_DOT)) {
+                if (hostname_is_valid(b, true)) {
                         hn = b;
                         source = HOSTNAME_TRANSIENT;
                 } else  {
