@@ -21,6 +21,7 @@
 #include "strv.h"
 #include "tmpfile-util.h"
 
+static const char *default_hwdb_bin_dir = "/etc/udev";
 static const char * const conf_file_dirs[] = {
         "/etc/udev/hwdb.d",
         UDEVLIBEXECDIR "/hwdb.d",
@@ -193,7 +194,7 @@ static int trie_insert(struct trie *trie, struct trie_node *node, const char *se
 
         for (size_t i = 0;; i++) {
                 size_t p;
-                char c;
+                uint8_t c;
                 struct trie_node *child;
 
                 for (p = 0; (c = trie->strings->buf[node->prefix_off + p]); p++) {
@@ -469,7 +470,7 @@ static int import_file(struct trie *trie, const char *filename, uint16_t file_pr
         _cleanup_fclose_ FILE *f = NULL;
         _cleanup_strv_free_ char **match_list = NULL;
         uint32_t line_number = 0;
-        int r;
+        int r, err;
 
         f = fopen(filename, "re");
         if (!f)
@@ -509,23 +510,24 @@ static int import_file(struct trie *trie, const char *filename, uint16_t file_pr
                                 break;
 
                         if (line[0] == ' ') {
-                                log_syntax(NULL, LOG_WARNING, filename, line_number, 0,
-                                           "Match expected but got indented property \"%s\", ignoring line.", line);
+                                r = log_syntax(NULL, LOG_WARNING, filename, line_number, SYNTHETIC_ERRNO(EINVAL),
+                                               "Match expected but got indented property \"%s\", ignoring line.", line);
                                 break;
                         }
 
                         /* start of record, first match */
                         state = HW_MATCH;
 
-                        r = strv_extend(&match_list, line);
-                        if (r < 0)
-                                return r;
+                        err = strv_extend(&match_list, line);
+                        if (err < 0)
+                                return err;
+
                         break;
 
                 case HW_MATCH:
                         if (len == 0) {
-                                log_syntax(NULL, LOG_WARNING, filename, line_number, 0,
-                                           "Property expected, ignoring record with no properties.");
+                                r = log_syntax(NULL, LOG_WARNING, filename, line_number, SYNTHETIC_ERRNO(EINVAL),
+                                               "Property expected, ignoring record with no properties.");
                                 state = HW_NONE;
                                 match_list = strv_free(match_list);
                                 break;
@@ -533,15 +535,18 @@ static int import_file(struct trie *trie, const char *filename, uint16_t file_pr
 
                         if (line[0] != ' ') {
                                 /* another match */
-                                r = strv_extend(&match_list, line);
-                                if (r < 0)
-                                        return r;
+                                err = strv_extend(&match_list, line);
+                                if (err < 0)
+                                        return err;
+
                                 break;
                         }
 
                         /* first data */
                         state = HW_DATA;
-                        (void) insert_data(trie, match_list, line, filename, file_priority, line_number, compat);
+                        err = insert_data(trie, match_list, line, filename, file_priority, line_number, compat);
+                        if (err < 0)
+                                r = err;
                         break;
 
                 case HW_DATA:
@@ -553,14 +558,16 @@ static int import_file(struct trie *trie, const char *filename, uint16_t file_pr
                         }
 
                         if (line[0] != ' ') {
-                                log_syntax(NULL, LOG_WARNING, filename, line_number, 0,
-                                           "Property or empty line expected, got \"%s\", ignoring record.", line);
+                                r = log_syntax(NULL, LOG_WARNING, filename, line_number, SYNTHETIC_ERRNO(EINVAL),
+                                               "Property or empty line expected, got \"%s\", ignoring record.", line);
                                 state = HW_NONE;
                                 match_list = strv_free(match_list);
                                 break;
                         }
 
-                        (void) insert_data(trie, match_list, line, filename, file_priority, line_number, compat);
+                        err = insert_data(trie, match_list, line, filename, file_priority, line_number, compat);
+                        if (err < 0)
+                                r = err;
                         break;
                 };
         }
@@ -584,10 +591,6 @@ int hwdb_update(const char *root, const char *hwdb_bin_dir, bool strict, bool co
          * source. If true, then hwdb.bin will be created without the information. systemd-hwdb command
          * should set the argument false, and 'udevadm hwdb' command should set it true. */
 
-        hwdb_bin = path_join(root, hwdb_bin_dir ?: "/etc/udev", "hwdb.bin");
-        if (!hwdb_bin)
-                return -ENOMEM;
-
         trie = new0(struct trie, 1);
         if (!trie)
                 return -ENOMEM;
@@ -608,18 +611,6 @@ int hwdb_update(const char *root, const char *hwdb_bin_dir, bool strict, bool co
         if (err < 0)
                 return log_error_errno(err, "Failed to enumerate hwdb files: %m");
 
-        if (strv_isempty(files)) {
-                if (unlink(hwdb_bin) < 0) {
-                        if (errno != ENOENT)
-                                return log_error_errno(errno, "Failed to remove compiled hwdb database %s: %m", hwdb_bin);
-
-                        log_info("No hwdb files found, skipping.");
-                } else
-                        log_info("No hwdb files found, compiled hwdb database %s removed.", hwdb_bin);
-
-                return 0;
-        }
-
         STRV_FOREACH(f, files) {
                 log_debug("Reading file \"%s\"", *f);
                 err = import_file(trie, *f, file_priority++, compat);
@@ -643,6 +634,10 @@ int hwdb_update(const char *root, const char *hwdb_bin_dir, bool strict, bool co
         log_debug("strings dedup'ed: %8zu bytes (%8zu)",
                   trie->strings->dedup_len, trie->strings->dedup_count);
 
+        hwdb_bin = path_join(root, hwdb_bin_dir ?: default_hwdb_bin_dir, "hwdb.bin");
+        if (!hwdb_bin)
+                return -ENOMEM;
+
         (void) mkdir_parents_label(hwdb_bin, 0755);
         err = trie_store(trie, hwdb_bin, compat);
         if (err < 0)
@@ -657,7 +652,7 @@ int hwdb_update(const char *root, const char *hwdb_bin_dir, bool strict, bool co
 
 int hwdb_query(const char *modalias, const char *root) {
         _cleanup_(sd_hwdb_unrefp) sd_hwdb *hwdb = NULL;
-        const char *key, *value, *p;
+        const char *key, *value;
         int r;
 
         assert(modalias);
@@ -687,7 +682,6 @@ int hwdb_query(const char *modalias, const char *root) {
 
 bool hwdb_should_reload(sd_hwdb *hwdb) {
         bool found = false;
-        const char* p;
         struct stat st;
 
         if (!hwdb)
