@@ -38,6 +38,7 @@ typedef enum NamingSchemeFlags {
         NAMING_XEN_VIF                   = 1 << 13, /* Generate names for Xen netfront devices */
         NAMING_BRIDGE_MULTIFUNCTION_SLOT = 1 << 14, /* Use PCI hotplug slot information associated with bridge, but only if PCI device is multifunction */
         NAMING_DEVICETREE_ALIASES        = 1 << 15, /* Generate names from devicetree aliases */
+        NAMING_USB_HOST                  = 1 << 16, /* Generate names for usb host */
 
         /* And now the masks that combine the features above */
         NAMING_V238 = 0,
@@ -51,6 +52,7 @@ typedef enum NamingSchemeFlags {
         NAMING_V250 = NAMING_V249 | NAMING_XEN_VIF,
         NAMING_V251 = NAMING_V250 | NAMING_BRIDGE_MULTIFUNCTION_SLOT,
         NAMING_V252 = NAMING_V251 | NAMING_DEVICETREE_ALIASES,
+        NAMING_V253 = NAMING_V252 | NAMING_USB_HOST,
 
         EXTRA_NET_NAMING_SCHEMES
 
