@@ -25,7 +25,6 @@
 #include "string-util.h"
 #include "strv.h"
 #include "terminal-util.h"
-#include "util.h"
 
 static pid_t pager_pid = 0;
 
@@ -83,24 +82,8 @@ static int no_quit_on_interrupt(int exe_name_fd, const char *less_opts) {
         return r;
 }
 
-static bool running_with_escalated_privileges(void) {
-        int r;
-
-        if (getenv("SUDO_UID"))
-                return true;
-
-        uid_t uid;
-        r = sd_pid_get_owner_uid(0, &uid);
-        if (r < 0) {
-                log_debug_errno(r, "sd_pid_get_owner_uid() failed, enabling pager secure mode: %m");
-                return true;
-        }
-
-        return uid != geteuid();
-}
-
 void pager_open(PagerFlags flags) {
-        _cleanup_close_pair_ int fd[2] = { -1, -1 }, exe_name_pipe[2] = { -1, -1 };
+        _cleanup_close_pair_ int fd[2] = PIPE_EBADF, exe_name_pipe[2] = PIPE_EBADF;
         _cleanup_strv_free_ char **pager_args = NULL;
         _cleanup_free_ char *l = NULL;
         const char *pager, *less_opts;
@@ -194,9 +177,16 @@ void pager_open(PagerFlags flags) {
                  * know to be good. */
                 int use_secure_mode = getenv_bool_secure("SYSTEMD_PAGERSECURE");
                 bool trust_pager = use_secure_mode >= 0;
-                if (use_secure_mode == -ENXIO)
-                        use_secure_mode = running_with_escalated_privileges();
-                else if (use_secure_mode < 0) {
+                if (use_secure_mode == -ENXIO) {
+                        uid_t uid;
+
+                        r = sd_pid_get_owner_uid(0, &uid);
+                        if (r < 0)
+                                log_debug_errno(r, "sd_pid_get_owner_uid() failed, enabling pager secure mode: %m");
+
+                        use_secure_mode = r < 0 || uid != geteuid();
+
+                } else if (use_secure_mode < 0) {
                         log_warning_errno(use_secure_mode, "Unable to parse $SYSTEMD_PAGERSECURE, assuming true: %m");
                         use_secure_mode = true;
                 }
