@@ -358,7 +358,6 @@ const SyscallFilterSet syscall_filter_sets[_SYSCALL_FILTER_SET_MAX] = {
                 "prlimit64\0"
                 "restart_syscall\0"
                 "riscv_flush_icache\0"
-                "riscv_hwprobe\0"
                 "rseq\0"
                 "rt_sigreturn\0"
                 "sched_getaffinity\0"
@@ -370,7 +369,6 @@ const SyscallFilterSet syscall_filter_sets[_SYSCALL_FILTER_SET_MAX] = {
                 "sigreturn\0"
                 "time\0"
                 "ugetrlimit\0"
-                "uretprobe\0"
         },
         [SYSCALL_FILTER_SET_AIO] = {
                 .name = "@aio",
@@ -469,7 +467,6 @@ const SyscallFilterSet syscall_filter_sets[_SYSCALL_FILTER_SET_MAX] = {
                 "fchdir\0"
                 "fchmod\0"
                 "fchmodat\0"
-                "fchmodat2\0"
                 "fcntl\0"
                 "fcntl64\0"
                 "fgetxattr\0"
@@ -855,7 +852,6 @@ const SyscallFilterSet syscall_filter_sets[_SYSCALL_FILTER_SET_MAX] = {
                 .name = "@sync",
                 .help = "Synchronize files and memory to storage",
                 .value =
-                /* Please also update the list in seccomp_suppress_sync(). */
                 "fdatasync\0"
                 "fsync\0"
                 "msync\0"
@@ -884,7 +880,6 @@ const SyscallFilterSet syscall_filter_sets[_SYSCALL_FILTER_SET_MAX] = {
                 "@signal\0"
                 "@sync\0"
                 "@timer\0"
-                "arm_fadvise64_64\0"
                 "capget\0"
                 "capset\0"
                 "copy_file_range\0"
@@ -1047,7 +1042,6 @@ static int add_syscall_filter_set(
                 bool log_missing,
                 char ***added) {
 
-        const char *sys;
         int r;
 
         /* Any syscalls that are handled are added to the *added strv. It needs to be initialized. */
@@ -1177,7 +1171,6 @@ int seccomp_parse_syscall_filter(
 
         if (name[0] == '@') {
                 const SyscallFilterSet *set;
-                const char *i;
 
                 set = syscall_filter_set_find(name);
                 if (!set) {
@@ -1917,7 +1910,6 @@ int parse_syscall_archs(char **l, Set **ret_archs) {
 }
 
 int seccomp_filter_set_add(Hashmap *filter, bool add, const SyscallFilterSet *set) {
-        const char *i;
         int r;
 
         assert(set);
@@ -2034,7 +2026,7 @@ int seccomp_protect_hostname(void) {
 static int seccomp_restrict_sxid(scmp_filter_ctx seccomp, mode_t m) {
         /* Checks the mode_t parameter of the following system calls:
          *
-         *       → chmod() + fchmod() + fchmodat() + fchmodat2()
+         *       → chmod() + fchmod() + fchmodat()
          *       → open() + creat() + openat()
          *       → mkdir() + mkdirat()
          *       → mknod() + mknodat()
@@ -2077,28 +2069,6 @@ static int seccomp_restrict_sxid(scmp_filter_ctx seccomp, mode_t m) {
         else
                 any = true;
 
-#if defined(__SNR_fchmodat2)
-        r = seccomp_rule_add_exact(
-                        seccomp,
-                        SCMP_ACT_ERRNO(EPERM),
-                        SCMP_SYS(fchmodat2),
-                        1,
-                        SCMP_A2(SCMP_CMP_MASKED_EQ, m, m));
-#else
-        /* It looks like this libseccomp does not know about fchmodat2().
-         * Pretend the fchmodat2() system call is not supported at all,
-         * regardless of the kernel version. */
-        r = seccomp_rule_add_exact(
-                        seccomp,
-                        SCMP_ACT_ERRNO(ENOSYS),
-                        __NR_fchmodat2,
-                        0);
-#endif
-        if (r < 0)
-                log_debug_errno(r, "Failed to add filter for fchmodat2: %m");
-        else
-                any = true;
-
         r = seccomp_rule_add_exact(
                         seccomp,
                         SCMP_ACT_ERRNO(EPERM),
@@ -2333,14 +2303,11 @@ int seccomp_suppress_sync(void) {
         uint32_t arch;
         int r;
 
-        /* This behaves slightly differently from SystemCallFilter=~@sync:0, in that negative fds (which
-         * we can determine to be invalid) are still refused with EBADF. See #34478.
-         *
-         * Additionally, O_SYNC/O_DSYNC are masked. */
+        /* This is mostly identical to SystemCallFilter=~@sync:0, but simpler to use, and separately
+         * manageable, and also masks O_SYNC/O_DSYNC */
 
         SECCOMP_FOREACH_LOCAL_ARCH(arch) {
                 _cleanup_(seccomp_releasep) scmp_filter_ctx seccomp = NULL;
-                const char *c;
 
                 r = seccomp_init_for_arch(&seccomp, arch, SCMP_ACT_ALLOW);
                 if (r < 0)
@@ -2355,21 +2322,11 @@ int seccomp_suppress_sync(void) {
                                 continue;
                         }
 
-                        if (STR_IN_SET(c, "fdatasync", "fsync", "sync_file_range", "sync_file_range2", "syncfs"))
-                                r = seccomp_rule_add_exact(
-                                                seccomp,
-                                                SCMP_ACT_ERRNO(0), /* success → we want this to be a NOP after all */
-                                                id,
-                                                1,
-                                                SCMP_A0(SCMP_CMP_LE, INT_MAX)); /* The rule handles arguments in unsigned. Hence, this
-                                                                                 * means non-negative fd matches the rule, and the negative
-                                                                                 * fd passed to the syscall (then it fails with EBADF). */
-                        else
-                                r = seccomp_rule_add_exact(
-                                                seccomp,
-                                                SCMP_ACT_ERRNO(0), /* success → we want this to be a NOP after all */
-                                                id,
-                                                0);
+                        r = seccomp_rule_add_exact(
+                                        seccomp,
+                                        SCMP_ACT_ERRNO(0), /* success → we want this to be a NOP after all */
+                                        id,
+                                        0);
                         if (r < 0)
                                 log_debug_errno(r, "Failed to add filter for system call %s, ignoring: %m", c);
                 }
