@@ -21,7 +21,7 @@
 #include "blockdev-util.h"
 #include "btrfs-util.h"
 #include "conf-parser.h"
-#include "def.h"
+#include "constants.h"
 #include "device-private.h"
 #include "device-util.h"
 #include "devnum-util.h"
@@ -168,7 +168,7 @@ static int battery_enumerator_new(sd_device_enumerator **ret) {
         return 0;
 }
 
-static int get_capacity_by_name(Hashmap *capacities_by_name, const char *name) {
+int get_capacity_by_name(Hashmap *capacities_by_name, const char *name) {
         void *p;
 
         assert(capacities_by_name);
@@ -536,7 +536,6 @@ int get_total_suspend_interval(Hashmap *last_capacity, usec_t *ret) {
 int battery_trip_point_alarm_exists(void) {
         _cleanup_(sd_device_enumerator_unrefp) sd_device_enumerator *e = NULL;
         sd_device *dev;
-        bool has_battery = false;
         int r;
 
         r = battery_enumerator_new(&e);
@@ -544,25 +543,21 @@ int battery_trip_point_alarm_exists(void) {
                 return log_debug_errno(r, "Failed to initialize battery enumerator: %m");
 
         FOREACH_DEVICE(e, dev) {
-                const char *alarm_attr;
-                int has_alarm;
+                int battery_alarm;
+                const char *s;
 
-                has_battery = true;
-
-                r = sd_device_get_sysattr_value(dev, "alarm", &alarm_attr);
+                r = sd_device_get_sysattr_value(dev, "alarm", &s);
                 if (r < 0)
-                        return log_device_debug_errno(dev, r, "Failed to read battery alarm attribute: %m");
+                        return log_device_debug_errno(dev, r, "Failed to read battery alarm: %m");
 
-                r = safe_atoi(alarm_attr, &has_alarm);
+                r = safe_atoi(s, &battery_alarm);
                 if (r < 0)
-                        return log_device_debug_errno(dev, r,
-                                                      "Failed to parse battery alarm attribute '%s': %m",
-                                                      alarm_attr);
-                if (has_alarm <= 0)
+                        return log_device_debug_errno(dev, r, "Failed to parse battery alarm: %m");
+                if (battery_alarm <= 0)
                         return false;
         }
 
-        return has_battery;
+        return true;
 }
 
 /* Return true if wakeup type is APM timer */
@@ -728,7 +723,7 @@ static int swap_device_to_device_id(const SwapEntry *swap, dev_t *ret_dev) {
  * filesystems, a debug message is logged and ret_offset is set to UINT64_MAX.
  */
 static int calculate_swap_file_offset(const SwapEntry *swap, uint64_t *ret_offset) {
-        _cleanup_close_ int fd = -1;
+        _cleanup_close_ int fd = -EBADF;
         _cleanup_free_ struct fiemap *fiemap = NULL;
         struct stat sb;
         int r;
