@@ -188,7 +188,7 @@ int specifier_machine_id(char specifier, const void *data, const char *root, con
         assert(ret);
 
         if (root) {
-                _cleanup_close_ int fd = -1;
+                _cleanup_close_ int fd = -EBADF;
 
                 fd = chase_symlinks_and_open("/etc/machine-id", root, CHASE_PREFIX_ROOT, O_RDONLY|O_CLOEXEC|O_NOCTTY, NULL);
                 if (fd < 0)
@@ -294,7 +294,7 @@ int specifier_architecture(char specifier, const void *data, const char *root, c
  * installation. */
 
 static int parse_os_release_specifier(const char *root, const char *id, char **ret) {
-        _cleanup_free_ char *v = NULL;
+        char *v = NULL;
         int r;
 
         assert(ret);
@@ -303,7 +303,7 @@ static int parse_os_release_specifier(const char *root, const char *id, char **r
         if (r >= 0)
                 /* parse_os_release() calls parse_env_file() which only sets the return value for
                  * entries found. Let's make sure we set the return value in all cases. */
-                *ret = TAKE_PTR(v);
+                *ret = v;
 
         /* Translate error for missing os-release file to EUNATCH. */
         return r == -ENOENT ? -EUNATCH : r;
