@@ -6,13 +6,6 @@
 #include <sys/mman.h>
 #include <sys/mount.h>
 #include <sys/wait.h>
-#include <util.h>
-
-/* When we include libgen.h because we need dirname() we immediately
- * undefine basename() since libgen.h defines it as a macro to the POSIX
- * version which is really broken. We prefer GNU basename(). */
-#include <libgen.h>
-#undef basename
 
 #include "sd-bus.h"
 
@@ -35,7 +28,6 @@
 #include "strv.h"
 #include "tests.h"
 #include "tmpfile-util.h"
-#include "uid-range.h"
 
 char* setup_fake_runtime_dir(void) {
         char t[] = "/tmp/fake-xdg-runtime-XXXXXX", *p;
@@ -126,8 +118,20 @@ void test_setup_logging(int level) {
         log_open();
 }
 
+int log_tests_skipped(const char *message) {
+        log_notice("%s: %s, skipping tests.",
+                   program_invocation_short_name, message);
+        return EXIT_TEST_SKIP;
+}
+
+int log_tests_skipped_errno(int r, const char *message) {
+        log_notice_errno(r, "%s: %s, skipping tests: %m",
+                         program_invocation_short_name, message);
+        return EXIT_TEST_SKIP;
+}
+
 int write_tmpfile(char *pattern, const char *contents) {
-        _cleanup_close_ int fd = -1;
+        _cleanup_close_ int fd = -EBADF;
 
         assert(pattern);
         assert(contents);
@@ -173,20 +177,6 @@ bool have_namespaces(void) {
         assert_not_reached();
 }
 
-bool userns_has_single_user(void) {
-        _cleanup_(uid_range_freep) UidRange *uidrange = NULL;
-
-        /* Check if we're in a user namespace with only a single user mapped in. We special case this
-         * scenario in a few tests because it's the only kind of namespace that can be created unprivileged
-         * and as such happens more often than not, so we make sure to deal with it so that all tests pass
-         * in such environments. */
-
-        if (uid_range_load_userns(&uidrange, NULL) < 0)
-                return false;
-
-        return uidrange->n_entries == 1 && uidrange->entries[0].nr == 1;
-}
-
 bool can_memlock(void) {
         /* Let's see if we can mlock() a larger blob of memory. BPF programs are charged against
          * RLIMIT_MEMLOCK, hence let's first make sure we can lock memory at all, and skip the test if we
