@@ -18,11 +18,11 @@
 #include "sd-messages.h"
 
 #include "btrfs-util.h"
+#include "build.h"
 #include "bus-error.h"
 #include "bus-locator.h"
 #include "bus-util.h"
-#include "def.h"
-#include "efivars.h"
+#include "constants.h"
 #include "exec-util.h"
 #include "fd-util.h"
 #include "fileio.h"
@@ -38,7 +38,6 @@
 #include "string-util.h"
 #include "strv.h"
 #include "time-util.h"
-#include "util.h"
 
 #define DEFAULT_HIBERNATE_DELAY_USEC_NO_BATTERY (2 * USEC_PER_HOUR)
 
@@ -225,12 +224,8 @@ static int execute(
                                * do it ourselves then. > 0 means: kernel already had a configured hibernation
                                * location which we shouldn't touch. */
                         r = write_hibernate_location_info(hibernate_location);
-                        if (r < 0) {
-                                if (is_efi_boot())
-                                        (void) efi_set_variable(EFI_SYSTEMD_VARIABLE(HibernateLocation), NULL, 0);
-
+                        if (r < 0)
                                 return log_error_errno(r, "Failed to prepare for hibernation: %m");
-                        }
                 }
 
                 r = write_mode(modes);
@@ -283,7 +278,7 @@ static int custom_timer_suspend(const SleepConfig *sleep_config) {
 
         while (battery_is_discharging_and_low() == 0) {
                 _cleanup_hashmap_free_ Hashmap *last_capacity = NULL, *current_capacity = NULL;
-                _cleanup_close_ int tfd = -1;
+                _cleanup_close_ int tfd = -EBADF;
                 struct itimerspec ts = {};
                 usec_t suspend_interval;
                 bool woken_by_timer;
