@@ -10,13 +10,13 @@
 #include "sd-daemon.h"
 
 #include "alloc-util.h"
+#include "build.h"
 #include "bus-internal.h"
 #include "bus-util.h"
 #include "errno-util.h"
 #include "io-util.h"
 #include "log.h"
 #include "main-func.h"
-#include "util.h"
 #include "version.h"
 
 #define DEFAULT_BUS_PATH "unix:path=/run/dbus/system_bus_socket"
@@ -183,16 +183,10 @@ static int run(int argc, char *argv[]) {
                 assert_cc(sizeof(usec_t) == sizeof(uint64_t));
 
                 r = sd_bus_process(a, &m);
-                if (r < 0) {
-                        if (ERRNO_IS_DISCONNECT(r)) /* Treat 'connection reset by peer' as clean exit condition */
-                                break;
-
+                if (r < 0)
                         return log_error_errno(r, "Failed to process bus a: %m");
-                }
-                if (m) {
-                        if (sd_bus_message_is_signal(m, "org.freedesktop.DBus.Local", "Disconnected"))
-                                break;
 
+                if (m) {
                         r = sd_bus_send(b, m, NULL);
                         if (r < 0)
                                 return log_error_errno(r, "Failed to send message: %m");
@@ -205,14 +199,12 @@ static int run(int argc, char *argv[]) {
                 if (r < 0) {
                         /* treat 'connection reset by peer' as clean exit condition */
                         if (ERRNO_IS_DISCONNECT(r))
-                                break;
+                                return 0;
 
                         return log_error_errno(r, "Failed to process bus: %m");
                 }
-                if (m) {
-                        if (sd_bus_message_is_signal(m, "org.freedesktop.DBus.Local", "Disconnected"))
-                                break;
 
+                if (m) {
                         r = sd_bus_send(a, m, NULL);
                         if (r < 0)
                                 return log_error_errno(r, "Failed to send message: %m");
@@ -244,14 +236,17 @@ static int run(int argc, char *argv[]) {
                 t = usec_sub_unsigned(MIN(timeout_a, timeout_b), now(CLOCK_MONOTONIC));
 
                 struct pollfd p[3] = {
-                        { .fd = fd,     .events = events_a           },
-                        { .fd = in_fd,  .events = events_b & POLLIN  },
-                        { .fd = out_fd, .events = events_b & POLLOUT },
+                        { .fd = fd,            .events = events_a           },
+                        { .fd = STDIN_FILENO,  .events = events_b & POLLIN  },
+                        { .fd = STDOUT_FILENO, .events = events_b & POLLOUT },
                 };
 
                 r = ppoll_usec(p, ELEMENTSOF(p), t);
-                if (r < 0)
+                if (r < 0) {
+                        if (ERRNO_IS_TRANSIENT(r)) /* don't be bothered by signals, i.e. EINTR */
+                                continue;
                         return log_error_errno(r, "ppoll() failed: %m");
+                }
         }
 
         return 0;
