@@ -10,7 +10,6 @@
 #include "special.h"
 #include "string-util.h"
 #include "unit-file.h"
-#include "util.h"
 
 /*
  * Implements the logic described in systemd.offline-updates(7).
@@ -20,14 +19,12 @@ static const char *arg_dest = NULL;
 
 static int generate_symlink(void) {
         const char *p = NULL;
-        int r;
 
-        r = laccess("/system-update", F_OK);
-        if (r < 0) {
-                if (r == -ENOENT)
+        if (laccess("/system-update", F_OK) < 0) {
+                if (errno == ENOENT)
                         return 0;
 
-                log_error_errno(r, "Failed to check for system update: %m");
+                log_error_errno(errno, "Failed to check for system update: %m");
                 return -EINVAL;
         }
 
