@@ -19,8 +19,8 @@
 #include "terminal-util.h"
 #include "tmpfile-util.h"
 
-#define EDIT_MARKER_START "### Anything between here and the comment below will become the new contents of the file"
-#define EDIT_MARKER_END "### Lines below this comment will be discarded"
+#define EDIT_MARKER_START "### Anything between here and the comment below will become the contents of the drop-in file"
+#define EDIT_MARKER_END "### Edits below this comment will be discarded"
 
 int verb_cat(int argc, char *argv[], void *userdata) {
         _cleanup_(hashmap_freep) Hashmap *cached_name_map = NULL, *cached_id_map = NULL;
@@ -110,12 +110,20 @@ int verb_cat(int argc, char *argv[], void *userdata) {
         return rc;
 }
 
-static int create_edit_temp_file(const char *new_path, const char *original_path, char ** const original_unit_paths, char **ret_tmp_fn) {
+static int create_edit_temp_file(
+                const char *new_path,
+                const char *original_path,
+                char ** const original_unit_paths,
+                char **ret_tmp_fn,
+                unsigned *ret_edit_line) {
+
         _cleanup_free_ char *t = NULL;
+        unsigned ln = 1;
         int r;
 
         assert(new_path);
         assert(ret_tmp_fn);
+        assert(ret_edit_line);
 
         r = tempfn_random(new_path, NULL, &t);
         if (r < 0)
@@ -154,8 +162,7 @@ static int create_edit_temp_file(const char *new_path, const char *original_path
                 if (!f)
                         return log_error_errno(errno, "Failed to open \"%s\": %m", t);
 
-                r = fchmod(fileno(f), 0644);
-                if (r < 0)
+                if (fchmod(fileno(f), 0644) < 0)
                         return log_error_errno(errno, "Failed to change mode of \"%s\": %m", t);
 
                 r = read_full_file(new_path, &new_contents, NULL);
@@ -164,13 +171,17 @@ static int create_edit_temp_file(const char *new_path, const char *original_path
 
                 fprintf(f,
                         "### Editing %s\n"
-                        EDIT_MARKER_START
-                        "\n\n%s%s\n"
+                        EDIT_MARKER_START "\n"
+                        "\n"
+                        "%s%s"
+                        "\n"
                         EDIT_MARKER_END,
                         new_path,
                         strempty(new_contents),
                         new_contents && endswith(new_contents, "\n") ? "" : "\n");
 
+                ln = 4; /* start editing at the contents */
+
                 /* Add a comment with the contents of the original unit files */
                 STRV_FOREACH(path, original_unit_paths) {
                         _cleanup_free_ char *contents = NULL;
@@ -200,6 +211,7 @@ static int create_edit_temp_file(const char *new_path, const char *original_path
         }
 
         *ret_tmp_fn = TAKE_PTR(t);
+        *ret_edit_line = ln;
 
         return 0;
 }
@@ -222,9 +234,7 @@ static int get_file_to_edit(
                 run = path_join(paths->runtime_config, name);
                 if (!run)
                         return log_oom();
-        }
 
-        if (arg_runtime) {
                 if (access(path, F_OK) >= 0)
                         return log_error_errno(SYNTHETIC_ERRNO(EEXIST),
                                                "Refusing to create \"%s\" because it would be overridden by \"%s\" anyway.",
@@ -237,34 +247,53 @@ static int get_file_to_edit(
         return 0;
 }
 
+typedef struct EditFile {
+        char *path;
+        char *tmp;
+        unsigned line;
+} EditFile;
+
+static void edit_file_free_all(EditFile **f) {
+        if (!f || !*f)
+                return;
+
+        for (EditFile *i = *f; i->path; i++) {
+                free(i->path);
+                free(i->tmp);
+        }
+
+        free(*f);
+}
+
 static int unit_file_create_new(
                 const LookupPaths *paths,
                 const char *unit_name,
                 const char *suffix,
                 char ** const original_unit_paths,
-                char **ret_new_path,
-                char **ret_tmp_path) {
+                EditFile *ret_edit_file) {
 
         _cleanup_free_ char *new_path = NULL, *tmp_path = NULL;
+        unsigned edit_line;
         const char *ending;
         int r;
 
         assert(unit_name);
-        assert(ret_new_path);
-        assert(ret_tmp_path);
+        assert(ret_edit_file);
 
         ending = strjoina(unit_name, suffix);
         r = get_file_to_edit(paths, ending, &new_path);
         if (r < 0)
                 return r;
 
-        r = create_edit_temp_file(new_path, NULL, original_unit_paths, &tmp_path);
+        r = create_edit_temp_file(new_path, NULL, original_unit_paths, &tmp_path, &edit_line);
         if (r < 0)
                 return r;
 
-        *ret_new_path = TAKE_PTR(new_path);
-        *ret_tmp_path = TAKE_PTR(tmp_path);
-
+        *ret_edit_file = (EditFile) {
+                .path = TAKE_PTR(new_path),
+                .tmp = TAKE_PTR(tmp_path),
+                .line = edit_line,
+        };
         return 0;
 }
 
@@ -272,16 +301,15 @@ static int unit_file_create_copy(
                 const LookupPaths *paths,
                 const char *unit_name,
                 const char *fragment_path,
-                char **ret_new_path,
-                char **ret_tmp_path) {
+                EditFile *ret_edit_file) {
 
         _cleanup_free_ char *new_path = NULL, *tmp_path = NULL;
+        unsigned edit_line;
         int r;
 
         assert(fragment_path);
         assert(unit_name);
-        assert(ret_new_path);
-        assert(ret_tmp_path);
+        assert(ret_edit_file);
 
         r = get_file_to_edit(paths, unit_name, &new_path);
         if (r < 0)
@@ -297,30 +325,30 @@ static int unit_file_create_copy(
                         return log_warning_errno(SYNTHETIC_ERRNO(EKEYREJECTED), "%s skipped.", unit_name);
         }
 
-        r = create_edit_temp_file(new_path, fragment_path, NULL, &tmp_path);
+        r = create_edit_temp_file(new_path, fragment_path, NULL, &tmp_path, &edit_line);
         if (r < 0)
                 return r;
 
-        *ret_new_path = TAKE_PTR(new_path);
-        *ret_tmp_path = TAKE_PTR(tmp_path);
-
+        *ret_edit_file = (EditFile) {
+                .path = TAKE_PTR(new_path),
+                .tmp = TAKE_PTR(tmp_path),
+                .line = edit_line,
+        };
         return 0;
 }
 
-static int run_editor(char **paths) {
+static int run_editor(const EditFile *files) {
         int r;
 
-        assert(paths);
+        assert(files);
 
         r = safe_fork("(editor)", FORK_RESET_SIGNALS|FORK_DEATHSIG|FORK_RLIMIT_NOFILE_SAFE|FORK_LOG|FORK_WAIT, NULL);
         if (r < 0)
                 return r;
         if (r == 0) {
-                char **editor_args = NULL;
                 size_t n_editor_args = 0, i = 1, argc;
-                const char **args, *editor;
-
-                argc = strv_length(paths)/2 + 1;
+                char **editor_args = NULL, **args;
+                const char *editor;
 
                 /* SYSTEMD_EDITOR takes precedence over EDITOR which takes precedence over VISUAL.  If
                  * neither SYSTEMD_EDITOR nor EDITOR nor VISUAL are present, we try to execute well known
@@ -331,17 +359,22 @@ static int run_editor(char **paths) {
                 if (!editor)
                         editor = getenv("VISUAL");
 
-                if (!isempty(editor)) {
+                if (isempty(editor))
+                        argc = 1;
+                else {
                         editor_args = strv_split(editor, WHITESPACE);
                         if (!editor_args) {
                                 (void) log_oom();
                                 _exit(EXIT_FAILURE);
                         }
                         n_editor_args = strv_length(editor_args);
-                        argc += n_editor_args - 1;
+                        argc = n_editor_args;
                 }
 
-                args = newa(const char*, argc + 1);
+                for (const EditFile *f = files; f->path; f++)
+                        argc += 2;
+
+                args = newa(char*, argc + 1);
 
                 if (n_editor_args > 0) {
                         args[0] = editor_args[0];
@@ -349,15 +382,26 @@ static int run_editor(char **paths) {
                                 args[i] = editor_args[i];
                 }
 
-                STRV_FOREACH_PAIR(original_path, tmp_path, paths)
-                        args[i++] = *tmp_path;
+                if (files[0].path && files[0].line > 1 && !files[1].path) {
+                        /* If editing a single file only, use the +LINE syntax to put cursor on the right line */
+                        if (asprintf(args + i, "+%u", files[0].line) < 0) {
+                                (void) log_oom();
+                                _exit(EXIT_FAILURE);
+                        }
+
+                        i++;
+                        args[i++] = files[0].tmp;
+                } else
+                        for (const EditFile *f = files; f->path; f++)
+                                args[i++] = f->tmp;
+
                 args[i] = NULL;
 
                 if (n_editor_args > 0)
                         execvp(args[0], (char* const*) args);
 
                 FOREACH_STRING(name, "editor", "nano", "vim", "vi") {
-                        args[0] = name;
+                        args[0] = (char*) name;
                         execvp(name, (char* const*) args);
                         /* We do not fail if the editor doesn't exist because we want to try each one of them
                          * before failing. */
@@ -374,22 +418,47 @@ static int run_editor(char **paths) {
         return 0;
 }
 
-static int find_paths_to_edit(sd_bus *bus, char **names, char ***paths) {
+static int find_paths_to_edit(
+                sd_bus *bus,
+                char **names,
+                EditFile **ret_edit_files) {
+
         _cleanup_(hashmap_freep) Hashmap *cached_name_map = NULL, *cached_id_map = NULL;
+        _cleanup_(edit_file_free_all) EditFile *edit_files = NULL;
         _cleanup_(lookup_paths_free) LookupPaths lp = {};
+        _cleanup_free_ char *drop_in_alloc = NULL, *suffix = NULL;
+        const char *drop_in;
+        size_t n_edit_files = 0;
         int r;
 
         assert(names);
-        assert(paths);
+        assert(ret_edit_files);
+
+        if (isempty(arg_drop_in))
+                drop_in = "override.conf";
+        else if (!endswith(arg_drop_in, ".conf")) {
+                drop_in_alloc = strjoin(arg_drop_in, ".conf");
+                if (!drop_in_alloc)
+                        return log_oom();
+
+                drop_in = drop_in_alloc;
+        } else
+                drop_in = arg_drop_in;
+
+        if (!filename_is_valid(drop_in))
+                return log_error_errno(SYNTHETIC_ERRNO(EINVAL), "Invalid drop-in file name '%s'.", drop_in);
+
+        suffix = strjoin(".d/", drop_in);
+        if (!suffix)
+                return log_oom();
 
         r = lookup_paths_init(&lp, arg_scope, 0, arg_root);
         if (r < 0)
                 return r;
 
         STRV_FOREACH(name, names) {
-                _cleanup_free_ char *path = NULL, *new_path = NULL, *tmp_path = NULL, *tmp_name = NULL;
+                _cleanup_free_ char *path = NULL;
                 _cleanup_strv_free_ char **unit_paths = NULL;
-                const char *unit_name;
 
                 r = unit_find_paths(bus, *name, &lp, false, &cached_name_map, &cached_id_map, &path, &unit_paths);
                 if (r == -EKEYREJECTED) {
@@ -403,6 +472,9 @@ static int find_paths_to_edit(sd_bus *bus, char **names, char ***paths) {
                 if (r < 0)
                         return r;
 
+                if (!GREEDY_REALLOC0(edit_files, n_edit_files + 2))
+                        return log_oom();
+
                 if (!path) {
                         if (!arg_force) {
                                 log_info("Run 'systemctl edit%s --force --full %s' to create a new unit.",
@@ -413,16 +485,23 @@ static int find_paths_to_edit(sd_bus *bus, char **names, char ***paths) {
                         }
 
                         /* Create a new unit from scratch */
-                        unit_name = *name;
-                        r = unit_file_create_new(&lp, unit_name,
-                                                 arg_full ? NULL : ".d/override.conf",
-                                                 NULL, &new_path, &tmp_path);
+                        r = unit_file_create_new(
+                                        &lp,
+                                        *name,
+                                        arg_full ? NULL : suffix,
+                                        NULL,
+                                        edit_files + n_edit_files);
                 } else {
-                        unit_name = basename(path);
+                        _cleanup_free_ char *unit_name = NULL;
+
+                        r = path_extract_filename(path, &unit_name);
+                        if (r < 0)
+                                return log_error_errno(r, "Failed to extract unit name from path '%s': %m", path);
+
                         /* We follow unit aliases, but we need to propagate the instance */
                         if (unit_name_is_valid(*name, UNIT_NAME_INSTANCE) &&
                             unit_name_is_valid(unit_name, UNIT_NAME_TEMPLATE)) {
-                                _cleanup_free_ char *instance = NULL;
+                                _cleanup_free_ char *instance = NULL, *tmp_name = NULL;
 
                                 r = unit_name_to_instance(*name, &instance);
                                 if (r < 0)
@@ -432,72 +511,81 @@ static int find_paths_to_edit(sd_bus *bus, char **names, char ***paths) {
                                 if (r < 0)
                                         return r;
 
-                                unit_name = tmp_name;
+                                free_and_replace(unit_name, tmp_name);
                         }
 
                         if (arg_full)
-                                r = unit_file_create_copy(&lp, unit_name, path, &new_path, &tmp_path);
+                                r = unit_file_create_copy(
+                                                &lp,
+                                                unit_name,
+                                                path,
+                                                edit_files + n_edit_files);
                         else {
                                 r = strv_prepend(&unit_paths, path);
                                 if (r < 0)
                                         return log_oom();
 
-                                r = unit_file_create_new(&lp, unit_name, ".d/override.conf", unit_paths, &new_path, &tmp_path);
+                                r = unit_file_create_new(
+                                                &lp,
+                                                unit_name,
+                                                suffix,
+                                                unit_paths,
+                                                edit_files + n_edit_files);
                         }
                 }
                 if (r < 0)
                         return r;
 
-                r = strv_push_pair(paths, new_path, tmp_path);
-                if (r < 0)
-                        return log_oom();
-
-                new_path = tmp_path = NULL;
+                n_edit_files++;
         }
 
+        *ret_edit_files = n_edit_files > 0 ? TAKE_PTR(edit_files) : NULL;
         return 0;
 }
 
 static int trim_edit_markers(const char *path) {
-        _cleanup_free_ char *contents = NULL;
-        char *contents_start = NULL;
-        const char *contents_end = NULL;
-        size_t size;
+        _cleanup_free_ char *old_contents = NULL, *new_contents = NULL;
+        char *contents_start, *contents_end;
+        const char *c = NULL;
         int r;
 
         /* Trim out the lines between the two markers */
-        r = read_full_file(path, &contents, NULL);
+        r = read_full_file(path, &old_contents, NULL);
         if (r < 0)
                 return log_error_errno(r, "Failed to read temporary file \"%s\": %m", path);
 
-        size = strlen(contents);
-
-        contents_start = strstr(contents, EDIT_MARKER_START);
+        contents_start = strstr(old_contents, EDIT_MARKER_START);
         if (contents_start)
                 contents_start += strlen(EDIT_MARKER_START);
         else
-                contents_start = contents;
+                contents_start = old_contents;
 
         contents_end = strstr(contents_start, EDIT_MARKER_END);
         if (contents_end)
-                strshorten(contents_start, contents_end - contents_start);
+                contents_end[0] = 0;
 
-        contents_start = strstrip(contents_start);
+        c = strstrip(contents_start);
+        if (isempty(c))
+                return 0; /* All gone now */
 
-        /* Write new contents if the trimming actually changed anything */
-        if (strlen(contents) != size) {
-                r = write_string_file(path, contents_start, WRITE_STRING_FILE_CREATE | WRITE_STRING_FILE_TRUNCATE | WRITE_STRING_FILE_AVOID_NEWLINE);
-                if (r < 0)
-                        return log_error_errno(r, "Failed to modify temporary file \"%s\": %m", path);
-        }
+        new_contents = strjoin(c, "\n"); /* Trim prefix and suffix, but ensure suffixed by single newline */
+        if (!new_contents)
+                return log_oom();
 
-        return 0;
+        if (streq(old_contents, new_contents)) /* Don't touch the file if the above didn't change a thing */
+                return 1; /* Unchanged, but good */
+
+        r = write_string_file(path, new_contents, WRITE_STRING_FILE_CREATE | WRITE_STRING_FILE_TRUNCATE | WRITE_STRING_FILE_AVOID_NEWLINE);
+        if (r < 0)
+                return log_error_errno(r, "Failed to modify temporary file \"%s\": %m", path);
+
+        return 1; /* Changed, but good */
 }
 
 int verb_edit(int argc, char *argv[], void *userdata) {
+        _cleanup_(edit_file_free_all) EditFile *edit_files = NULL;
         _cleanup_(lookup_paths_free) LookupPaths lp = {};
         _cleanup_strv_free_ char **names = NULL;
-        _cleanup_strv_free_ char **paths = NULL;
         sd_bus *bus;
         int r;
 
@@ -533,34 +621,33 @@ int verb_edit(int argc, char *argv[], void *userdata) {
                         return log_error_errno(SYNTHETIC_ERRNO(EINVAL), "Cannot edit %s: unit is masked.", *tmp);
         }
 
-        r = find_paths_to_edit(bus, names, &paths);
+        r = find_paths_to_edit(bus, names, &edit_files);
         if (r < 0)
                 return r;
-
-        if (strv_isempty(paths))
+        if (!edit_files)
                 return -ENOENT;
 
-        r = run_editor(paths);
+        r = run_editor(edit_files);
         if (r < 0)
                 goto end;
 
-        STRV_FOREACH_PAIR(original, tmp, paths) {
+        for (EditFile *f = edit_files; f->path; f++) {
                 /* If the temporary file is empty we ignore it. This allows the user to cancel the
                  * modification. */
-                r = trim_edit_markers(*tmp);
+                r = trim_edit_markers(f->tmp);
                 if (r < 0)
+                        goto end;
+                if (r == 0) /* has no actual contents? then ignore it */
                         continue;
 
-                if (null_or_empty_path(*tmp)) {
-                        log_warning("Editing \"%s\" canceled: temporary file is empty.", *original);
-                        continue;
-                }
-
-                r = rename(*tmp, *original);
+                r = RET_NERRNO(rename(f->tmp, f->path));
                 if (r < 0) {
-                        r = log_error_errno(errno, "Failed to rename \"%s\" to \"%s\": %m", *tmp, *original);
+                        log_error_errno(r, "Failed to rename \"%s\" to \"%s\": %m", f->tmp, f->path);
                         goto end;
                 }
+
+                f->tmp = mfree(f->tmp);
+                log_info("Successfully installed edited file '%s'.", f->path);
         }
 
         r = 0;
@@ -572,16 +659,18 @@ int verb_edit(int argc, char *argv[], void *userdata) {
         }
 
 end:
-        STRV_FOREACH_PAIR(original, tmp, paths) {
-                (void) unlink(*tmp);
+        for (EditFile *f = ASSERT_PTR(edit_files); f->path; f++) {
+
+                if (f->tmp)
+                        (void) unlink(f->tmp);
 
                 /* Removing empty dropin dirs */
                 if (!arg_full) {
                         _cleanup_free_ char *dir = NULL;
 
-                        r = path_extract_directory(*original, &dir);
+                        r = path_extract_directory(f->path, &dir);
                         if (r < 0)
-                                return log_error_errno(r, "Failed to extract directory from '%s': %m", *original);
+                                return log_error_errno(r, "Failed to extract directory from '%s': %m", f->path);
 
                         /* No need to check if the dir is empty, rmdir does nothing if it is not the case. */
                         (void) rmdir(dir);
