@@ -121,7 +121,8 @@ static int set_exit_code(uint8_t code) {
 }
 
 int verb_start_special(int argc, char *argv[], void *userdata) {
-        bool termination_action; /* An action that terminates the system, can be performed also by signal. */
+        bool termination_action; /* An action that terminates the manager, can be performed also by
+                                  * signal. */
         enum action a;
         int r;
 
@@ -139,36 +140,21 @@ int verb_start_special(int argc, char *argv[], void *userdata) {
                         return r;
         }
 
-        termination_action = IN_SET(a, ACTION_HALT, ACTION_POWEROFF, ACTION_REBOOT);
+        r = prepare_firmware_setup();
+        if (r < 0)
+                return r;
 
-        if (termination_action) {
-                r = prepare_firmware_setup();
-                if (r < 0)
-                        return r;
+        r = prepare_boot_loader_menu();
+        if (r < 0)
+                return r;
 
-                r = prepare_boot_loader_menu();
-                if (r < 0)
-                        return r;
-
-                r = prepare_boot_loader_entry();
-                if (r < 0)
-                        return r;
-        }
+        r = prepare_boot_loader_entry();
+        if (r < 0)
+                return r;
 
         if (a == ACTION_REBOOT) {
-                const char *arg = NULL;
-
-                if (argc > 1) {
-                        if (arg_reboot_argument)
-                                return log_error_errno(SYNTHETIC_ERRNO(EINVAL), "Both --reboot-argument= and positional argument passed to reboot command, refusing.");
-
-                        log_notice("Positional argument to reboot command is deprecated, please use --reboot-argument= instead. Accepting anyway.");
-                        arg = argv[1];
-                } else
-                        arg = arg_reboot_argument;
-
-                if (arg) {
-                        r = update_reboot_parameter_and_warn(arg, false);
+                if (arg_reboot_argument) {
+                        r = update_reboot_parameter_and_warn(arg_reboot_argument, false);
                         if (r < 0)
                                 return r;
                 }
@@ -195,6 +181,10 @@ int verb_start_special(int argc, char *argv[], void *userdata) {
                         return r;
         }
 
+        termination_action = IN_SET(a,
+                                    ACTION_HALT,
+                                    ACTION_POWEROFF,
+                                    ACTION_REBOOT);
         if (termination_action && arg_force >= 2)
                 return halt_now(a);
 
@@ -226,7 +216,7 @@ int verb_start_special(int argc, char *argv[], void *userdata) {
 
                         arg_no_block = true;
 
-                } else if (IN_SET(a, ACTION_EXIT))
+                } else if (a == ACTION_EXIT)
                         /* Since exit is so close in behaviour to power-off/reboot, let's also make
                          * it asynchronous, in order to not confuse the user needlessly with unexpected
                          * behaviour. */
