@@ -1,7 +1,6 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include <fcntl.h>
-#include <sys/file.h>
 
 #include "btrfs-util.h"
 #include "fd-util.h"
@@ -9,7 +8,6 @@
 #include "format-util.h"
 #include "log.h"
 #include "string-util.h"
-#include "util.h"
 
 int main(int argc, char *argv[]) {
         BtrfsQuotaInfo quota;
