@@ -32,7 +32,6 @@
 #include "tmpfile-util.h"
 #include "unit.h"
 #include "user-util.h"
-#include "util.h"
 #include "virt.h"
 
 static char *user_runtime_unit_dir = NULL;
@@ -203,17 +202,6 @@ static bool is_inaccessible_available(void) {
         return true;
 }
 
-static void start_parent_slices(Unit *unit) {
-        Unit *slice;
-
-        slice = UNIT_GET_SLICE(unit);
-        if (slice) {
-                start_parent_slices(slice);
-                int r = unit_start(slice, NULL);
-                assert_se(r >= 0 || r == -EALREADY);
-        }
-}
-
 static void _test(const char *file, unsigned line, const char *func,
                   Manager *m, const char *unit_name, int status_expected, int code_expected) {
         Unit *unit;
@@ -221,9 +209,6 @@ static void _test(const char *file, unsigned line, const char *func,
         assert_se(unit_name);
 
         assert_se(manager_load_startable_unit_or_warn(m, unit_name, NULL, &unit) >= 0);
-        /* We need to start the slices as well otherwise the slice cgroups might be pruned
-         * in on_cgroup_empty_event. */
-        start_parent_slices(unit);
         assert_se(unit_start(unit, NULL) >= 0);
         check_main_result(file, line, func, m, unit, status_expected, code_expected);
 }
@@ -391,7 +376,7 @@ static void test_exec_personality(Manager *m) {
 
 #elif defined(__i386__)
         test(m, "exec-personality-x86.service", 0, CLD_EXITED);
-#elif defined(__loongarch_lp64)
+#elif defined(__loongarch64)
         test(m, "exec-personality-loongarch64.service", 0, CLD_EXITED);
 #else
         log_notice("Unknown personality, skipping %s", __func__);
@@ -574,7 +559,7 @@ static int find_libraries(const char *exec, char ***ret) {
         _cleanup_(sd_event_source_unrefp) sd_event_source *sigchld_source = NULL;
         _cleanup_(sd_event_source_unrefp) sd_event_source *stdout_source = NULL;
         _cleanup_(sd_event_source_unrefp) sd_event_source *stderr_source = NULL;
-        _cleanup_close_pair_ int outpipe[2] = {-1, -1}, errpipe[2] = {-1, -1};
+        _cleanup_close_pair_ int outpipe[2] = PIPE_EBADF, errpipe[2] = PIPE_EBADF;
         _cleanup_strv_free_ char **libraries = NULL;
         _cleanup_free_ char *result = NULL;
         pid_t pid;
@@ -591,7 +576,7 @@ static int find_libraries(const char *exec, char ***ret) {
         r = safe_fork("(spawn-ldd)", FORK_RESET_SIGNALS|FORK_DEATHSIG|FORK_LOG, &pid);
         assert_se(r >= 0);
         if (r == 0) {
-                if (rearrange_stdio(-1, TAKE_FD(outpipe[1]), TAKE_FD(errpipe[1])) < 0)
+                if (rearrange_stdio(-EBADF, TAKE_FD(outpipe[1]), TAKE_FD(errpipe[1])) < 0)
                         _exit(EXIT_FAILURE);
 
                 (void) close_all_fds(NULL, 0);
@@ -666,11 +651,6 @@ static void test_exec_mount_apivfs(Manager *m) {
 
         assert_se(user_runtime_unit_dir);
 
-        r = find_executable("ldd", NULL);
-        if (r < 0) {
-                log_notice_errno(r, "Skipping %s, could not find 'ldd' command: %m", __func__);
-                return;
-        }
         r = find_executable("touch", &fullpath_touch);
         if (r < 0) {
                 log_notice_errno(r, "Skipping %s, could not find 'touch' command: %m", __func__);
