@@ -1,7 +1,6 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include <fcntl.h>
-#include <sysexits.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
 
@@ -12,7 +11,6 @@
 #include "string-util.h"
 #include "tests.h"
 #include "user-util.h"
-#include "util.h"
 #include "virt.h"
 
 TEST(namespace_cleanup_tmpdir) {
@@ -85,8 +83,7 @@ TEST(tmpdir) {
 }
 
 static void test_shareable_ns(unsigned long nsflag) {
-        _cleanup_close_pair_ int s[2] = { -1, -1 };
-        bool permission_denied = false;
+        _cleanup_close_pair_ int s[2] = PIPE_EBADF;
         pid_t pid1, pid2, pid3;
         int r, n = 0;
         siginfo_t si;
@@ -96,15 +93,15 @@ static void test_shareable_ns(unsigned long nsflag) {
                 return;
         }
 
-        assert_se(socketpair(AF_UNIX, SOCK_DGRAM|SOCK_CLOEXEC, 0, s) >= 0);
+        assert_se(socketpair(AF_UNIX, SOCK_DGRAM, 0, s) >= 0);
 
         pid1 = fork();
         assert_se(pid1 >= 0);
 
         if (pid1 == 0) {
                 r = setup_shareable_ns(s, nsflag);
-                assert_se(r >= 0 || ERRNO_IS_PRIVILEGE(r));
-                _exit(r >= 0 ? r : EX_NOPERM);
+                assert_se(r >= 0);
+                _exit(r);
         }
 
         pid2 = fork();
@@ -112,8 +109,8 @@ static void test_shareable_ns(unsigned long nsflag) {
 
         if (pid2 == 0) {
                 r = setup_shareable_ns(s, nsflag);
-                assert_se(r >= 0 || ERRNO_IS_PRIVILEGE(r));
-                _exit(r >= 0 ? r : EX_NOPERM);
+                assert_se(r >= 0);
+                exit(r);
         }
 
         pid3 = fork();
@@ -121,38 +118,24 @@ static void test_shareable_ns(unsigned long nsflag) {
 
         if (pid3 == 0) {
                 r = setup_shareable_ns(s, nsflag);
-                assert_se(r >= 0 || ERRNO_IS_PRIVILEGE(r));
-                _exit(r >= 0 ? r : EX_NOPERM);
+                assert_se(r >= 0);
+                exit(r);
         }
 
         r = wait_for_terminate(pid1, &si);
         assert_se(r >= 0);
         assert_se(si.si_code == CLD_EXITED);
-        if (si.si_status == EX_NOPERM)
-                permission_denied = true;
-        else
-                n += si.si_status;
+        n += si.si_status;
 
         r = wait_for_terminate(pid2, &si);
         assert_se(r >= 0);
         assert_se(si.si_code == CLD_EXITED);
-        if (si.si_status == EX_NOPERM)
-                permission_denied = true;
-        else
-                n += si.si_status;
+        n += si.si_status;
 
         r = wait_for_terminate(pid3, &si);
         assert_se(r >= 0);
         assert_se(si.si_code == CLD_EXITED);
-        if (si.si_status == EX_NOPERM)
-                permission_denied = true;
-        else
-                n += si.si_status;
-
-        /* LSMs can cause setup_shareable_ns() to fail with permission denied, do not fail the test in that
-         * case (e.g.: LXC with AppArmor on kernel < v6.2). */
-        if (permission_denied)
-                return (void) log_tests_skipped("insufficient privileges");
+        n += si.si_status;
 
         assert_se(n == 1);
 }
@@ -187,7 +170,7 @@ TEST(protect_kernel_logs) {
         assert_se(pid >= 0);
 
         if (pid == 0) {
-                _cleanup_close_ int fd = -1;
+                _cleanup_close_ int fd = -EBADF;
 
                 fd = open("/dev/kmsg", O_RDONLY | O_CLOEXEC);
                 assert_se(fd > 0);
