@@ -16,7 +16,6 @@
 #include "strv.h"
 #include "tests.h"
 #include "tmpfile-util.h"
-#include "util.h"
 
 TEST(print_paths) {
         log_info("DEFAULT_PATH=%s", DEFAULT_PATH);
@@ -27,6 +26,10 @@ TEST(path) {
         assert_se(path_is_absolute("/"));
         assert_se(!path_is_absolute("./"));
 
+        assert_se(is_path("/dir"));
+        assert_se(is_path("a/b"));
+        assert_se(!is_path("."));
+
         assert_se(streq(basename("./aa/bb/../file.da."), "file.da."));
         assert_se(streq(basename("/aa///.file"), ".file"));
         assert_se(streq(basename("/aa///file..."), "file..."));
@@ -50,105 +53,11 @@ TEST(path) {
         assert_se(!path_equal_filename("/b", "/c"));
 }
 
-TEST(is_path) {
-        assert_se(!is_path("foo"));
-        assert_se(!is_path("dos.ext"));
-        assert_se( is_path("/dir"));
-        assert_se( is_path("a/b"));
-        assert_se( is_path("a/b.ext"));
-
-        assert_se(!is_path("."));
-        assert_se(!is_path(""));
-        assert_se(!is_path(".."));
-
-        assert_se( is_path("/dev"));
-        assert_se( is_path("/./dev"));
-        assert_se( is_path("/./dev/."));
-        assert_se( is_path("/./dev."));
-        assert_se( is_path("//dev"));
-        assert_se( is_path("///dev"));
-        assert_se( is_path("/dev/"));
-        assert_se( is_path("///dev/"));
-        assert_se( is_path("/./dev/"));
-        assert_se( is_path("/../dev/"));
-        assert_se( is_path("/dev/sda"));
-        assert_se( is_path("/dev/sda5"));
-        assert_se( is_path("/dev/sda5b3"));
-        assert_se( is_path("/dev/sda5b3/idontexit"));
-        assert_se( is_path("/../dev/sda"));
-        assert_se( is_path("/../../dev/sda5"));
-        assert_se( is_path("/../../../dev/sda5b3"));
-        assert_se( is_path("/.././.././dev/sda5b3/idontexit"));
-        assert_se( is_path("/sys"));
-        assert_se( is_path("/sys/"));
-        assert_se( is_path("/./sys"));
-        assert_se( is_path("/./sys/."));
-        assert_se( is_path("/./sys."));
-        assert_se( is_path("/sys/what"));
-        assert_se( is_path("/sys/something/.."));
-        assert_se( is_path("/sys/something/../"));
-        assert_se( is_path("/sys////"));
-        assert_se( is_path("/sys////."));
-        assert_se( is_path("/sys/.."));
-        assert_se( is_path("/sys/../"));
-        assert_se( is_path("/usr/../dev/sda"));
-}
-
-TEST(is_device_path) {
-        assert_se(!is_device_path("foo"));
-        assert_se(!is_device_path("dos.ext"));
-        assert_se(!is_device_path("/dir"));
-        assert_se(!is_device_path("a/b"));
-        assert_se(!is_device_path("a/b.ext"));
-
-        assert_se(!is_device_path("."));
-        assert_se(!is_device_path(""));
-        assert_se(!is_device_path(".."));
-
-        assert_se(!is_device_path("/dev"));
-        assert_se(!is_device_path("/./dev"));
-        assert_se(!is_device_path("/./dev/."));
-        assert_se(!is_device_path("/./dev."));
-        assert_se( is_device_path("/./dev/foo"));
-        assert_se( is_device_path("/./dev/./foo"));
-        assert_se(!is_device_path("/./dev./foo"));
-        assert_se(!is_device_path("//dev"));
-        assert_se(!is_device_path("///dev"));
-        assert_se(!is_device_path("/dev/"));
-        assert_se(!is_device_path("///dev/"));
-        assert_se(!is_device_path("/./dev/"));
-        assert_se(!is_device_path("/../dev/"));
-        assert_se( is_device_path("/dev/sda"));
-        assert_se( is_device_path("/dev/sda5"));
-        assert_se( is_device_path("/dev/sda5b3"));
-        assert_se( is_device_path("/dev/sda5b3/idontexit"));
-        assert_se(!is_device_path("/../dev/sda"));
-        assert_se(!is_device_path("/../../dev/sda5"));
-        assert_se(!is_device_path("/../../../dev/sda5b3"));
-        assert_se(!is_device_path("/.././.././dev/sda5b3/idontexit"));
-        assert_se(!is_device_path("/sys"));
-        assert_se(!is_device_path("/sys/"));
-        assert_se(!is_device_path("/./sys"));
-        assert_se(!is_device_path("/./sys/."));
-        assert_se(!is_device_path("/./sys."));
-        assert_se( is_device_path("/./sys/foo"));
-        assert_se( is_device_path("/./sys/./foo"));
-        assert_se(!is_device_path("/./sys./foo"));
-        assert_se( is_device_path("/sys/what"));
-        assert_se( is_device_path("/sys/something/.."));
-        assert_se( is_device_path("/sys/something/../"));
-        assert_se(!is_device_path("/sys////"));
-        assert_se(!is_device_path("/sys////."));
-        assert_se( is_device_path("/sys/.."));
-        assert_se( is_device_path("/sys/../"));
-        assert_se(!is_device_path("/usr/../dev/sda"));
-}
-
-static void test_path_simplify_one(const char *in, const char *out, PathSimplifyFlags flags) {
+static void test_path_simplify_one(const char *in, const char *out) {
         char *p;
 
         p = strdupa_safe(in);
-        path_simplify_full(p, flags);
+        path_simplify(p);
         log_debug("/* test_path_simplify(%s) → %s (expected: %s) */", in, p, out);
         assert_se(streq(p, out));
 }
@@ -157,37 +66,34 @@ TEST(path_simplify) {
         _cleanup_free_ char *hoge = NULL, *hoge_out = NULL;
         char foo[NAME_MAX * 2];
 
-        test_path_simplify_one("", "", 0);
-        test_path_simplify_one("aaa/bbb////ccc", "aaa/bbb/ccc", 0);
-        test_path_simplify_one("//aaa/.////ccc", "/aaa/ccc", 0);
-        test_path_simplify_one("///", "/", 0);
-        test_path_simplify_one("///", "/", PATH_SIMPLIFY_KEEP_TRAILING_SLASH);
-        test_path_simplify_one("///.//", "/", 0);
-        test_path_simplify_one("///.//.///", "/", 0);
-        test_path_simplify_one("////.././///../.", "/../..", 0);
-        test_path_simplify_one(".", ".", 0);
-        test_path_simplify_one("./", ".", 0);
-        test_path_simplify_one("./", "./", PATH_SIMPLIFY_KEEP_TRAILING_SLASH);
-        test_path_simplify_one(".///.//./.", ".", 0);
-        test_path_simplify_one(".///.//././/", ".", 0);
+        test_path_simplify_one("", "");
+        test_path_simplify_one("aaa/bbb////ccc", "aaa/bbb/ccc");
+        test_path_simplify_one("//aaa/.////ccc", "/aaa/ccc");
+        test_path_simplify_one("///", "/");
+        test_path_simplify_one("///.//", "/");
+        test_path_simplify_one("///.//.///", "/");
+        test_path_simplify_one("////.././///../.", "/../..");
+        test_path_simplify_one(".", ".");
+        test_path_simplify_one("./", ".");
+        test_path_simplify_one(".///.//./.", ".");
+        test_path_simplify_one(".///.//././/", ".");
         test_path_simplify_one("//./aaa///.//./.bbb/..///c.//d.dd///..eeee/.",
-                               "/aaa/.bbb/../c./d.dd/..eeee", 0);
+                               "/aaa/.bbb/../c./d.dd/..eeee");
         test_path_simplify_one("//./aaa///.//./.bbb/..///c.//d.dd///..eeee/..",
-                               "/aaa/.bbb/../c./d.dd/..eeee/..", 0);
+                               "/aaa/.bbb/../c./d.dd/..eeee/..");
         test_path_simplify_one(".//./aaa///.//./.bbb/..///c.//d.dd///..eeee/..",
-                               "aaa/.bbb/../c./d.dd/..eeee/..", 0);
+                               "aaa/.bbb/../c./d.dd/..eeee/..");
         test_path_simplify_one("..//./aaa///.//./.bbb/..///c.//d.dd///..eeee/..",
-                               "../aaa/.bbb/../c./d.dd/..eeee/..", 0);
-        test_path_simplify_one("abc///", "abc/", PATH_SIMPLIFY_KEEP_TRAILING_SLASH);
+                               "../aaa/.bbb/../c./d.dd/..eeee/..");
 
         memset(foo, 'a', sizeof(foo) -1);
         char_array_0(foo);
 
-        test_path_simplify_one(foo, foo, 0);
+        test_path_simplify_one(foo, foo);
 
         hoge = strjoin("/", foo);
         assert_se(hoge);
-        test_path_simplify_one(hoge, hoge, 0);
+        test_path_simplify_one(hoge, hoge);
         hoge = mfree(hoge);
 
         hoge = strjoin("a////.//././//./b///././/./c/////././//./", foo, "//.//////d/e/.//f/");
@@ -196,7 +102,7 @@ TEST(path_simplify) {
         hoge_out = strjoin("a/b/c/", foo, "//.//////d/e/.//f/");
         assert_se(hoge_out);
 
-        test_path_simplify_one(hoge, hoge_out, 0);
+        test_path_simplify_one(hoge, hoge_out);
 }
 
 static void test_path_compare_one(const char *a, const char *b, int expected) {
@@ -289,7 +195,7 @@ TEST(path_equal_root) {
 TEST(find_executable_full) {
         char *p;
         char* test_file_name;
-        _cleanup_close_ int fd = -1;
+        _cleanup_close_ int fd = -EBADF;
         char fn[] = "/tmp/test-XXXXXX";
 
         assert_se(find_executable_full("sh", NULL, NULL, true, &p, NULL) == 0);
@@ -372,7 +278,7 @@ TEST(find_executable) {
 
 static void test_find_executable_exec_one(const char *path) {
         _cleanup_free_ char *t = NULL;
-        _cleanup_close_ int fd = -1;
+        _cleanup_close_ int fd = -EBADF;
         pid_t pid;
         int r;
 
@@ -537,8 +443,8 @@ TEST(fsck_exists) {
         /* Ensure we use a sane default for PATH. */
         assert_se(unsetenv("PATH") == 0);
 
-        /* We might or might not find one of these, so keep the test lax. */
-        assert_se(fsck_exists_for_fstype("minix") >= 0);
+        /* fsck.minix is provided by util-linux and will probably exist. */
+        assert_se(fsck_exists_for_fstype("minix") == 1);
 
         assert_se(fsck_exists_for_fstype("AbCdE") == 0);
         assert_se(fsck_exists_for_fstype("/../bin/") == 0);
@@ -650,17 +556,6 @@ TEST(path_startswith) {
         test_path_startswith_one("/foo/bar/barfoo/", "////foo/bar/barfoo/", "/foo/bar/barfoo/", "");
         test_path_startswith_one("/foo/bar/barfoo/", "/foo/bar/barfoo", "/foo/bar/barfoo/", "");
 
-        test_path_startswith_one("/foo/./bar///barfoo/./.", "/foo", "/foo/./", "bar///barfoo/./.");
-        test_path_startswith_one("/foo/./bar///barfoo/./.", "/foo/", "/foo/./", "bar///barfoo/./.");
-        test_path_startswith_one("/foo/./bar///barfoo/./.", "/", "/", "foo/./bar///barfoo/./.");
-        test_path_startswith_one("/foo/./bar///barfoo/./.", "////", "/",  "foo/./bar///barfoo/./.");
-        test_path_startswith_one("/foo/./bar///barfoo/./.", "/foo//bar/////barfoo///", "/foo/./bar///barfoo/./.", "");
-        test_path_startswith_one("/foo/./bar///barfoo/./.", "/foo/bar/barfoo////", "/foo/./bar///barfoo/./.", "");
-        test_path_startswith_one("/foo/./bar///barfoo/./.", "/foo/bar///barfoo/", "/foo/./bar///barfoo/./.", "");
-        test_path_startswith_one("/foo/./bar///barfoo/./.", "/foo////bar/barfoo/", "/foo/./bar///barfoo/./.", "");
-        test_path_startswith_one("/foo/./bar///barfoo/./.", "////foo/bar/barfoo/", "/foo/./bar///barfoo/./.", "");
-        test_path_startswith_one("/foo/./bar///barfoo/./.", "/foo/bar/barfoo", "/foo/./bar///barfoo/./.", "");
-
         test_path_startswith_one("/foo/bar/barfoo/", "/foo/bar/barfooa/", NULL, NULL);
         test_path_startswith_one("/foo/bar/barfoo/", "/foo/bar/barfooa", NULL, NULL);
         test_path_startswith_one("/foo/bar/barfoo/", "", NULL, NULL);
@@ -705,23 +600,19 @@ TEST(prefix_root) {
 TEST(file_in_same_dir) {
         char *t;
 
-        t = file_in_same_dir("/", "a");
+        assert_se(file_in_same_dir("/", "a", &t) == -EADDRNOTAVAIL);
+
+        assert_se(file_in_same_dir("/", "/a", &t) >= 0);
         assert_se(streq(t, "/a"));
         free(t);
 
-        t = file_in_same_dir("/", "/a");
-        assert_se(streq(t, "/a"));
+        assert_se(file_in_same_dir("", "a", &t) == -EINVAL);
+
+        assert_se(file_in_same_dir("a/", "x", &t) >= 0);
+        assert_se(streq(t, "x"));
         free(t);
 
-        t = file_in_same_dir("", "a");
-        assert_se(streq(t, "a"));
-        free(t);
-
-        t = file_in_same_dir("a/", "a");
-        assert_se(streq(t, "a/a"));
-        free(t);
-
-        t = file_in_same_dir("bar/foo", "bar");
+        assert_se(file_in_same_dir("bar/foo", "bar", &t) >= 0);
         assert_se(streq(t, "bar/bar"));
         free(t);
 }
