@@ -3,13 +3,13 @@
 #include "alloc-util.h"
 #include "env-util.h"
 #include "errno-util.h"
+#include "initrd-util.h"
 #include "log.h"
 #include "macro.h"
 #include "proc-cmdline.h"
 #include "special.h"
 #include "string-util.h"
 #include "tests.h"
-#include "util.h"
 
 static int obj;
 
