@@ -103,7 +103,7 @@ typedef struct Context {
 } Context;
 
 #define CONTEXT_EMPTY {                                 \
-                .fd = -1,                               \
+                .fd = -EBADF,                           \
                 .media_feature = _FEATURE_INVALID,      \
                 .media_state = _MEDIA_STATE_INVALID,    \
         }
@@ -476,7 +476,7 @@ static int cd_profiles(Context *c) {
 
                 switch (feature) {
                 case 0x00:
-                        log_debug("GET CONFIGURATION: feature 'profiles', with %u entries", features[i + 3] / 4U);
+                        log_debug("GET CONFIGURATION: feature 'profiles', with %u entries", features[i + 3] / 4);
                         feature_profiles(c, features + i + 4, MIN(features[i + 3], len - i - 4));
                         break;
                 default:
@@ -817,7 +817,7 @@ static int feature_to_string_compare_func(const FeatureToString *a, const Featur
 }
 
 static void print_feature(Feature feature, const char *prefix) {
-        const FeatureToString *found, in = {
+        FeatureToString *found, in = {
                 .feature = feature,
         };
 
