@@ -17,6 +17,7 @@
 #include "rm-rf.h"
 #include "stdio-util.h"
 #include "string-util.h"
+#include "udev-util.h"
 #include "udev-watch.h"
 
 int device_new_from_watch_handle_at(sd_device **ret, int dirfd, int wd) {
@@ -160,26 +161,29 @@ static int udev_watch_clear(sd_device *dev, int dirfd, int *ret_wd) {
 
         if (ret_wd)
                 *ret_wd = wd;
-        r = 1;
+        r = 0;
 
 finalize:
         /* 5. remove symlink ID -> wd.
          * The file is always owned by the device. Hence, it is safe to remove it unconditionally. */
         if (unlinkat(dirfd, id, 0) < 0 && errno != ENOENT)
-                log_device_debug_errno(dev, errno, "Failed to remove '/run/udev/watch/%s', ignoring: %m", id);
+                log_device_debug_errno(dev, errno, "Failed to remove '/run/udev/watch/%s': %m", id);
 
         return r;
 }
 
 int udev_watch_begin(int inotify_fd, sd_device *dev) {
         char wd_str[DECIMAL_STR_MAX(int)];
-        _cleanup_close_ int dirfd = -1;
+        _cleanup_close_ int dirfd = -EBADF;
         const char *devnode, *id;
         int wd, r;
 
         assert(inotify_fd >= 0);
         assert(dev);
 
+        if (device_for_action(dev, SD_DEVICE_REMOVE))
+                return 0;
+
         r = sd_device_get_devname(dev, &devnode);
         if (r < 0)
                 return log_device_debug_errno(dev, r, "Failed to get device node: %m");
@@ -225,7 +229,7 @@ on_failure:
 }
 
 int udev_watch_end(int inotify_fd, sd_device *dev) {
-        _cleanup_close_ int dirfd = -1;
+        _cleanup_close_ int dirfd = -EBADF;
         int wd, r;
 
         assert(dev);
@@ -245,7 +249,7 @@ int udev_watch_end(int inotify_fd, sd_device *dev) {
 
         /* First, clear symlinks. */
         r = udev_watch_clear(dev, dirfd, &wd);
-        if (r <= 0)
+        if (r < 0)
                 return r;
 
         /* Then, remove inotify watch. */
