@@ -558,10 +558,7 @@ int xdg_autostart_service_generate_unit(
 
         r = xdg_autostart_format_exec_start(service->exec_string, &exec_start);
         if (r < 0) {
-                log_full_errno(r == -ENOENT ? LOG_INFO : LOG_WARNING, r,
-                               r == -ENOENT ? "%s: not generating unit, executable specified in Exec= does not exist."
-                                            : "%s: not generating unit, error parsing Exec= line: %m",
-                               service->path);
+                log_warning_errno(r, "%s: not generating unit, error parsing Exec= line: %m", service->path);
                 return 0;
         }
 
@@ -611,7 +608,7 @@ int xdg_autostart_service_generate_unit(
                 "ExitType=cgroup\n"
                 "ExecStart=:%s\n"
                 "Restart=no\n"
-                "TimeoutStopSec=5s\n"
+                "TimeoutSec=5s\n"
                 "Slice=app.slice\n",
                 exec_start);
 
