@@ -45,8 +45,6 @@ EOF
 
 portablectl "${ARGS[@]}" attach --now --runtime /usr/share/minimal_0.raw minimal-app0
 
-portablectl is-attached minimal-app0
-portablectl inspect /usr/share/minimal_0.raw minimal-app0.service
 systemctl is-active minimal-app0.service
 systemctl is-active minimal-app0-foo.service
 systemctl is-active minimal-app0-bar.service && exit 1
@@ -55,8 +53,6 @@ systemctl is-active minimal-app0-bar.service && exit 1
 # Let's set timeout to improve performance.
 timeout "$TIMEOUT" portablectl "${ARGS[@]}" reattach --now --runtime /usr/share/minimal_1.raw minimal-app0
 
-portablectl is-attached minimal-app0
-portablectl inspect /usr/share/minimal_0.raw minimal-app0.service
 systemctl is-active minimal-app0.service
 systemctl is-active minimal-app0-bar.service
 systemctl is-active minimal-app0-foo.service && exit 1
@@ -69,47 +65,6 @@ portablectl detach --now --runtime /usr/share/minimal_1.raw minimal-app0
 portablectl list | grep -q -F "No images."
 busctl tree org.freedesktop.portable1 --no-pager | grep -q -F '/org/freedesktop/portable1/image/minimal_5f1' && exit 1
 
-# Ensure we don't regress (again) when using --force
-
-mkdir -p /run/systemd/system.attached/minimal-app0.service.d/
-cat <<EOF >/run/systemd/system.attached/minimal-app0.service
-[Unit]
-Description=Minimal App 0
-EOF
-cat <<EOF >/run/systemd/system.attached/minimal-app0.service.d/10-profile.conf
-[Unit]
-Description=Minimal App 0
-EOF
-cat <<EOF >/run/systemd/system.attached/minimal-app0.service.d/20-portable.conf
-[Unit]
-Description=Minimal App 0
-EOF
-systemctl daemon-reload
-
-portablectl "${ARGS[@]}" attach --force --now --runtime /usr/share/minimal_0.raw minimal-app0
-
-portablectl is-attached --force minimal-app0
-portablectl inspect --force /usr/share/minimal_0.raw minimal-app0.service
-systemctl is-active minimal-app0.service
-systemctl is-active minimal-app0-foo.service
-systemctl is-active minimal-app0-bar.service && exit 1
-
-portablectl "${ARGS[@]}" reattach --force --now --runtime /usr/share/minimal_1.raw minimal-app0
-
-portablectl is-attached --force minimal-app0
-portablectl inspect --force /usr/share/minimal_0.raw minimal-app0.service
-systemctl is-active minimal-app0.service
-systemctl is-active minimal-app0-bar.service
-systemctl is-active minimal-app0-foo.service && exit 1
-
-portablectl list | grep -q -F "minimal_1"
-busctl tree org.freedesktop.portable1 --no-pager | grep -q -F '/org/freedesktop/portable1/image/minimal_5f1'
-
-portablectl detach --force --now --runtime /usr/share/minimal_1.raw minimal-app0
-
-portablectl list | grep -q -F "No images."
-busctl tree org.freedesktop.portable1 --no-pager | grep -q -F '/org/freedesktop/portable1/image/minimal_5f1' && exit 1
-
 # portablectl also works with directory paths rather than images
 
 unsquashfs -dest /tmp/minimal_0 /usr/share/minimal_0.raw
@@ -196,11 +151,7 @@ status="$(portablectl is-attached --extension /tmp/app10.raw /usr/share/minimal_
 
 portablectl inspect --force --cat --extension /tmp/app10.raw /usr/share/minimal_0.raw app0 | grep -q -F "Extension Release: /tmp/app10.raw"
 
-# Ensure that we can detach even when an image has been deleted already (stop the unit manually as
-# portablectl won't find it)
-rm -f /tmp/app10.raw
-systemctl stop app0.service
-portablectl detach --force --runtime --extension /tmp/app10.raw /usr/share/minimal_0.raw app0
+portablectl detach --force --now --runtime --extension /tmp/app10.raw /usr/share/minimal_0.raw app0
 
 # portablectl also works with directory paths rather than images
 
@@ -212,7 +163,7 @@ mount /usr/share/minimal_0.raw /tmp/rootdir
 # Fix up os-release to drop the valid PORTABLE_SERVICES field (because we are
 # bypassing the sysext logic in portabled here it will otherwise not see the
 # extensions additional valid prefix)
-grep -v "^PORTABLE_PREFIXES=" /tmp/rootdir/etc/os-release > /tmp/os-release-fix/etc/os-release
+grep -v "^PORTABLE_PREFIXES=" /tmp/rootdir/etc/os-release >/tmp/os-release-fix/etc/os-release
 
 mount -t overlay overlay -o lowerdir=/tmp/os-release-fix:/tmp/app1:/tmp/rootdir /tmp/overlay
 
@@ -225,28 +176,6 @@ systemctl is-active app1.service
 
 portablectl detach --now --runtime overlay app1
 
-# Ensure --force works also when symlinking
-mkdir -p /run/systemd/system.attached/app1.service.d
-cat <<EOF >/run/systemd/system.attached/app1.service
-[Unit]
-Description=App 1
-EOF
-cat <<EOF >/run/systemd/system.attached/app1.service.d/10-profile.conf
-[Unit]
-Description=App 1
-EOF
-cat <<EOF >/run/systemd/system.attached/app1.service.d/20-portable.conf
-[Unit]
-Description=App 1
-EOF
-systemctl daemon-reload
-
-portablectl "${ARGS[@]}" attach --force --copy=symlink --now --runtime /tmp/overlay app1
-
-systemctl is-active app1.service
-
-portablectl detach --now --runtime overlay app1
-
 umount /tmp/overlay
 
 portablectl "${ARGS[@]}" attach --copy=symlink --now --runtime --extension /tmp/app0 --extension /tmp/app1 /tmp/rootdir app0 app1
