@@ -3,11 +3,6 @@
 set -eux
 set -o pipefail
 
-if [[ "$(sysctl -ne kernel.apparmor_restrict_unprivileged_userns)" -eq 1 ]]; then
-    echo "Cannot create unprivileged user namespaces" >/skipped
-    exit 0
-fi
-
 systemd-analyze log-level debug
 
 runas() {
@@ -106,7 +101,7 @@ runas testuser systemd-run --wait --user --unit=test-network \
 
 (! runas testuser systemd-run --wait --user --unit=test-kernel-tunable \
     -p PrivateUsers=yes -p ProtectKernelTunables=yes \
-    sh -c "echo 0 > /proc/sys/user/max_user_namespaces")
+    sh -c "echo 0 >/proc/sys/user/max_user_namespaces")
 
 (! runas testuser systemd-run --wait --user --unit=test-kernel-mod \
     -p PrivateUsers=yes -p ProtectKernelModules=yes \
