@@ -55,12 +55,12 @@ test_timezone() {
     assert_in "Local time:" "$(timedatectl --no-pager)"
 
     echo 'change timezone'
-    assert_eq "$(timedatectl --no-pager set-timezone Europe/Kyiv 2>&1)" ""
-    assert_eq "$(readlink /etc/localtime | sed 's#^.*zoneinfo/##')" "Europe/Kyiv"
+    assert_eq "$(timedatectl --no-pager set-timezone Europe/Kiev 2>&1)" ""
+    assert_eq "$(readlink /etc/localtime | sed 's#^.*zoneinfo/##')" "Europe/Kiev"
     if [[ -f /etc/timezone ]]; then
-        assert_eq "$(cat /etc/timezone)" "Europe/Kyiv"
+        assert_eq "$(cat /etc/timezone)" "Europe/Kiev"
     fi
-    assert_in "Time zone: Europe/Kyiv \(EES*T, \+0[0-9]00\)" "$(timedatectl)"
+    assert_in "Time zone: Europe/Kiev \(EES*T, \+0[0-9]00\)" "$(timedatectl)"
 
     if [[ -n "$ORIG_TZ" ]]; then
         echo 'reset timezone to original'
@@ -107,7 +107,7 @@ LOCAL"
     check_adjtime_not_exist
 
     echo 'UTC set in adjtime file'
-    printf '0.0 0 0\n0\nUTC\n' > /etc/adjtime
+    printf '0.0 0 0\n0\nUTC\n' >/etc/adjtime
     timedatectl set-local-rtc 0
     assert_eq "$(cat /etc/adjtime)" "0.0 0 0
 0
@@ -118,7 +118,7 @@ UTC"
 LOCAL"
 
     echo 'non-zero values in adjtime file'
-    printf '0.1 123 0\n0\nLOCAL\n' > /etc/adjtime
+    printf '0.1 123 0\n0\nLOCAL\n' >/etc/adjtime
     timedatectl set-local-rtc 0
     assert_eq "$(cat /etc/adjtime)" "0.1 123 0
 0
@@ -129,7 +129,7 @@ UTC"
 LOCAL"
 
     echo 'fourth line adjtime file'
-    printf '0.0 0 0\n0\nLOCAL\nsomethingelse\n' > /etc/adjtime
+    printf '0.0 0 0\n0\nLOCAL\nsomethingelse\n' >/etc/adjtime
     timedatectl set-local-rtc 0
     assert_eq "$(cat /etc/adjtime)" "0.0 0 0
 0
@@ -142,60 +142,60 @@ LOCAL
 somethingelse"
 
     echo 'no final newline in adjtime file'
-    printf '0.0 0 0\n0\nUTC' > /etc/adjtime
+    printf '0.0 0 0\n0\nUTC' >/etc/adjtime
     timedatectl set-local-rtc 0
     check_adjtime_not_exist
-    printf '0.0 0 0\n0\nUTC' > /etc/adjtime
+    printf '0.0 0 0\n0\nUTC' >/etc/adjtime
     timedatectl set-local-rtc 1
     assert_eq "$(cat /etc/adjtime)" "0.0 0 0
 0
 LOCAL"
 
     echo 'only one line in adjtime file'
-    printf '0.0 0 0\n' > /etc/adjtime
+    printf '0.0 0 0\n' >/etc/adjtime
     timedatectl set-local-rtc 0
     check_adjtime_not_exist
-    printf '0.0 0 0\n' > /etc/adjtime
+    printf '0.0 0 0\n' >/etc/adjtime
     timedatectl set-local-rtc 1
     assert_eq "$(cat /etc/adjtime)" "0.0 0 0
 0
 LOCAL"
 
     echo 'only one line in adjtime file, no final newline'
-    printf '0.0 0 0' > /etc/adjtime
+    printf '0.0 0 0' >/etc/adjtime
     timedatectl set-local-rtc 0
     check_adjtime_not_exist
-    printf '0.0 0 0' > /etc/adjtime
+    printf '0.0 0 0' >/etc/adjtime
     timedatectl set-local-rtc 1
     assert_eq "$(cat /etc/adjtime)" "0.0 0 0
 0
 LOCAL"
 
     echo 'only two lines in adjtime file'
-    printf '0.0 0 0\n0\n' > /etc/adjtime
+    printf '0.0 0 0\n0\n' >/etc/adjtime
     timedatectl set-local-rtc 0
     check_adjtime_not_exist
-    printf '0.0 0 0\n0\n' > /etc/adjtime
+    printf '0.0 0 0\n0\n' >/etc/adjtime
     timedatectl set-local-rtc 1
     assert_eq "$(cat /etc/adjtime)" "0.0 0 0
 0
 LOCAL"
 
     echo 'only two lines in adjtime file, no final newline'
-    printf '0.0 0 0\n0' > /etc/adjtime
+    printf '0.0 0 0\n0' >/etc/adjtime
     timedatectl set-local-rtc 0
     check_adjtime_not_exist
-    printf '0.0 0 0\n0' > /etc/adjtime
+    printf '0.0 0 0\n0' >/etc/adjtime
     timedatectl set-local-rtc 1
     assert_eq "$(cat /etc/adjtime)" "0.0 0 0
 0
 LOCAL"
 
     echo 'unknown value in 3rd line of adjtime file'
-    printf '0.0 0 0\n0\nFOO\n' > /etc/adjtime
+    printf '0.0 0 0\n0\nFOO\n' >/etc/adjtime
     timedatectl set-local-rtc 0
     check_adjtime_not_exist
-    printf '0.0 0 0\n0\nFOO\n' > /etc/adjtime
+    printf '0.0 0 0\n0\nFOO\n' >/etc/adjtime
     timedatectl set-local-rtc 1
     assert_eq "$(cat /etc/adjtime)" "0.0 0 0
 0
@@ -203,14 +203,7 @@ LOCAL"
 }
 
 assert_ntp() {
-    local value="${1:?}"
-
-    for _ in {0..9}; do
-        [[ "$(busctl get-property org.freedesktop.timedate1 /org/freedesktop/timedate1 org.freedesktop.timedate1 NTP)" == "b $value" ]] && return 0
-        sleep .5
-    done
-
-    return 1
+    assert_eq "$(busctl get-property org.freedesktop.timedate1 /org/freedesktop/timedate1 org.freedesktop.timedate1 NTP)" "b $1"
 }
 
 start_mon() {
